#include <stdio.h>			/* stdio */
#include <stdarg.h>			/* stdarg */
#include <string.h>			/* string function */
#include <malloc.h>			/* memory allocation function */
#include <stdlib.h>			/* standart library function */
#include <unistd.h>			/* unix standart function */
#include <time.h>			/* time function */
#include <errno.h>			/* errno codes */
#include <fcntl.h>			/* fcntl */
#include <termios.h>			/* termios */
#include <pwd.h>			/* pwd */
#include <sys/wait.h>			/* wait functions */
#include <sys/types.h>			/* system types */
#include <dirent.h>			/* _POSIX_NAME_MAX, direntry functions */
#include "ucrp.h"			/* ucrp function */
/* to disable all debug code undefine __UCRP_DEBUG__ macro */
#define __UCRP_DEBUG__			/* debug code enable */
//#define __UCRP_BTS_TABLE_DEBUG__	/* bts translate table debug macro */
#define __UCRP_SIG_DEBUG__		/* signal debug macro */
#ifndef __UCRP_DEBUG__	 		/* debug macro undefined check */
#undef  __UCRP_SIG_DEBUG__		/* signal debug macro undefine */
#undef  __UCRP_BTS_TABLE_DEBUG__	/* bts translate table debug macro undefine */
#endif						/* __UCRP_DEBUG__ */
#define UCRP_USAGE \
"UCRP %d.%d (c) 2006-2011 Dmitriy I.Cherkashin, www.ucrouter.ru\r\n"  \
"email: dch@ucrouter.ru , divch@users.sourceforge.net , dch@ucrouter.com\r\n"  \
"phone: +7-916-315-1943; ICQ: 474-363-900;\r\n"  \
"This is not public software, you can't use this program without\r\n"  \
"license agreement, which available via option --license\r\n"  \
"with noted above author. To see license agreement use:\r\n"  \
"ucrp --license\r\n"  \
"  or\r\n"  \
"ucrp --license-ru\r\n"  \
"  for russian license translation.\r\n"
#define UCRP_LICENSE \
"UCRP %d.%d (c) 2006-2011 Dmitriy I.Cherkashin, www.ucrouter.ru\r\n"  \
"email: dch@ucrouter.ru , divch@users.sourceforge.net , dch@ucrouter.com\r\n"  \
"phone: +7-916-315-1943; ICQ: 474-363-900;\r\n"  \
"This program is free software with source codes\r\n"  \
"downloadable via internation communicational net.\r\n"  \
"If you have source codes of this software original or modified,\r\n"  \
"you must provide ability to free receive original source codes\r\n"  \
"of this software to another users even in internation communicational\r\n"  \
"net absense without any unusual additional payment exept\r\n" \
"downloading files from internation communicational net cost or\r\n" \
"mail cost.\r\n" \
"Program ucrp source files can contain don't public extension,\r\n" \
"which include extension code breaked with\r\n" \
"#ifdef __UCRP_PRIVATE__ , #endif construction:\r\n" \
"#ifdef __UCRP_PRIVATE__\r\n" \
"<don't public source code>\r\n" \
"#endif\r\n" \
"If you have ucrp program source code with text string __UCRP_PRIVATE__\r\n" \
"inside files you can't use and redistribute this files without\r\n" \
"talked or writed agreement with noted above author.\r\n" \
"You can remove ucrp extension code and use, redistribute this files\r\n" \
"as free software under this license.\r\n"
#define UCRP_LICENSE_RU \
"UCRP %d.%d (c) 2006-2011  ., www.ucrouter.ru\r\n" \
" : dch@ucrouter.ru,divch@users.sourceforge.net,dch@ucrouter.com\r\n"  \
": +7-916-315-1943; ICQ: 474-363-900;\r\n" \
"     \r\n" \
"      \r\n" \
" .\r\n" \
"       \r\n" \
"  ,     \r\n" \
"     \r\n" \
"     \r\n" \
"      \r\n" \
"  ,     \r\n" \
"    \r\n" \
"     .\r\n" \
"   ucrp     ,\r\n" \
"      \r\n" \
"#ifdef __UCRP_PRIVATE__ , #endif :\r\n" \
"#ifdef __UCRP_PRIVATE__\r\n" \
"<  >\r\n" \
"#endif\r\n" \
"      ucrp   \r\n" \
"__UCRP_PRIVATE__  ,      \r\n" \
"         .\r\n" \
"     ucrp      \r\n" \
"      .\r\n"
#define UCRP_BS   	     0x7f	/* back space key */
#define UCRP_DEL 	 0x335b1b	/* delete key */
u_int8_t ucrp_version_hi = 1;		/* program version hight digit */
u_int8_t ucrp_version_lo = 30;		/* program version lo digit */
/* command type */
#define UCRP_ENCRYPT 	 0x0001		/* encrypt file command */
#define UCRP_DECRYPT 	 0x0002		/* decrypt file command */
#define UCRP_INFO 	 0x0004		/* encrypted file info */
#define UCRP_CLEAR 	 0x0008		/* bash & mc history clear & clear free space */
/* exit command */
#define UCRP_LIC	 0x0001 	/* license agreement print and exit */
#define UCRP_LICRU	 0x0002 	/* license agreement ru print and exit */
#define UCRP_HELP	 0x0003 	/* help print and exit */
#define UCRP_VERSION	 0x0004		/* version print and exit */
#define UCRP_TODO	 0x0005		/* todo note print and exit */
#define UCRP_CHANGELOG	 0x0006		/* change log print and exit */
#define UCRP_MAN	 0x0007		/* man page print and exit */
#define UCRP_PASSWD_SIZE 	1024	/* passwd size */
#define UCRP_FILE_NAME_SIZE 	1024	/* file name size */
char ucrp_passwd[UCRP_PASSWD_SIZE] = ""; /* password */
char ucrp_pfile[UCRP_FILE_NAME_SIZE] = ""; /* password file */
u_int8_t ucrp_hide_passwd = 1;		/* hide password, 1 - *, 2 - $, 3 - d */
u_int8_t ucrp_quiet = 0;		/* quiet mode */
u_int8_t ucrp_clear_mode = 0;		/* clear disk space mode */
#define UCRP_CLEAR_INC 	0 		/* increment filling */
#define UCRP_CLEAR_CLR 	1 		/* write 0 */
#define UCRP_CLEAR_SET 	2 		/* write FF */
u_int32_t ucrp_cmd = 0;			/* ucrp command */
FILE * ucrp_finp = 0;			/* file description input pointer */
FILE * ucrp_fout = 0;			/* file description output pointer */
u_int16_t ucrp_path_size = 256;		/* ucrp path size */
char ucrp_inpfile[UCRP_FILE_NAME_SIZE] = ""; /* input file name */
char ucrp_outfile[UCRP_FILE_NAME_SIZE] = ""; /* file output name */
char ucrp_curdir[UCRP_FILE_NAME_SIZE] = ""; /* current/work directory */
char ucrp_prvdir[UCRP_FILE_NAME_SIZE] = ""; /* previous current directory */
#define UCRP_COMP_LEN 8192		/* compatibility buffer length */
u_char ucrp_comp_buf[UCRP_COMP_LEN] = ""; /* compatibility buffer */
#define __UCRP_XOR_ENABLE__		/* enable XOR algoritm */
#define __UCRP_ADD_ENABLE__		/* enable ADD algoritm */
#define __UCRP_ROT_ENABLE__		/* enable ROT algoritm */
#define __UCRP_TAB_ENABLE__		/* enable TAB algoritm */
#define __UCRP_ECC_ENABLE__		/* enable ECC algoritm */
#define __UCRP_BTS_ENABLE__		/* enable BTS algoritm */
#define __UCRP_XRE_ENABLE__		/* enable XRE algoritm */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
#define UCRP_XRE_100    71		/* xor enhanced v.1.10 */
#define UCRP_XRE_118    64		/* xor enhanced v.1.18 */
#endif 						/* __UCRP_XRE_ENABLE__ */
#ifdef  __UCRP_XOR_ENABLE__		/* XOR enable macro check */
#define UCRP_XOR_100   	64		/* xor compatibility v.1.10 bit number */
#define UCRP_XOR_118   	71		/* xor compatibility v.1.18 bit number */
#endif 						/* __UCRP_XOR_ENABLE__ */
#ifdef  __UCRP_ADD_ENABLE__		/* ADD enable macro check */
#define UCRP_ADD_100   	65		/* add compatibility v.1.10 bit number */
#define UCRP_ADD_118   	65		/* add compatibility v.1.18 bit number */
#endif 						/* __UCRP_ADD_ENABLE__ */
#ifdef  __UCRP_ROT_ENABLE__		/* ROT enable macro check */
#define UCRP_ROT_100   	66		/* byte rotate compatibility v.1.10 bit number */
#define UCRP_ROT_118   	66		/* byte rotate compatibility v.1.10 bit number */
#endif 						/* __UCRP_ROT_ENABLE__ */
#ifdef  __UCRP_BTS_ENABLE__		/* enable BTS algoritm macro check */
#define UCRP_BTS_100   	69		/* bit swapping algoritm v. 1.10 */
#define UCRP_BTS_118   	67		/* bit swapping algoritm v. 1.18 */
#endif  					/* __UCRP_BTS_ENABLE__ */
#ifdef  __UCRP_TAB_ENABLE__		/* TAB enable macro check */
#define UCRP_TAB_100   	67		/* byte table compatibility v.1.10 bit number */
#define UCRP_TAB_118   	68		/* byte table compatibility v.1.18 bit number */
#endif 						/* __UCRP_TAB_ENABLE__ */
#ifdef  __UCRP_ECC_ENABLE__		/* ECC enable macro check */
#define UCRP_ECC_100	68		/* ecc compatibility v.1.10 bit number */
#define UCRP_ECC_118   	69		/* ecc compatibility v.1.18 bit number */
#endif						/* __UCRP_ECC_ENABLE__ */
u_int8_t ucrp_xor_comp = UCRP_XOR_118;  /* XOR compatibility bit number */
u_int8_t ucrp_xre_comp = UCRP_XRE_118;  /* XRE compatibility bit number */
u_int8_t ucrp_add_comp = UCRP_ADD_118;	/* ADD compatibility bit number */
u_int8_t ucrp_rot_comp = UCRP_ROT_118;  /* ROT compatibility bit number */
u_int8_t ucrp_bts_comp = UCRP_BTS_118;  /* BTS compatibility bit number */
u_int8_t ucrp_tab_comp = UCRP_TAB_118;  /* TAB compatibility bit number */
u_int8_t ucrp_ecc_comp = UCRP_ECC_118;	/* ECC compatibility bit number */
#define UCRP_SRC_BUF        72		/* source buffer size bit number */
#define UCRP_SRC_BUF_LEN    32 		/* source buffer size field length */
#define UCRP_DST_BUF	   104		/* destination buffer size */
#define UCRP_DST_BUF_LEN    32		/* destination buffer size field length */
#define UCRP_KEY_LEN      8192		/* key buffer length */
u_char ucrp_key_buf[UCRP_KEY_LEN] = "";	/* key buffer */
#define UCRP_XOR_BASE		   0	/* xor parameter key buffer bit number */
#define UCRP_XOR_LEN               8	/* xor parameter field length */
#define UCRP_ADD_BASE		   8	/* add parameter key buffer bit number */
#define UCRP_ADD_LEN               8	/* add parameter field length */
#define UCRP_ROT_BASE             16	/* rot parameter key buffer bit number */
#define UCRP_ROT_LEN               8	/* rot parameter field length */
#define UCRP_TAB_BASE_100         24	/* tab parameter key buffer bit number */
#define UCRP_TAB_LEN		   8	/* tab parameter field length */
#define UCRP_BTS_BASE_100         32	/* bts parameter key buffer bit number */
#define UCRP_BTS_LEN               8	/* bts parameter field length */
#define UCRP_BTS_TABLE_BASE_100   40	/* bts table bit number */
#define UCRP_BTS_TABLE_LEN        64	/* bts table length */
#define UCRP_XRL_BASE            104	/* xor mask change law */
#define UCRP_XRL_LEN               5	/* xor mask change law length */
#define UCRP_XRL_PRM_BASE        109	/* xor mask change law parameter */
#define UCRP_XRL_PRM_LEN           3	/* xor mask change law parameter length */
#define UCRP_TABLE_BASE_100     2048	/* table bit number */
#define UCRP_TABLE_LEN          1024	/* table length */
#define UCRP_BTS_BASE_118         24    /* bts parameter key buffer bit number v. 1.18 */
#define UCRP_TAB_BASE_118         32	/* tab parameter key buffer bit number v. 1.18 */
#define UCRP_BTS_TABLE_BASE_118 2048	/* bts table bit number v. 1.18 */
#define UCRP_TABLE_BASE_118     2112	/* 2048 + 64 table bit number v. 1.18 */
u_int16_t ucrp_bts_base = UCRP_BTS_BASE_118; /* bit swap parameter bit number, default v. 1.18 */
u_int16_t ucrp_tab_base = UCRP_TAB_BASE_118; /* table parameter bit number, default v. 1.18 */
u_int16_t ucrp_bts_table_base = UCRP_BTS_TABLE_BASE_118; /* bts table bit number */
u_int16_t ucrp_table_base = UCRP_TABLE_BASE_118; /* bts table bit number */
/* xor mask change law */
#define UCRP_LAW_NULL        0		/* law null */
#define UCRP_LAW_AD3         1		/* law add */
#define UCRP_LAW_ROT         2		/* law rot */
#define UCRP_LAW_TAB_100     3		/* tab v. 1.00 */
#define UCRP_LAW_BTS_100     4		/* bts v. 1.00 */
#define UCRP_LAW_BTS_118     3		/* bts v. 1.18 */
#define UCRP_LAW_TAB_118     3		/* tab v. 1.18 */
#define UCRP_LAW_TAB_128     4		/* tab v. 1.28 */
u_int8_t ucrp_law_bts = UCRP_LAW_BTS_118; /* law bts version 1.18 default */
u_int8_t ucrp_law_tab = UCRP_LAW_TAB_128; /* law tab version 1.28 default */
#define UCRP_LAW_MAX	     4		/* low maximal value */
#define UCRP_SRC_SIZE  0x10000		/* source buffer size */
#define UCRP_DST_SIZE (UCRP_SRC_SIZE + ((UCRP_SRC_SIZE / 256) * 3)) /* destination buffer size */
u_int32_t ucrp_src_buf_size = UCRP_SRC_SIZE; /* source buffer size */
u_int32_t ucrp_dst_buf_size = UCRP_DST_SIZE; /* destination buffer size */
u_char * ucrp_src = 0;			/* source buffer */
u_char * ucrp_dst = 0;			/* destination buffer */
#ifdef __UCRP_XOR_ENABLE__		/* XOR enable macro check */
#define UCRP_XOR_ENABLE  0x00000001	/* xor enable */
#endif						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* ADD enable macro check */
#define UCRP_ADD_ENABLE  0x00000002	/* add enable */
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* rotate enable macro check */
#define UCRP_ROT_ENABLE  0x00000004	/* byte rotate enable */
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
#define UCRP_BTS_ENABLE  0x00000008	/* bit swaping enable */
#endif						/* __UCRP_BTS_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
#define UCRP_TAB_ENABLE  0x00000010	/* table enable */
#endif 						/* __UCRP_TAB_ENABLE__ */
#ifdef __UCRP_ECC_ENABLE__		/* ECC enable macro check */
#define UCRP_ECC_ENABLE  0x00000020	/* ecc enable */
#endif						/* __UCRP_ECC_ENABLE__ */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
#define UCRP_XRE_ENABLE  0x00000080	/* xor enhanced enable */
#endif						/* __UCRP_XRE_ENABLE__ */
u_int32_t ucrp_enable = (		/* enable mask */
 0xFFFFFFFF ^ 				/* enable all */
 (UCRP_XOR_ENABLE)			/* xor disable */
);
#define UMSG_STRING		"%s"	/* string format */
#define UMSG_DECIMAL		"%d"	/* decimal format */
#define UMSG_EMPTY		""	/* empty string */
#define UMSG_CRNL		"\r\n"	/* <CR><NL> */
#define	UMSG_CHARACTER		"%c"	/* character format */
#define	UMSG_02X		"%02x"	/* hex 2 character width format */
#define UMSG_PRINT_OUTBUF_ERROR	"Out of print function buffer error, bytes processed {%d}\r\n"
#define UMSG_READBYTE 		"r+b"	/* read in byte mode */
#define UMSG_WRITEBYTE 		"w+b"	/* write in byte mode */
#define UMSG_OPEN_ERROR		"File {%s} open error\r\n" /* file open error format */
#define UMSG_BACKSPACE 		"\b"	/* back space */
#define UMSG_SPACE 		" "	/* space */
#define UMSG_STDIN_ERROR 	"\r\nBad stdin descriptor\r\n"
#define	UMSG_SELECT_ERROR	"Select function error\r\n" /* select error message */
#define UMSG_TCGETATTR_STDIN_ERROR "\r\ntcgetattr stdin error : {%d}\r\n"
#define UMSG_TCSETATTR_STDIN_ERROR "\r\ntcsetattr stdin error : {%d}\r\n"
#define UMSG_STDIN_READ_ERROR	"\r\nstandart input read error : {%d}\r\n"
#define UMSG_GET_FCNTL_ERROR 	"File {%s} read mode error\r\n"
#define UMSG_O_NONBLOCK_ERROR 	"Set non block mode error\r\n"
#define UMSG_STDIN_NONBLOCK_ERROR "Standart input, set non block mode error\r\n"
#define UMSG_STDIN_RESTORE_ERROR "Standart input, restore setting error\r\n"
#define UMSG_CANCELED 		"{Canceled}"
#define UMSG_RANDOM_FILE_NAME	"/dev/random"
#define UMSG_URAND_FILE_NAME	"/dev/urandom"
#define UMSG_INFO_VERSION	"version {%s}\r\n" /* --info version format */
#define UMSG_INFO_XOR		"xor {%s} {%d}\r\n" /* --info xor format */
#define UMSG_INFO_XRE		"xre {%s} {%d}\r\n" /* --info xre format */
#define UMSG_INFO_ON		"on"	/* on string */
#define UMSG_INFO_OFF		"off"	/* off string */
#define UMSG_LAW_NULL		"null"	/* law null */
#define UMSG_LAW_AD3		"add"	/* law add */
#define UMSG_LAW_ROT		"rot"	/* law rot */
#define UMSG_LAW_TAB		"tab"	/* law tab string */
#define UMSG_LAW_BTS		"bts"	/* law bts string */
#define UMSG_LAW_UNKN		"unkn"	/* law unknown string */
#define UMSG_HASH 		"hash:"	/* hash string */
#define UMSG_BTTB		"bttb:"	/* bit table */
#define UMSG_INFO_XRL		"xrl {%s} {%d}\r\n" /* xor law string */
#define UMSG_INFO_XRL_PRM   	"xrpr {%d}\r\n" /* xor law parametes format  */
#define UMSG_INFO_ADD		"add {%s} {%d}\r\n" /* info add format */
#define UMSG_INFO_ROT 		"rot {%s} {%d}\r\n" /* info rot format */
#define UMSG_INFO_TAB 		"tab {%s} {%d}\r\n" /* info tab format */
#define UMSG_INFO_ECC 		"ecc {%s}\r\n"	/* info ecc format */
#define UMSG_INFO_BTS 		"bts {%s} {%d}\r\n" /* info bts format */
#define UMSG_INFO_SRC_BLEN 	"source block length {%d} {0x%x}\r\n"
#define UMSG_INFO_DST_BLEN	"destination block length {%d} {0x%x}\r\n"
#define UMSG_INFO_TRTABLE 	"translation table:\r\n" /* translate table format */
#define UMSG_XOR_ERR  		"xor encrypt/decrypt error\r\n" /* xor error message */
#ifdef __UCRP_TAB_DEBUG__		/* table encrypt/decrypt debug macro check */
#define UMSG_TAB_EN 		"tab encrypt {%d}\r\n"
#define UMSG_TAB_EN_END		"tab end\r\n"
#define UMSG_TAB_DE 		"tab decrypt {%d}\r\n"
#define UMSG_TAB_DE_END		"tab end\r\n"
#endif 						/* __UCRP_TAB_DEBUG__ */
#define UMSG_ALLOC_ERROR	"Buffer allocation error, size {%d}, hex {%x}\r\n"
#define UMSG_BASH_HISTORY	"/%s%s.bash_history" /* bash history format */
#define UMSG_MC_HISTORY		"/%s%s.mc/history" /* mc history format */
#define UMSG_FILE_EXIST 	"f"	/* ucrp_file exist mode */
#define UMSG_FILE_UNLINK	"Unlink {%s}\r\n" /* unlink format */
#define UMSG_FILE_ABSENT	"Absent {%s}\r\n" /* file absent format */
#define UMSG_UCRP_CLEAR		"ucrp_clear.%03d" /* file clear format */
#define UMSG_ROOT 		"root"	/* default user name */
#define UMSG_SLASH 		"/"	/* slash */
#define UMSG_BREKET_START	"{"	/* breket open */
#define UMSG_BREKET_END		"}"	/* breket close */
#define UMSG_BREKET_POS		"{%d}"	/* file position */
#define UMSG_17S		"%17s"	/* 17s format */
#define UMSG_SIGHUP    		"1)SIGHUP - Hangup"
#define UMSG_SIGINT    		"2)SIGINT - Interrupt"
#define UMSG_SIGQUIT   		"3)SIGQUIT - Quit"
#define UMSG_SIGILL    		"4)SIGILL - Illegal instruction"
#define UMSG_SIGTRAP   		"5)SIGTRAP - trap"
#define UMSG_SIGABRT   		"6)SIGABRT - abort"
#define UMSG_SIGBUS    		"7)SIGBUS - buss error"
#define UMSG_SIGFPE    		"8)SIGFPE - floating point exeption"
#define UMSG_SIGKILL   		"9)SIGKILL - kill"
#define UMSG_SIGUSR1   		"10)SIGUSR1 - user signal 1"
#define UMSG_SIGSEGV   		"11)SIGSEGV - segmentation violation"
#define UMSG_SIGUSR2   		"12)SIGUSR2 - user signal 2"
#define UMSG_SIGPIPE   		"13)SIGPIPE - pipe"
#define UMSG_SIGALRM   		"14)SIGALRM - alarm"
#define UMSG_SIGTERM   		"15)SIGTERM - term"
#define UMSG_SIGSTKFLT 		"16)SIGSTKFLT - stack fault"
#define UMSG_SIGCHLD   		"17)SIGCHLD - children"
#define UMSG_SIGCONT   		"18)SIGCONT - continue"
#define UMSG_SIGSTOP   		"19)SIGSTOP - stop"
#define UMSG_SIGTSTP   		"20)SIGTSTP - terminal stop, terminal command"
#define UMSG_SIGTTIN   		"21)SIGTTIN - terminal read"
#define UMSG_SIGTTOU   		"22)SIGTTOU - terminal write"
#define UMSG_SIGURG    		"23)SIGURG - urg"
#define UMSG_SIGXCPU   		"24)SIGXCPU - cpu limit exceeded"
#define UMSG_SIGXFSZ   		"25)SIGXFSZ - file size exceeded"
#define UMSG_SIGVTALRM 		"26)SIGVTALRM - virtual alarm clock"
#define UMSG_SIGPROF   		"27)SIGPROF - profilling alarm clock"
#define UMSG_SIGWINCH  		"28)SIGWINCH - window size changed"
#define UMSG_SIGIO     		"29)SIGIO - I/O"
#define UMSG_SIGPWR    		"30)SIGPWR - power failure restart"
#define UMSG_SIGSYS    		"31)SIGSYS - bad system call"
#define UMSG_BAD_SIGNAL_CALL	"Bad signal call sinfo=0x%x , sptr=0x%x\r\n"
#define UMSG_SIG_INFO \
"Signal num {%d}, errno {%d}, code {%d}\r\n" \
"pid {%d}, uid {%d}, status {%d}, utime {%d}, stime {%d}\r\n" \
"value {%d}, int {%d}, ptr {%d}, addr {%d}, band {%d}, fd {%d}\r\n" \
"sptr {%x}\r\n"
#define UMSG_VAR_LOG		"/var/log/%s"	/* format */
#define UMSG_SIGNAL_DUMP	"ucrpsdump" 	/* signal dump file name */
#define UMSG_SIG_EMPTY_SET_ERROR  "Clear nonblocked signals mask error\r\n"
#define UMSG_SIG_ADD_SET_ERROR	"Set signals mask {%x} error\r\n"
#define UMSG_SIG_ACTION_ERROR	"Set signal {%d} handler error\r\n"
#define UMSG_OPT_ENCRYPT 	"--encrypt"	/* encrypt option */
#define UMSG_OPT_DECRYPT	"--decrypt"	/* decrypt option */
#define UMSG_OPT_CLEAR		"--clear"	/* clear command */
#define UMSG_OPT_QUIET		"--quiet"	/* quiet string */
#define UMSG_OPT_CURRENT_DIR	"--current-dir"	/* current-dir command */
#define UMSG_OPT_CHANGE_DIR	"--change-dir"	/* change-dir command */
#define UMSG_OPT_DISABLE	"--disable"	/* disable option */
#define UMSG_OPT_ENABLE		"--enable"	/* enable option */
#define UMSG_OPT_BLOCK_SIZE	"--block-size"	/* option block size */
#define UMSG_OPT_PASSWD		"--passwd"
#define UMSG_OPT_INFO		"--info"
#define UMSG_OPT_HIDE_PASSWD	"--hide-passwd"	/* hide password option */
#define UMSG_OPT_SHOW_PASSWD	"--show-passwd" /* show password option */
#define UMSG_OPT_HELP		"--help"	/* help option */
#define UMSG_OPT_LICENSE	"--license"	/* license option */
#define UMSG_OPT_LICENSE_RU	"--license-ru"	/* license-ru option */
#define UMSG_OPT_VERSION 	"--version"	/* version command */
#define UMSG_OPT_TODO 		"--todo"	/* todo option */
#define UMSG_OPT_CHANGELOG	"--changelog"	/* change log command */
#define UMSG_OPT_MAN 		"--man"		/* man page option */
#define UMSG_ARG_XOR 		"xor"	/* xor argument */
#define UMSG_ARG_ADD 		"add"	/* add argument */
#define UMSG_ARG_ROT 		"rot"	/* rot argument */
#define UMSG_ARG_TAB 		"tab"	/* tab argument */
#define UMSG_ARG_ECC 		"ecc"	/* ecc argument */
#define UMSG_ARG_BTS 		"bts"	/* bts argument */
#define UMSG_ARG_BEV 		"bev"	/* bev argument */
#define UMSG_ARG_XRE 		"xre"	/* xre argument */
#define UMSG_ARG_STD 		"std"	/* argument standart */ 
#define UMSG_ARG_CLEAR_INC	"inc"	/* clear increment filling */
#define UMSG_ARG_CLEAR_CLR	"0"	/* clear bits */
#define UMSG_ARG_CLEAR_SET	"1"	/* set bits */
#define UMSG_ARG_HIDE_PASSWD_STD "*"	/* standard */
#define UMSG_ARG_HIDE_PASSWD_DOL "$"	/* $ */
#define UMSG_ARG_HIDE_PASSWD_D  "d"	/* d */
#define UMSG_OPTION 		"Option"
#define UMSG_REQUERY_ARG	"requery argument"
#define UMSG_CURRENT_DIR	"Current dir:{%s}\r\n"
#define UMSG_CHDIR_ERROR	"Change directory {%s} error\r\n"
#define UMSG_INP_FILE		"Input file:{%s}\r\n"
#define UMSG_OUT_FILE		"Output file:{%s}\r\n"
#define UMSG_SRC_BLOCK_SIZE	"Source block size: {%d}, hex {%x}\r\n"
#define UMSG_DST_BLOCK_SIZE	"Destination block size: {%d}, hex {%x}\r\n"
#define UMSG_COMMAND		"Command:"
#define UMSG_CMD_ENCRYPT	"{encrypt}"	/* encrypt command string */
#define UMSG_CMD_DECRYPT	"{decrypt}"	/* decrypt command string */
#define UMSG_CMD_INFO		"{info}"	/* info command string */
#define UMSG_CMD_CLEAR		"{clear}"	/* clear command string */
#define UMSG_PASSWD		"NumLock, CapsLok sensitive\r\npasswd:{%s}\r\n"
#define UMSG_PASSWD_REQUEST	"request"	/* password request */
#define UMSG_ECMD_ERROR		"Internal error, unknown end command code {%d}\r\n"
#define UMSG_PASSWD_FILE	"Password file:{%s}\r\n"
#define UMSG_CLOSE_ERROR	"File {%s} close error\r\n"
#define UMSG_FILE_PASSWD_CATTED "File {%s} to passwd catted\r\n"
#define UMSG_USR_DBIN		"/usr/dbin/%s"
#define UMSG_DBIN		"/dbin/%s"
#define UMSG_INP_FILE_OPEN_ERR	"Input file {%s} open error\r\n"
#define UMSG_OUT_FILE_OPEN_ERR	"Output file {%s} open error\r\n"
#define UMSG_FILE_NOT_VALID	"Not valid input file {%s}\r\n"
#define UMSG_DECIMAL_STRING	"\r\n{%d} {%s} "
#define UMSG_DOT		"."
#define UMSG_OK			" Ok"
#define UMSG_FAIL		" Fail"
#define UMSG_SKIPPED		" Skipped"
#define UMSG_EN_DECRYPT_ERROR 	"\r\nEncryption/decryption error\r\n"
#define UMSG_WRITE_ERROR	"File {%s} write error\r\n"
#define UMSG_SRC_DST_BUF_ERROR	"Source or destination buffer allocation error\r\n"
#define	UMSG_VERSION 		"\r\nVersion : {%d.%d}\r\n" /* version format */
#define	UMSG_CHANGELOG \
    "\r\nChangeLog\r\n"
extern volatile u_int8_t kill_link;	/* > 1, exit from select loop */
#if 1
int16_t ucrp_error(const char *frmt, ...) /* error message print */
{
 if(ucrp_quiet == 1) {			/* quiet mode check */
    return(1);
 }
 u_int8_t buf[512];			/* message buffer */
 int16_t rvl;				/* return code */
 rvl = 1;				/* return code start value */
 va_list valt;				/* argument list */
 va_start(valt, frmt);			/* argument list init */
 int32_t cnt;				/* bytes writed */
 cnt = vsnprintf(			/* buffer form */
    (char *)buf,			/* buffer */
    sizeof(buf),			/* buffer size */
    frmt,				/* format */
    valt				/* argument list */
 );
 va_end(valt);				/* argument list end */
 if(cnt < 0) {				/* buffer form error check */
    rvl = 0;				/* return code clear */
 } else {
    if(cnt >= (int)sizeof(buf)) {	/* out of buffer check */
	buf[sizeof(buf)-1] = '\0';	/* terminator character write */
	rvl = 0;			/* return code clear */
    }
    if(fprintf(				/* print error message to stderr */
	stderr,				/* stderr */
	UMSG_STRING,			/* string format */
	buf				/* message */
    ) < 0) {				/* error check */
	rvl = 0;			/* return code clear */
    }
    if(fflush(stderr) < 0) {		/* fflush return code check */
	rvl = 0;			/* return code clear */
    }
 }
 return(rvl);				/* return code out */
}
#else
#define ucrp_error(...) 
#endif 						/* 1 */
int16_t ucrp_debug(const char *frmt, ...) /* debug message print */
{
 if(ucrp_quiet == 1) {			/* quiet mode check */
    return(1);
 }
 u_int8_t buf[1024];			/* message buffer */
 int16_t rvl;				/* return code */
 rvl = 1;				/* return code start */
 va_list valt;				/* argument list */
 va_start(valt, frmt);			/* argument list init */
 int32_t cnt;				/* bytes writed */
 cnt = vsnprintf(			/* buffer form */
    (char *)buf,			/* buffer */
    sizeof(buf),			/* buffer size */
    frmt,				/* format */
    valt				/* argument list */
 );
 va_end(valt);				/* argument list end */
 if(cnt < 0) { 				/* function return code check */
    rvl = 0;				/* return code clear */
 } else {
    if(cnt >= (int)sizeof(buf)) {	/* out of buffer check */
	buf[sizeof(buf)-1] = '\0';	/* terminator character write */
	rvl = 0;			/* return code clear */
    }
    if(fprintf(				/* debug message print to stderr */
        stderr, 			/* stderr */
        UMSG_STRING, 			/* string format */
        buf				/* message buffer */
    ) < 0) {				/* print error check */
        rvl = 0;			/* return code clear */
    }
    if(fflush(stderr) < 0) {		/* stderr flush */
        rvl = 0;			/* return code clear */
    }
 }
 return(rvl);				/* function return */
}
#if 1
int16_t ucrp_printf(const char *frmt, ... ) 
{
 static char buf[4096];			/* print buffer */
 if(ucrp_quiet == 1) {			/* quiet mode check */
    return(1);
 }
 int16_t rvl;				/* return code */
 rvl = 1;				/* return code start */
 va_list valt;				/* argument list */
 va_start(valt, frmt);			/* argument list init */
 int32_t cnt;				/* bytes writed */
 cnt = vsnprintf(			/* buffer form */
    buf, 				/* message buffer */
    sizeof(buf), 			/* buffer size */
    frmt, 				/* format */
    valt				/* argument list */
 );
 va_end(valt);				/* argument end */
 if(cnt < 0) {				/* print error check */
    return(0);				/* return errror */	
 }
 if(cnt >= (int)sizeof(buf)) {		/* out of buffer check */
    fprintf(				/* stderr print */
	stderr, 			/* stderr */
	UMSG_PRINT_OUTBUF_ERROR,	/* message format */
	cnt				/* bytes writed */
    );
    fflush(stderr);			/* stderr flush */
    rvl = 0;				/* return code clear */
    buf[sizeof(buf)-1] = '\0';		/* terminator character write */
 }
    if(fprintf(				/* message print to standart output */
	stdout,				/* stdout */
	UMSG_STRING,			/* string format */
	buf				/* message buffer */
    ) < 0) {				/* print error check */
	rvl = 0;			/* return code clear */
    }
    if(fflush(stdout) < 0) {		/* stdout flush return code check */
    	rvl = 0;			
    }
 return(rvl);
}
#else /* 0 */
#define ucrp_printf printf
#endif							
/* file existing & mode test
qual f - test regular file, l - test link, d - test directory,
r - test read, w - test write, x - test executable,
o - other, g- group, u - user
*/
int16_t ucrp_file(char *qual, char * frmt, ...)
{
 u_int8_t * buf;			/* file name buffer */
 buf = (u_int8_t *) malloc(		/* file name buffer allocate */
    ucrp_path_size			/* allocation buffer size */
 );
 if(buf == 0) {				/* buffer allocation check */
    return(0);				/* return error */
 }
 va_list valt;				/* variable argument list */
 va_start(valt, frmt);			/* variable argument list init */
 int32_t cnt;				/* bytes writed/processed */
 cnt = vsnprintf(			/* file name form */
    (char *)buf, 			/* file name buffer */
    ucrp_path_size, 			/* file name buffer size */
    frmt, 				/* format */
    valt				/* argument list */
 );
 va_end(valt);				/* argument list deinit */
 if(
    (cnt < 0) || 			/* print error check */
    (cnt >= ucrp_path_size)		/* out of buffer check */
 ) {
    free(buf);				/* buffer free */
    buf = 0;				/* buffer pointer clear */
    return(0);				/* file test error */
 }
 if(qual == 0) {			/* qual null pointer */
    int16_t rvl;			/* return code */
    if(access((char *)buf,F_OK) < 0) {	/* file presence check */
        rvl = 0;			/* file not exist */
    } else {
	rvl = 1;			/* file exist */
    }
    free(buf);				/* buffer allocated free */
    buf = 0;				/* buffer pointer clear */
    return(rvl);			/* file existance return */
 }
 if(
    (*(qual+0) == 'f') && 		/* file existance */
    (*(qual+1) == '\0')			/* only */
 ) {
    int16_t rvl;			/* return code */
    if(access((char *)buf,F_OK) < 0) {	/* file presence check */
        rvl = 0;			/* file not exist */
    } else {
	rvl = 1;			/* file exist */
    }
    free(buf);				/* buffer allocated free */
    buf = 0;				/* buffer pointer clear */
    return(rvl);			/* file existance return */
 }
 u_int8_t freg;				/* 0 - regular file, 1 - dir, 2 - link */
 u_int8_t prot;				/* file protection */
 mode_t mode;				/* file mode */
 freg = 0;				/* regular file */
 mode = 0;				/* file mode */
 prot = 0;				/* file protection */
 while(*(qual) != '\0') {		/* qual loop */
    switch(*qual) {			/* qual character switch */
	case 'f' : freg = 0; break;	/* regulay file */
	case 'd' : freg = 1; break;	/* dirrectory */
	case 'l' : freg = 2; break;	/* link */
	case 'u' : prot = 0; break;	/* user/owner */
	case 'g' : prot = 1; break;	/* group */
	case 'o' : prot = 2; break;	/* other */
	case 'r' : 			/* read */
	         if(prot == 0) { mode |= S_IRUSR; } /* user can read */
	    else if(prot == 1) { mode |= S_IRGRP; } /* group can read */
	    else if(prot == 2) { mode |= S_IROTH; } /* other can read */
	    break;	
	case 'w' : 			/* write */
	         if(prot == 0) { mode |= S_IWUSR; } /* user can write */
	    else if(prot == 1) { mode |= S_IWGRP; } /* group can write */
	    else if(prot == 2) { mode |= S_IWOTH; } /* other can write */
	    break;
	case 'x' :			/* execute */
	         if(prot == 0) { mode |= S_IXUSR; } /* user can execute */
	    else if(prot == 1) { mode |= S_IXGRP; } /* group can execute */
	    else if(prot == 2) { mode |= S_IXOTH; } /* other can execute */
	    break;	
	default  :			/* qualificator unknown */
	    free(buf);			/* file name buffer free */
	    buf = 0;			/* buffer pointer clear */
	    return(0);			/* return error */
    }
    qual += 1;				/* qualificator next */
 }
 struct stat stbuf;			/* file status */
 if(stat((char *)buf, &stbuf) < 0) {	/* file status read return code check */
    free(buf);				/* file name buffer free */
    buf = 0;				/* file name pointer clear */
    return(0);				/* return error */
 }
 if(buf != 0) {				/* file name buffer pointer check */
    free(buf);				/* file name buffer free */
    buf = 0;				/* file name pointer clear */
 }
 if(freg == 0) {			/* regular file check */
    if(S_ISREG(stbuf.st_mode) == 0) {	/* file bit in status check */
	return(0);			/* don't regular file */
    }
 } else 
 if(freg == 1) {			/* directory check */
    if(S_ISDIR(stbuf.st_mode) == 0) {	/* directory bit in status check */
	return(0);			/* don't directory */
    }
 } else 
 if(freg == 2) {			/* file link check */
    if(S_ISLNK(stbuf.st_mode) == 0) {	/* link bit in status check */
	return(0);			/* don't file link */
    }
 } else {
    return(0);
 }
 if((stbuf.st_mode & mode) == mode) {	/* status mode seted bit check */
    return(1);				/* file & mode ok */
 } else {
    return(0);
 }
}
int16_t ucrp_unlink(char * frmt, ...)	/* unlink file */
{
 u_int8_t * buf;			/* file name buffer pointer */
 buf = (u_int8_t *)malloc(		/* file name buffer allocation */
    ucrp_path_size			/* buffer size */
 );
 if(buf == 0) {				/* buffer allocation check */
    return(0);
 }
 va_list valt;				/* variable argument list */
 va_start(valt, frmt);			/* variable argument list init */
 int32_t cnt;				/* bytes writed/processed */
 cnt = vsnprintf(			/* file name form */
    (char *)buf, 			/* file name buffer */
    ucrp_path_size,			/* file name buffer size */
    frmt,				/* format */
    valt				/* argument list */
 );
 va_end(valt);				/* argument list deinit */
 if(
    (cnt < 0) ||			/* print error check */
    (cnt >= ucrp_path_size)		/* out of buffer check */
 ) {
    free(buf);				/* file buffer free */
    buf = 0;				/* buffer pointer clear */
    return(0);				/* unlink error */
 } 
 int16_t rvl;				/* return code */
 if(unlink((char *)buf) < 0) {		/* unlink function return code check */
    rvl = 0;				/* return code clear */
 } else {
    rvl = 1;				/* unlink ok */
 }
 if(buf != 0) {				/* buffer pointer check */
    free(buf);				/* buffer free */
    buf = 0;				/* buffer pointer clear */
 }
 return(rvl);
}
int16_t ucrp_clear(char * frmt, ...)	/* file clear */
{
 u_int8_t * buf;			/* file name buffer */
 buf = (u_int8_t *)malloc(		/* file name allocation */
    ucrp_path_size			/* file name buffer size */
 );
 if(buf == 0) {				/* buffer allocation check */
    return(0);				/* buffer allocation error */
 }
 int16_t rvl;				/* return code */
 rvl = 1;				/* return code start */
 va_list valt;				/* argument list */
 va_start(valt, frmt);			/* argument list init */
 int32_t cnt;				/* bytes writed/processed */
 cnt = vsnprintf(			/* file name form */
    (char *)buf,			/* file name buffer */
    ucrp_path_size,			/* file name buffer size */
    frmt,				/* format */
    valt				/* argument list */
 );
 va_end(valt);				/* argument list deinit */
 if(
    (cnt < 0) ||			/* print error check */
    (cnt >= ucrp_path_size)		/* out of buffer check */
 ) {
    free(buf);				/* file name buffer free */
    buf = 0;				/* file name buffer pointer clear */
    return(0);
 }
 FILE * file;				/* file description pointer */
 file = fopen(				/* file open */
    (char *)buf,			/* file name */
    UMSG_READBYTE			/* read mode */
 );
 if(file == 0) {			/* file open check */
    ucrp_error(				/* error message print */
	UMSG_OPEN_ERROR, 		/* file open error format */
	buf				/* file name buffer */
    );
    if(buf != 0) {			/* file buffer pointer check */
	free(buf);			/* file buffer free */
	buf = 0;			/* file buffer pointer clear */
    }
    return(0);				/* file clear error */
 }
 if(buf != 0) {				/* file buffer check */
    free(buf);				/* file buffer free */
    buf = 0;				/* file buffer pointer clear */
 }
 if(fseek(file, 0, SEEK_END) < 0) {	/* seek file end */
    rvl = 0;				/* return code clear */
 }
 u_int32_t len;				/* file length */
 len = ftell(file);			/* file length read */
 if(fseek(file, 0, SEEK_SET) < 0) {	/* file rewind */
    rvl = 0;				/* seek error */
 }
 if(ucrp_clear_mode == UCRP_CLEAR_INC) { /* increment mode clear test */
    u_int8_t wch;			/* character writed */
    wch = 0;				/* character writed start value */
    u_int32_t pos;			/* file position */
    for(pos = 0; pos < len; pos++) {	/* file write loop */
	if(fwrite(&wch, 1, 1, file) < 0) { /* write character */
	    rvl = 0;			/* return code clear */
	}
    }
 } else {
    u_int32_t wch;			/* word writed */
    if(ucrp_clear_mode == UCRP_CLEAR_CLR) {/* null mode clear test */
	wch = 0;			/* word writed start value */
    } else
    if(ucrp_clear_mode == UCRP_CLEAR_SET) {/* set mode clear test */
	wch = (~0);			/* word writed start value */
    } else {
	wch = 0;			/* word writed default */
    }
    u_int32_t pos;			/* file position */
    for(pos = 0; pos < len;) {		/* file write loop */
	pos += sizeof(wch);		/* file position next */
	if(pos < len) {			/* file position check */
	    if(fwrite(&wch, 4, 1, file) != 1) { /* word write */
		rvl = 0;		/* return code clear */
	    }
	} else {			/* out of file */
	    len = ((pos - len) & 3);	/* file tail size calculate */
	    if(len > 0) {		/* length check */
		if(fwrite(&wch, len, 1, file) != 1) { /* file tail write */
		    rvl = 0;		/* return code clear */
		}
	    }
	}
    }
 }
 if(fclose(file) < 0) {			/* file close return code check */
    rvl = 0;				/* return code clear */
 }
 return(rvl);				/* function return */
}
int16_t ucrp_usage(void)		/* help message print */
{
    ucrp_printf(			/* help message print */
	UCRP_USAGE			/* copyright string */
	"Usage: ucrp [options] <input file> [<output file>]\r\n" /* command line */
	"where options may be:\r\n"	/* options comment */
	"--license      - show this program license agreement\r\n" /* license option */
	"--license-ru   - show this program license agreement translated for russian language\r\n" /* license-ru option */
	"--encrypt      - encrypt file\r\n" /* encrypt file option */
	"--decrypt      - decrypt file\r\n" /* decrypt file option */
	"--passwd <passwd> | request - password\r\n" /* password option */
	"--hide-passwd [*|$|d] - hide password\r\n" /* hide password option */
	"--show-passwd  - show password\r\n" /* show password option */
	"--block-size	- block size\r\n" /* block size specification */
	"--change-dir   - change current directory\r\n" /* directory name option */
	"--info         - print encrypt file info message\r\n" /* info option */
	"--clear [inc|0,1] - clear current directory disk space and bash & mc history\r\n" /* clear command */
	"                 inc - write increment byte, 0 - write zero, 1 - write FF\r\n"
	"--disable "			/* disable command */
#ifdef  __UCRP_XOR_ENABLE__		/* XOR enable macro check */
	"xor|"				/* disable xor alg */
#endif						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* ADD enable macro check */
	"add|"				/* add alg disable */
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
	"rot|"				/* rotate alg disable */
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
	"tab|"				/* table alg disable */
#endif						/* __UCRP_TAB_ENABLE__ */
#ifdef __UCRP_ECC_ENABLE__		/* ECC enable macro check */
	"ecc|"				/* ECC alg disable */
#endif						/* __UCRP_ECC_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
	"bts|"				/* bts alg disable */
#endif						/* __UCRP_BTS_ENABLE__ */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
	"xre|"				/* xre disable */
#endif						/* __UCRP_XRE_ENABLE__ */
	"std - disable standart algoritms\r\n" /* std option */
	"--enable "			/* enable option */
#ifdef __UCRP_XOR_ENABLE__		/* XOR enable macro check */
	"xor|"				/* xor enable */
#endif						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* ADD enable macro check */
	"add|"				/* add enable */
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
	"rot|"				/* rotate enable */
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
	"tab|"				/* table enable */
#endif						/* __UCRP_TAB_ENABLE__ */
#ifdef __UCRP_ECC_ENABLE__		/* ECC enable macro check */
	"ecc|"				/* ecc enable */
#endif						/* __UCRP_ECC_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
	"bts|"				/* bts enable */
#endif						/* __UCRP_BTS_ENABLE__ */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
	"xre|"				/* xre enable */
#endif						/* __UCRP_XRE_ENABLE__ */
	"std - enable standart algoritms\r\n" /* standart algs enable */
	"--quiet        - quiet mode\r\n" /* quiet option */
	"--version      - version print and exit\r\n" /* version command */
	"--todo         - print todo and exit\r\n" /* todo command */
	"--changelog    - changelog print and exit\r\n" /* change log command */
	"--man          - man page print and exit\r\n" /* man command */
	"--help         - help note print and exit\r\n", /* help command */
	ucrp_version_hi,		/* version hight number */
	ucrp_version_lo			/* version low number */
    );
 return(1);
}
int16_t ucrp_man(void)			/* print man page */
{
 ucrp_printf(
    "MAN page ucrp v. %d.%d\r\n"	/* version number */
    "Usage: ucrp [options] <input file> [<output file>]\r\n" /* command line */
    "where options may be:\r\n"		/* options comment */
    "--license      - show this program license agreement\r\n" /* license option */
    "--license-ru   - show this program license agreement translated for russian language\r\n" /* license-ru option */
    "--encrypt      - encrypt file, example:\r\n" /* encrypt file option */
    "   ucrp --encrypt <input file> <output file> - encrypt file with empty password\r\n"
    "--decrypt      - decrypt file\r\n" /* decrypt file option */
    "password options:\r\n"
    "--passwd <passwd> | request - password from command line or keyboard\r\n" /* password option */
    "--hide-passwd [*|$|d] - hide password,print characters * or $, or d insted typed characters\r\n" /* hide password option */
    "--show-passwd  - show password\r\n" /* show password option */
    "--block-size   - block size\r\n" /* block size specification */
    "--change-dir   - change current directory\r\n" /* directory name option */
    "--info         - print encrypt file info message\r\n" /* info option */
    "--clear [inc|0,1] - clear current directory disk space and bash & mc history\r\n" /* clear command */
    "                 inc - write increment byte, 0 - write zero, 1 - write FF\r\n"
    "--disable "			/* disable command */
#ifdef  __UCRP_XOR_ENABLE__		/* XOR enable macro check */
    "xor|"				/* disable xor alg */
#endif						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* ADD enable macro check */
    "add|"				/* add alg disable */
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
    "rot|"				/* rotate alg disable */
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
    "tab|"				/* table alg disable */
#endif						/* __UCRP_TAB_ENABLE__ */
#ifdef __UCRP_ECC_ENABLE__		/* ECC enable macro check */
    "ecc|"				/* ECC alg disable */
#endif						/* __UCRP_ECC_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
    "bts|"				/* bts alg disable */
#endif						/* __UCRP_BTS_ENABLE__ */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
    "xre|"				/* xre disable */
#endif						/* __UCRP_XRE_ENABLE__ */
    "std - disable standart algoritms\r\n" /* std option */
    "--enable "				/* enable option */
#ifdef __UCRP_XOR_ENABLE__		/* XOR enable macro check */
    "xor|"				/* xor enable */
#endif						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* ADD enable macro check */
    "add|"				/* add enable */
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
    "rot|"				/* rotate enable */
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
    "tab|"				/* table enable */
#endif						/* __UCRP_TAB_ENABLE__ */
#ifdef __UCRP_ECC_ENABLE__		/* ECC enable macro check */
    "ecc|"				/* ecc enable */
#endif						/* __UCRP_ECC_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
    "bts|"				/* bts enable */
#endif						/* __UCRP_BTS_ENABLE__ */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
    "xre|"				/* xre enable */
#endif						/* __UCRP_XRE_ENABLE__ */
    "std - enable standart algoritms\r\n" /* standart algs enable */
    "--quiet        - quiet mode\r\n" /* quiet option */
    "--version      - version print and exit\r\n" /* version command */
    "--todo         - print todo and exit\r\n" /* todo command */
    "--changelog    - changelog print and exit\r\n" /* change log command */
    "--man          - man page print and exit\r\n" /* man command */
    "--help         - help note print and exit\r\n", /* help command */
    ucrp_version_hi,			/* version hight number */
    ucrp_version_lo			/* version low number */
 );
 return(1);
}
int16_t ucrp_license(void) /* cutted version - if you have source codes */
{ /* of this software, you must provide this ability to another users */
 ucrp_printf(
    UCRP_LICENSE,			/* license note print */
    ucrp_version_hi,			/* version number hight */
    ucrp_version_lo			/* version number low */
 );
 return(1);
}
int16_t ucrp_license_ru(void) /*   -      */
{ /*   ,      */
/*    */
 ucrp_printf(				/* russian license print */
    UCRP_LICENSE_RU,			/* russian license message */
    ucrp_version_hi,			/* version number hight */
    ucrp_version_lo			/* version number low */
 );
 return(1);
}
int16_t ucrp_version(void)		/* ucrp version print */
{
 if(ucrp_printf(			/* version string print */
    UMSG_VERSION, 			/* version format */
    ucrp_version_hi,			/* version hight number */
    ucrp_version_lo			/* version low number */
 ) < 0) {				/* print return code check */
    return(0);				/* version print error */
 }
 return(1);				/* version print ok */
}
int16_t ucrp_todo(void)			/* todo note print */
{
 char * msg = 
    "";
 ucrp_printf(msg);
 return(1);
}
int16_t ucrp_changelog(void)		/* change log print */
{
 ucrp_printf(				/* print function */
    UMSG_CHANGELOG			/* change log note */
 );
 return(1);
}
int16_t ucrp_init(void)			/* ucrp parameters init */
{
 int32_t pos;				/* password position */
 for(pos = 0; pos < (int) sizeof(ucrp_passwd); ++pos) { /* password loop */
    ucrp_passwd[pos] = 0;		/* password clear */
 }
 return(1);
}
int16_t ucrp_key(u_int32_t key, u_char * buf, int16_t len) /* key check, return length in bytes */
{
 if(
    (buf == 0) || 			/* buffer pointer check */
    (len <= 0)				/* length check */
 ) {
    return(0);
 }
 if(key <= 0xFF) {			/* key length one byte check */
    u_int8_t bchr;			/* character readed */
    bchr = *(buf + 0);			/* character read */
    if(bchr == key) {			/* key and character compare */
        return(1);			/* one byte length */
    } else {
        return(0);			/* don't key */
    }
 } else 
 if(key <= 0xFFFF) {			/* two bytes key check */
    if(len < 2) {			/* length check */
	return(0);			/* key don't found */
    } else {
        u_int16_t bchr;			/* two bytes character */
	bchr =				/* character read */
	    ( (((u_int32_t) *(buf + 1)) << 8) |  /* upper byte read */
	       ((u_int32_t) *(buf + 0)) ); /* low byte read */
	if(key == bchr) {		/* key & character compare */
	    return(2);			/* two byes length */
	} else {
	    return(0);			/* key don't found */
	} 
    }
 } else 
 if(key <= 0xFFFFFF) {			/* three bytes key check */
    if(len < 3) {			/* length check */
	return(0);			/* three bytes key don't found */
    } else {
        u_int32_t bchr;			/* three bytes word */
	bchr = ( 			/* three bytes word read */
	    (((u_int32_t) *(buf + 2)) << 16) |  /* byte 2 read */
	    (((u_int32_t) *(buf + 1)) << 8) |  /* byte 1 read */
	     ((u_int32_t) *(buf + 0))	/* byte 0 read */
	);
	if(key == bchr) {		/* key and three bytes word compare */
	    return(3);			/* 3 bytes */
	} else {
	    return(0);			/* key don't found */
	}
    }
 } else {				/* for bytes character */
    if(len < 4) {			/* length check */
	return(0);			/* key don't found */
    } else {
	u_int32_t bchr;			/* for bytes word */
	bchr = ( 			/* word read */
	  (((u_int32_t) *(buf + 3)) << 24) | /* byte 3 read */
	  (((u_int32_t) *(buf + 2)) << 16) | /* byte 2 read */
	  (((u_int32_t) *(buf + 1)) << 8) |  /* byte 1 read */
	   ((u_int32_t) *(buf + 0)) 	/* byte 0 read */
	);
	if(key == bchr) {		/* key and word compare */
	    return(4);			/* four bytes */
	} else {
	    return(0);			/* key don't found */
	}
    }
 }
}
/* echo = 0 - no echo, 1 - echo, 2 - '*', 3 - '$', 4 - 'd'
wait = 0 - no wait, 1 = wait */
int16_t ucrp_select(u_int8_t * buf, int16_t len, u_int8_t echo, u_int8_t wait)
{
 fd_set r_fds;				/* read file mask */
 FD_ZERO(&r_fds);			/* read mask clear */
 int stdinfd;				/* standart input descriptor */
 stdinfd = fileno(stdin);		/* standart input descriptor read */
 if(					/* standart input descriptor check */
    (stdinfd < 0) || 			/* descriptor low check */
    (stdinfd >= (int)sizeof(r_fds) * 8) /* descriptor hight check */
 ) {
    ucrp_error(				/* error message print */
	UMSG_STDIN_ERROR		/* stdin bad descriptor error message */
    );
    return(-1);				/* select error */
 }
 int maxfd;				/* maximal descriptor number */
 maxfd = stdinfd;			/* maximal descriptor number write */
#define TC_GET_SET_ATTR
#ifdef TC_GET_SET_ATTR
 struct termios stdintios;		/* standart input attributes */
 if(tcgetattr(stdinfd, &stdintios) < 0) { /* attributes read return code check */
    ucrp_error(				/* error message print */
	UMSG_TCGETATTR_STDIN_ERROR,	/* message format */
	errno				/* errno code */
    );
    return(-1);				/* select error */
 }
 struct termios tios;			/* attributes modified */
 tios = stdintios;			/* attributes read */
 cfmakeraw(&tios);			/* attributes raw */
 if(
    tcsetattr(				/* attributes set */
	stdinfd,			/* standart input descriptor */
	TCSANOW,			/* immedeatly */
	&tios				/* attributes raw */
    ) < 0) { 				/* set attributes error check */
    ucrp_error(				/* error message print */
	UMSG_TCSETATTR_STDIN_ERROR, 	/* message format */
	errno				/* error code */
    );
    return(-1);				/* function error */
 }        
#endif 
 int stdinprm;				/* stdin settings */
 stdinprm = fcntl(stdinfd ,F_GETFL,0);	/* stdin settings read */
 if(fcntl(stdinfd, F_SETFL, O_NONBLOCK) < 0) { /* set nonblock mode error check */
    ucrp_error(				/* error message print */
	UMSG_STDIN_NONBLOCK_ERROR	/* message */
    );
#ifdef TC_GET_SET_ATTR
    if(tcsetattr(stdinfd, TCSANOW, &stdintios) < 0) { /* stdin attributes restore */
	ucrp_error(				/* error message print */
	    UMSG_TCSETATTR_STDIN_ERROR, 	/* message format */
	    errno				/* error code */
	);
    }        
#endif 
    return(-1);				/* function error */
 }
 int16_t rvl;				/* return code */
 rvl = 1;				/* return code start */
 int16_t pos;				/* buffer position */
 pos = 0;				/* buffer position start */
 while(pos < len) {			/* buffer print loop */
    u_int8_t bch;			/* character */
    bch = *(buf + pos);			/* buffer character read */
    if(bch == '\0') {			/* string terminator check */
	{ break; }			/* loop exit */
    }
    if(echo == 1) {			/* echo check */
    } else 
    if(echo == 2) {			/* echo * check */
	bch = '*';			/* character printed write */
    } else
    if(echo == 3) {			/* echo $ check */
        bch = '$';			/* character printed write */
    } else 
    if(echo == 4) {			/* echo d check */
        bch = 'd';			/* character printed write */
    } else {
	bch = '*';			/* character printed default */
    }
    fprintf(				/* character print */
	stdout,				/* standart output */
	UMSG_CHARACTER,			/* character format */
	bch				/* ccharacter */
    );
    pos += 1;				/* next buffer position */
 }
 fflush(stdout);
 while(1) {				/* select loop */
    FD_SET(				/* read descriptor bit set */
	stdinfd,			/* standart input descriptor */
	& r_fds				/* read mask */
    );     
    struct timeval tval;		/* time interval */
    if(wait > 0) {			/* wait, seconds check */
        tval.tv_sec = wait;		/* wait time write */
    } else {
	tval.tv_sec = 0;		/* no wait */
    }
    tval.tv_usec = 0;			/* usec */
    int32_t cnt;			/* descriptor count */
    cnt = select(			/* select function */
        maxfd + 1,			/* maximal descriptor + 1 */
	& r_fds,			/* read mask */
	0,				/* write mask */
	0,				/* exeption mask */
	& tval				/* time interval */
    );
    if(kill_link > 1) {			/* term signal check */
	{ break; }			/* -> loop exit */
    }
    if(cnt < 0) {			/* select error */
        if(
    	    (errno == EWOULDBLOCK) ||	/* operation blocked check */
	    (errno == EINTR)		/* operation interrupted check */
	) {	
	    { continue; }		/* loop next */
	} else {
    	    rvl = -1;			/* function error */
	    ucrp_error(			/* error message print */
		UMSG_SELECT_ERROR,	/* message format */
	        errno			/* error code */
	    );
    	    { break; }			/* loop exit */
	}
    } else
    if(cnt == 0) {			/* descriptor count check */
        if(wait == 0) {			/* wait check */
    	    { break; }			/* no wait -> loop exit */
	} else {
    	    { continue; }		/* loop next */
	}
    }
    if(FD_ISSET(stdinfd,& r_fds) == 0) { /* read mask check */
	{ continue; }			/* loop next */
    }
    u_int8_t rdbuf[32];			/* read buffer */
    u_int8_t rch;			/* character readed */
    rdbuf[0] = 0;			/* read buffer clear */
    rch = '\0';				/* read character clear */
    while(1) {				/* read loop */
	while(1) {			/* buffer read loop */
	    cnt = read(stdinfd, rdbuf, sizeof(rdbuf)); /* stdin read */
	    if(cnt >= 0) {		/* read bytes check */
		{ break; }		/* >= 0 -> break */
	    }  
	    if(				/* errno code check */
		(errno == EWOULDBLOCK) || /* operation blocked check */
		(errno == EINTR)	/* operation interrupted check */
	    ) {	
		{ continue; }		/* loop next */
	    } else {
    		rvl = -1;		/* read error */
		ucrp_error(		/* error message print */
		    UMSG_STDIN_READ_ERROR, /* message format */
	    	    errno		/* error code */
		);
    		{ break; }		/* loop exit */
	    }
	}
	if(
	    (rvl == -1) ||		/* return code check */
	    (cnt <= 0) ||		/* character readed check */
	    (kill_link > 1)		/* term received check */
	) {
	    { break; }			/* loop exit */
	}
	int16_t rdpos = 0;		/* read position */
	while(rdpos < cnt) {		/* read buffer loop */
    	    u_int8_t key;		/* BS or delete */
    	    u_int8_t keylen;		/* key length */
	    keylen = ucrp_key(UCRP_BS, rdbuf, cnt); /* BS check */
	    if(keylen > 0) { 		/* key length check */
		key = 1;		/* BS key */
	    } else {
		keylen = ucrp_key(UCRP_DEL, rdbuf, cnt); /* delete check */
		if(keylen > 0) { 	/* key length check */
		    key = 1;		/* delete key */
	        } else {
		    key = 0;		/* don't delete or BS */
		}
	    }
	    if(key > 0) {		/* BS or delete */
		rch = 0x08;
		rdpos += keylen;	/* buffer position next */
		if(pos > 0) {		/* position check */
		    pos -= 1;		/* position previous */
		    if(buf != 0) {	/* buffer pointer don't null check */
	        	if(pos < len) {	/* position out of buffer check */
	    	    	    *(buf + pos) = '\0'; /* string terminator write */
			}
		    }
		    if(			/* echo mode check */
			(echo == 1) || 	/* echo character entered check */
			(echo == 2) || 	/* echo * mode check */
			(echo == 3) || 	/* echo $ mode check */
			(echo == 4) 	/* echo d mode check */
		    ) {
			fprintf(stdout,UMSG_BACKSPACE); /* back space print */
	    		fprintf(stdout,UMSG_SPACE);     /* space format */
		        fprintf(stdout,UMSG_BACKSPACE); /* back space print */
			fflush(stdout);
		    }
		}
	    } else {			/* don't delete or BS */
		rch = rdbuf[rdpos];	/* character read */
		if(			/* read cheracter check */
		    (rch == '\n') || 	/* new line check */
		    (rch == '\r')	/* <CR> check */
		) {
		    rch = '\n';		/* new line entered */
		    fprintf(stdout,UMSG_CRNL); /* <CR><NL> print */
		    fflush(stdout);
	    	    { break; }		/* -> string entered */
		} else
		if((rch == 0x1B)) {	/* escape character */
	    	    pos = 0;		/* buffer position clear */
	    	    fprintf(		/* canceled message print */
    			stdout,		/* standart output */
			UMSG_CANCELED	/* canceled message */
		    );
		    fflush(stdout);
	    	    { break; }		/* loop exit */
		} else {		/* valid character */
	    	    if(buf != 0) {	/* buffer pointer check */
	    		if(
			    (pos >= 0) || /* buffer position check */
			    (pos < len) /* position out of buffer check */
			) { 
			    *(buf + pos) = rch;
			}
		    }
	    	    ++pos;		/* output position next */
		    if(echo == 1) {	/* echo check */
		    } else 
		    if(echo == 2) {	/* echo * check */
			rch = '*';	/* character printed write */
		    } else
		    if(echo == 3) {	/* echo $ check */
    			rch = '$';	/* character printed write */
		    } else 
		    if(echo == 4) {	/* echo d check */
    			rch = 'd';	/* character printed write */
		    } else {
			rch = '*';	/* character printed default */
		    }
		    fprintf(		/* character print */
			stdout,		/* standart output */
			UMSG_CHARACTER,	/* character format */
			rch		/* character readed */
		    );
		    fflush(stdout);
		}
		rdpos += 1;		/* position readed next */
	    } 
	}				/* read loop end */
	if(				/* loop termination check */
	    (rdpos < cnt) ||		/* input canceled check */
	    (rch == 0x1B) ||		/* input escaped */
	    (rch == '\n')		/* input end entered */
	) {
	    { break; }			/* -> loop exit */
	}
    }
    if(					/* loop exit check */
	(wait == 0) ||			/* no wait mode check */
	(rvl == -1) ||			/* return code check */
	(rch == 0x1B) ||		/* input escaped */
	(rch == '\n') ||		/* input terminator */
	(kill_link > 1)			/* more then one term received */
    ) {
        { break; }			/* -> loop exit */
    }
 }
 if(buf != 0 && len > 0) {		/* buffer pointer & buffer length check */
    if(pos > len) {			/* position out of buffer check */
	pos = len - 1;			/* position end */
    }
    *(buf + pos) = '\0';		/* terminator write */
 }
 if(fcntl(				/* stdin settings restore */
    stdinfd,				/* stdin descriptor */
    F_SETFL,				/* set parameters command */
    stdinprm				/* parameter original */
 ) < 0) {	
    ucrp_error(				/* error message print */
    	UMSG_STDIN_RESTORE_ERROR	/* error message */
    );
    rvl = -1;				/* function error write */
 }
#ifdef TC_GET_SET_ATTR
 if(tcsetattr(stdinfd, TCSANOW, &stdintios) < 0) {
    ucrp_error(				/* error message print */
	UMSG_TCSETATTR_STDIN_ERROR, 	/* message format */
	errno				/* error code */
    );
    rvl = -1;				/* function error write */
 }        
#endif 
 if(rvl < 0) {				/* function error check */
    return(rvl);			/* function error return */
 } else {
    return(pos);			/* buffer position return */
 }
}
int ucrp_random_fd = -1;		/* /dev/random file descriptor */
int ucrp_unrand_fd = -1;		/* /dev/urandom file descriptor */
int16_t ucrp_random_init(void)		/* random init */
{
 int16_t rvl;				/* return code */
 rvl = 1;				/* start return code */
 ucrp_random_fd = open(			/* open random in non block mode */
    UMSG_RANDOM_FILE_NAME,		/* /dev/random */
    (O_RDONLY | O_NONBLOCK)		/* read only nonblock mode */
 );
 if(ucrp_random_fd >= 0) {		/* /dev/random open check */
    int fdprm;				/* random device settings */
    fdprm = fcntl(			/* /dev/random settings read */
	ucrp_random_fd, 		/* file descriptor */
	F_GETFL				/* get settings command */
    );	
    if(fdprm < 0) {			/* read error check */
	ucrp_error(			/* error message print */
	    UMSG_GET_FCNTL_ERROR,	/* format */
	    UMSG_RANDOM_FILE_NAME	/* file name /dev/random */
	);
	rvl = 0;			/* return code clear */
    }
    if(fcntl(				/* /dev/random non block mode set */
	ucrp_random_fd, 		/* file descriptor */
	F_SETFL, 			/* set parameters command */
	(fdprm | O_NONBLOCK)		/* device settings */
    ) < 0) {
	ucrp_error(			/* error message print */
	    UMSG_O_NONBLOCK_ERROR	/* non block mode set error message */
	);
	rvl = 0;			/* return code clear */
    }
 }
 ucrp_unrand_fd = open(			/* /dev/urandom open in non block mode */
    UMSG_URAND_FILE_NAME,		/* /dev/urandom string */
    (O_RDONLY | O_NONBLOCK)		/* read only nonblock mode */
 );
 if(ucrp_unrand_fd >= 0) {
    int fdprm;
    fdprm = fcntl(
	ucrp_unrand_fd,
	F_GETFL
    );
    if(fdprm < 0) {
	ucrp_error(			/* error message print */
	    UMSG_GET_FCNTL_ERROR,	/* message format */
	    UMSG_URAND_FILE_NAME	/* file name */
	);
	rvl = 0;			/* return code clear */
    }
    if(fcntl(				/* /dev/urandom set non block mode */
	ucrp_unrand_fd, 		/* file descriptor */
	F_SETFL, 			/* set parameter */
	(fdprm | O_NONBLOCK)		/* setiongs */
    ) < 0) {
	ucrp_error(			/* error message print */
	    UMSG_O_NONBLOCK_ERROR	/* non block mode set error message */
	);
	rvl = 0;			/* return code clear */
    }
 }
 if(
    (ucrp_random_fd >= 0) || 		/* /dev/random file descriptor check */
    (ucrp_unrand_fd >= 0)		/* /dev/urandom file descriptor check */
 ) {
    return(rvl);			/* function return */
 } else {
    return(0);				/* random init error */
 }
}
int16_t ucrp_random_close(void)		/* random devices close */
{
 if(ucrp_random_fd >= 0) {		/* /dev/random descriptor check */
    close(ucrp_random_fd);		/* /dev/random close */
    ucrp_random_fd = -1;		/* /dev/random descriptor clear  */
 }
 if(ucrp_unrand_fd >= -1) {		/* /dev/urandom descriptor check */
    close(ucrp_unrand_fd);		/* /dev/urandom close */
    ucrp_unrand_fd = -1;		/* /dev/urandom descriptor clear  */
 }
 return(1);
}
u_int8_t ucrp_random(void)		/* random byte read */
{
 static u_int32_t rch = 0;		/* random word */
 if(ucrp_random_fd != -1) {		/* /dev/random descriptor check */
    if(read(ucrp_random_fd, &rch, 1) == 1) { /* byte read return code check */
	return(rch);			/* /dev/random readed from byte return */
    } 
 }
 if(ucrp_unrand_fd != -1) {		/* /dev/urandom descriptor check */
    if(read(ucrp_unrand_fd, &rch, 1) == 1) { /* byte read return code check */
	return(rch);			/* /dev/urandom readed from byte return */
    } 
 }
 rch = (rch << 11) | (rch >> 21); 	/* psevdo random word rotate */
 rch += 1;				/* psevdo random word increment */
 return((u_int8_t)rch); 		/* psevdo random return */
}
int32_t ucrp_comp(u_int16_t pos, int32_t prm) /* read/write compatibility bit */
{
 u_int16_t byte;			/* byte number */
 u_int8_t bit;				/* bit number */
 byte = (pos >> 3);			/* byte number calculate */
 bit = (pos & 0x7);			/* bit number calculate */
 if(byte >= sizeof(ucrp_comp_buf)) {	/* byte number out of buffer check */
    return(0);				/* read/write error */
 }
 int32_t ret;				/* bit readed */
 ret = ((ucrp_comp_buf[byte] >> (7-bit)) & 0x1); /* bit save */
 if(prm >= 0) {				/* write bit check, < 0 read only */
    if(prm > 0) {			/* 1 check */
	ucrp_comp_buf[byte] |=  (0x80 >> bit); /* bit set */
    } else {				/* -> 0 */
	ucrp_comp_buf[byte] &= ~(0x80 >> bit); /* bit clear */
    }
 }
 return(ret);				/* bit readed return */
}
int32_t ucrp_comp_prm(u_int16_t pos, int32_t len, int32_t prm) /* read write compatibility field */
{
 int32_t ret;				/* return field value */
 ret = 0;				/* return parameter clear */
 int32_t idx;				/* bit index */
 for(idx = 0; idx < len; ++idx) {	/* bit loop */
    ret <<= 1;				/* shift left */
    ret |= ucrp_comp(pos + idx, -1);	/* read bit */
 }
 if(prm >= 0) {				/* write field check */
    for(idx = 0; idx < len; ++idx) {	/* bit loop */
	if((prm & (1 << (len - idx - 1))) != 0) { /* bit check */
	    ucrp_comp(pos + idx, 1);	/* write 1 */
	} else {
	    ucrp_comp(pos + idx, 0);	/* write 0 */
	}
    }	
 }
 return(ret);
}
int16_t ucrp_comp_init(void)		/* compatibility buffer init */
{
 u_int16_t pos;				/* compatibility position */
 for(pos = 0; pos < sizeof(ucrp_comp_buf); ++pos) { /* compatibility init loop */
    ucrp_comp_buf[pos] = 0;		/* compatibility buffer clear */
 }
 ucrp_comp_buf[0] = 'u';		/* program name */
 ucrp_comp_buf[1] = 'c';		/* ucrp1.30 */
 ucrp_comp_buf[2] = 'r';
 ucrp_comp_buf[3] = 'p';
 ucrp_comp_buf[4] = '1';
 ucrp_comp_buf[5] = '.';
 ucrp_comp_buf[6] = '3';
 ucrp_comp_buf[7] = '0';
#ifdef __UCRP_XOR_ENABLE__		/* XOR enable macro check */
 if((ucrp_enable & UCRP_XOR_ENABLE) != 0) { /* xor enable check */
    ucrp_comp(ucrp_xor_comp, 1);	/* xor compatibility bit set */
 }
#endif						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* ADD enable macro check */
 if((ucrp_enable & UCRP_ADD_ENABLE) != 0) {  /* add enable check */
    ucrp_comp(ucrp_add_comp, 1);	/* add compatibility enable */
 }
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
 if((ucrp_enable & UCRP_ROT_ENABLE) != 0) { /* rot enable check */
    ucrp_comp(ucrp_rot_comp, 1);	/* rotate compatibility set */
 }
#endif 						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
 if((ucrp_enable & UCRP_TAB_ENABLE) != 0) { /* tab enable check */
    ucrp_comp(ucrp_tab_comp, 1);	/* byte table enable */
 }
#endif						/* __UCRP_TAB_ENABLE__ */
#ifdef __UCRP_ECC_ENABLE__		/* ECC enable macro check */
 if((ucrp_enable & UCRP_ECC_ENABLE) != 0) { /* ecc enable check */
    ucrp_comp(ucrp_ecc_comp, 1);	/* ECC enable */
 }
#endif						/* __UCRP_ECC_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
 if((ucrp_enable & UCRP_BTS_ENABLE) != 0) { /* bts enable check */
    ucrp_comp(ucrp_bts_comp, 1);	/* bit swapping enable */	
 }
#endif						/* __UCRP_BTS_ENABLE__ */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
 if((ucrp_enable & UCRP_XRE_ENABLE) != 0) { /* xre enable check */
    ucrp_comp(ucrp_xre_comp, 1);	/* xre enable */
 } 
#endif					/* __UCRP_XRE_ENABLE__ */
 ucrp_comp_prm(				/* source buffer size write to compatibility buffer */
    UCRP_SRC_BUF,			/* source buffer size bit number */
    UCRP_SRC_BUF_LEN, 			/* source buffer size field length */
    ucrp_src_buf_size			/* source buffer size */
 ); 
 ucrp_comp_prm(				/* destination buffer size write to compatibility buffer */
    UCRP_DST_BUF,			/* destination buffer size bit number */
    UCRP_DST_BUF_LEN, 			/* destination buffer size field length */
    ucrp_dst_buf_size			/* destination buffer size */
 );
 return(1);
}
int16_t ucrp_comp_check(void)		/* compatibility buffer check */
{
 if(
    (
	(ucrp_comp_buf[0] == 'u') &&	/* u uenc test */
	(ucrp_comp_buf[1] == 'e') &&	/* e */
	(ucrp_comp_buf[2] == 'n') &&	/* n */
	(ucrp_comp_buf[3] == 'c')	/* c */
    ) || 
    (
	(ucrp_comp_buf[0] == 'u') &&	/* u ucrp test */
	(ucrp_comp_buf[1] == 'c') &&	/* c */
	(ucrp_comp_buf[2] == 'r') &&	/* r */
	(ucrp_comp_buf[3] == 'p')	/* p */
    ) 
 ) {
    if(
	(ucrp_comp_buf[4] == ' ') &&	/* ' ' uenc 1.0 test */
        (ucrp_comp_buf[5] == '1') &&	/* 1 */
	(ucrp_comp_buf[6] == '.') &&	/* . */
        (ucrp_comp_buf[7] == '0')	/* 0 */
    ) {
	return(1);			/* uenc 1.0 file */
    } else 
    if(
	(ucrp_comp_buf[4] == '1') &&	/* 1 check, ucrp1.dd check */
	(ucrp_comp_buf[5] == '.') &&	/* . */
	(
	    ucrp_comp_buf[6] >= '0' && 	/* d */
	    ucrp_comp_buf[6] <= '9'
	) &&
	(
	    ucrp_comp_buf[7] >= '0' && 	/* d */
	    ucrp_comp_buf[7] <= '9'
	)
    ) {
        return(1);			/* ucrp1.dd or uenc1.dd format */
    } else 
    {
        return(0);
    }
 } else {				/* don't uenc or ucrp */
    return(0);				/* not valid file */
 }
}
u_int32_t ucrp_comp_version(void)	/* return file version */
{
 if(					/* program name check */
    (
	(ucrp_comp_buf[0] == 'u') &&	/* u uenc test */
	(ucrp_comp_buf[1] == 'e') &&	/* e */
	(ucrp_comp_buf[2] == 'n') &&	/* n */
	(ucrp_comp_buf[3] == 'c')	/* c */
    ) || 
    (
	(ucrp_comp_buf[0] == 'u') &&	/* u ucrp test */
	(ucrp_comp_buf[1] == 'c') &&	/* c */
	(ucrp_comp_buf[2] == 'r') &&	/* r */
	(ucrp_comp_buf[3] == 'p')	/* p */
    ) 
 ) {
    if(
	(ucrp_comp_buf[4] == ' ') &&	/* ' ' uenc 1.0 test */
        (ucrp_comp_buf[5] == '1') &&	/* 1 */
	(ucrp_comp_buf[6] == '.') &&	/* . */
        (ucrp_comp_buf[7] == '0')	/* 0 */
    ) {
	return(100);			/* return version 100 */
    } else 
    if(
	(ucrp_comp_buf[4] == '1') &&	/* 1 */
	(ucrp_comp_buf[5] == '.') &&	/* . */
	(
	    ucrp_comp_buf[6] >= '0' && 	/* d */
	    ucrp_comp_buf[6] <= '9'
	) &&
	(
	    ucrp_comp_buf[7] >= '0' && 	/* d */
	    ucrp_comp_buf[7] <= '9'
	)
    ) {
	u_int32_t ver;			/* ucrp file version */
	ver = 100 + 			/* version hight number */			
	    (ucrp_comp_buf[6] - '0') * 10 +  /* version lo number hi digit */
	    (ucrp_comp_buf[7] - '0');	/* version lo number lo digit */
        return(ver);			/* file version return */
    } else 
    {
        return(0);			/* not ucrp file, version 0 */
    }
 } else {
    return(0);				/* not ucrp file, version 0 */
 }
}
int32_t ucrp_key(u_int16_t pos, int32_t prm) /* bit read/write from/to key buffer */
{
 u_int16_t byte;			/* byte number */	
 u_int8_t bit;				/* bit number */	
 byte = (pos >> 3);			/* byte number calculate */
 bit = (pos & 0x7);			/* bit number calculate */
 if(byte >= sizeof(ucrp_key_buf)) {	/* byte number out of buffer check */
    return(0);				/* read/write error */
 }
 int32_t ret; 				/* return bit value */
 ret = ((ucrp_key_buf[byte] >> (7-bit)) & 0x1); /* bit read */
 if(prm >= 0) {				/* write check, < 0 - read only */
    if(prm > 0) {			/* write 1 check */
	ucrp_key_buf[byte] |= (0x80 >> bit); /* bit set */
    } else {				/* write 0 check */
	ucrp_key_buf[byte] &= ~(0x80 >> bit); /* bit clear */
    }
 }
 return(ret);				/* function return */
}
int32_t ucrp_key_prm(u_int16_t pos, int32_t len, int32_t prm) /* read write key field */
{
 int32_t ret;				/* return field value */
 ret = 0;				/* return parameter clear */
 int32_t idx;				/* bit index */
 for(idx = 0; idx < len; ++idx) {	/* bit loop */
    ret <<= 1;				/* shift left */
    ret |= ucrp_key(pos + idx, -1);	/* read bit */
 }
 if(prm >= 0) {				/* write field check */
    for(idx = 0; idx < len; ++idx) {	/* bit loop */
	if((prm & (1 << (len - idx - 1))) != 0) { /* bit check */
	    ucrp_key(pos + idx, 1);	/* write 1 */
	} else {
	    ucrp_key(pos + idx, 0);	/* write 0 */
	}
    }	
 }
 return(ret);
}
int16_t ucrp_key_init(void)		/* key buffer init */
{
 u_int16_t pos;				/* key buffer position */
 for(pos = 0; pos < sizeof(ucrp_key_buf); ++pos) { /* key buffer loop */
    ucrp_key_buf[pos] = 0;		/* key buffer clear */
 }
#if defined __UCRP_XOR_ENABLE__ || 	/* XOR enable macro check */ \
defined __UCRP_XRE_ENABLE__ /* or XRE enable macro check */
 u_int8_t xor_prm;			/* xor parameter */
 xor_prm = ucrp_random();		/* xor parameter generate */
 ucrp_key_prm(				/* xor parameter write */
    UCRP_XOR_BASE,			/* xor parameter key buffer bit number */
    UCRP_XOR_LEN,			/* xor parameter field length */
    xor_prm				/* xor parameter */
 );
#endif 						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* add enable macro check */
 u_int8_t add_prm;			/* add parameter */
 add_prm = ucrp_random();		/* add parameter generate */
 ucrp_key_prm(				/* add parameter write */
    UCRP_ADD_BASE,			/* add parameter key buffer bit number */
    UCRP_ADD_LEN,			/* add parameter field length */
    add_prm				/* add parameter */
 );
#endif 						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
 u_int8_t rot_prm;			/* rotate parameter */
 rot_prm = (ucrp_random() & 0x7);	/* rotate parameter generate */
 ucrp_key_prm(				/* rotate parameter write */
    UCRP_ROT_BASE,			/* rot parameter key buffer bit number */
    UCRP_ROT_LEN,			/* rot parameter field length */
    rot_prm				/* rotate parameter */
 );
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
 u_int8_t tab_prm;			/* tab parameter */
 tab_prm = ucrp_random();		/* tab parameter generate */
 ucrp_key_prm(				/* key field function */
    ucrp_tab_base,			/* table offset base bit number */
    UCRP_TAB_LEN,			/* table offset base field length */
    tab_prm				/* table offset */
 );
#endif						/* __UCRP_TAB_ENABLE__ */
 u_int8_t tab[256];			/* translate table */
 for(pos = 0; pos < 256; ++pos) {	/* table init loop */
    tab[pos] = pos;			/* translate table item init */
 }
 for(pos = 0; pos < 256 * 4; ++pos) {	/* element change loop */
    u_int8_t ch0;			/* element table index 0 */
    u_int8_t ch1;			/* element table index 1 */
    u_int8_t prm0;			/* parameter 0 */
    u_int8_t prm1;			/* parameter 1 */
    ch0 = ucrp_random();		/* element table index 0 generate */
    ch1 = ucrp_random();		/* element table index 1 generate */
    prm0 = tab[ch0];			/* parameter 0 read */
    prm1 = tab[ch1];			/* parameter 1 read */
    tab[ch0] = prm1;			/* parameter 1 write */
    tab[ch1] = prm0;			/* parameter 0 write */
 }  
 for(pos = 0; pos < 256; ++pos) {	/* translate table write loop */
    ucrp_key_prm(			/* key field function */
	(ucrp_table_base + pos * 8), 	/* bit position */
	8, 				/* field length */
	tab[pos]			/* table item */
    );
 }
#if defined __UCRP_XOR_ENABLE__ || /* XOR enable macro check */ \
defined __UCRP_XRE_ENABLE__ /* XRE enable macro check */
 {					/* xor or xre defined */
    u_int8_t xrlw;			/* xor law */
    u_int8_t xrprm;			/* xor law parameter */
    xrlw = (ucrp_random() % (UCRP_LAW_MAX + 1)); /* xor law generate */
    xrprm = (ucrp_random() & 0x7);	/* xor law parameter generate */
    ucrp_key_prm(			/* key field function */
	UCRP_XRL_BASE,			/* xor law bit number */
	UCRP_XRL_LEN, 			/* xor law field length */
	xrlw				/* xor law */
    );
    ucrp_key_prm(			/* key field function */
	UCRP_XRL_PRM_BASE,		/* xor law parameter bit number */
	UCRP_XRL_PRM_LEN,		/* xor law field length */
	xrprm				/* xor law */
    );
 }
#endif 						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
 {					/* bit table init & write */
    u_int8_t bts_prm;			/* bts parameter */
    bts_prm = ucrp_random();		/* bts parameter read */
    ucrp_key_prm(			/* key field function */
	ucrp_bts_base,			/* bts parameter bit number */
	UCRP_BTS_LEN,			/* bts parameter field length */
	bts_prm				/* bts parameter */
    );
    u_char bttb[8];			/* bit swapping table */
    u_int8_t btps;			/* bit swapping table position */
    for(btps = 0; btps < 8; ++btps) {	/* bit table init loop */
	bttb[btps] = btps;		/* bit table item init */
    }
    for(btps = 0; btps < 16; ++btps) {	/* exchange loop */
	u_int8_t ch0;			/* table position 0 */
	u_int8_t ch1;			/* table position 1 */
	u_int8_t prm0;			/* readed number 0 */
	u_int8_t prm1;			/* readed number 1 */
	ch0 = (ucrp_random() & 0x7);	/* table position 0 generate */
	ch1 = (ucrp_random() & 0x7);	/* table position 1 generate */
	prm0 = bttb[ch0];		/* number 0 read */
	prm1 = bttb[ch1];		/* number 1 read */
	bttb[ch0] = prm1;		/* number 1 write */
	bttb[ch1] = prm0;		/* number 0 write */
    }
    for(btps = 0; btps < 8; ++btps) {	/* bit swap table write */
	ucrp_key_prm(			/* key field function */
	    ucrp_bts_table_base + btps * 8,  /* bit number calculate */
	    8,				/* field length */
	    bttb[btps]			/* bts table item */
	);
    }
 }  
#endif						/* __UCRP_BTS_ENABLE__ */
 return(1);
}
int16_t ucrp_comp_read(void)		/* compatibility buffer read */
{
 if(ucrp_finp == 0) {			/* file description pointer check */
    return(0);				/* function error */
 }
 if(fread(				/* file read */
    ucrp_comp_buf, 			/* compatibility buffer */
    1, 					/* item size */
    sizeof(ucrp_comp_buf), 		/* compability buffer size */
    ucrp_finp				/* file description pointer */
 ) != sizeof(ucrp_comp_buf)) {		/* read count check */
    return(0);				/* compatibility read error */
 }
 ucrp_src_buf_size = 			/* source buffer size read */
    ucrp_comp_prm( 			/* compatibility field function */
	UCRP_SRC_BUF,			/* source buffer size bit number */
	UCRP_SRC_BUF_LEN,		/* source buffer size field length */
	-1				/* read command */
    );
 if(ucrp_src_buf_size == 0) {		/* source buffer size check */
    ucrp_src_buf_size = UCRP_SRC_SIZE;	/* source buffer size default */
 }
 ucrp_dst_buf_size = 			/* destination buffer size read */
    ucrp_comp_prm(			/* compatibility field function */
	UCRP_DST_BUF,			/* destination buffer size bit number */
	UCRP_DST_BUF_LEN, 		/* destination buffer size field length */
	-1				/* read command */
    );
 if(ucrp_dst_buf_size == 0) {		/* destination buffer size check */
    ucrp_dst_buf_size = UCRP_DST_SIZE;	/* destination buffer size default */
 }
 return(1);
}
int16_t ucrp_comp_write(void)		/* compatibility buffer write */
{
 if(ucrp_fout == 0) {			/* output file description pointer check */
    return(0);				/* function error return */
 }
 if(fwrite(				/* file write */
    ucrp_comp_buf, 			/* compatibility buffer */
    1, 					/* item size */
    sizeof(ucrp_comp_buf), 		/* compability buffer size */
    ucrp_fout				/* file out description pointer */
 ) != sizeof(ucrp_comp_buf)) {		/* write count check */
    return(0);				/* compatibility write error */
 }
 return(1);
}
int16_t ucrp_key_read(void)		/* key buffer read */
{
 if(ucrp_finp == 0) {			/* input file description pointer check */
    return(0);				/* function error return */
 }
 if(fread(				/* read function */
    ucrp_key_buf, 			/* key buffer */
    1, 					/* item size */
    sizeof(ucrp_key_buf), 		/* key buffer size */
    ucrp_finp				/* input file description pointer */
 ) != sizeof(ucrp_key_buf)) {		/* read bytes check */
    return(0);
 }
 return(1);
}
int16_t ucrp_key_write(void)
{
 if(ucrp_fout == 0) {			/* output file description pointer check */
    return(0);
 }
 if(fwrite(				/* write function */
    ucrp_key_buf, 			/* key buffer */
    1,					/* item size */
    sizeof(ucrp_key_buf), 		/* key buffer size */
    ucrp_fout				/* output file description pointer */
 ) != sizeof(ucrp_key_buf)) {		/* write bytes check */
    return(0);
 }
 return(1);
}
int16_t ucrp_info(void)			/* print file information */
{
 {					/* ucrp version print */
    u_int8_t vrbuf[9];			/* version string buffer */
    u_int8_t vrpos;			/* buffer position */
    for(vrpos = 0; vrpos + 1 < (int)sizeof(vrbuf); ++vrpos) { /* version string copy loop */
	vrbuf[vrpos] = ucrp_comp_buf[vrpos]; /* character copy */
	if(vrbuf[vrpos] == '\0') {	/* string terminator check */
	    { break; }			/* => loop exit */
	}
    }
    vrpos = sizeof(vrbuf) - 1;		/* version buffer last position */
    vrbuf[vrpos] = '\0';		/* string terminator write */
    ucrp_printf(			/* version number print */
	UMSG_INFO_VERSION,		/* message format */
	vrbuf				/* version string */
    );
 }
#if defined __UCRP_XOR_ENABLE__		/* XOR enable macro check */
 if(ucrp_passwd[0] == '\0') {		/* password empty check */
    u_int8_t xor_comp;			/* xor compatibility */
    u_int8_t xor_prm;			/* xor parameter */
    xor_comp = ucrp_comp(ucrp_xor_comp, -1); /* xor compatibility bit read */
    xor_prm =				/* xor parameter read */
	ucrp_key_prm(			/* key field function */
	    UCRP_XOR_BASE,		/* xor parameter key buffer bit number */
	    UCRP_XOR_LEN,		/* xor parameter field length */
	    -1				/* read command */
	);
    ucrp_printf(			/* xor info print */
	UMSG_INFO_XOR, 			/* --info xor format */
	(xor_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), /* xor enable string */ 
	xor_prm				/* xor parameter */
    );
    {
	u_int8_t xrlw;			/* xor law */
	u_int8_t xrpr;			/* xor law parameter */
        char * xrstr;			/* xor law string */
	xrlw = 				/* xor law read */
	    ucrp_key_prm(		/* key field function */
		UCRP_XRL_BASE,		/* xor law bit number */
		UCRP_XRL_LEN,		/* xor law field length */
		-1			/* read command */
	    );
	xrpr = 				/* xor law parameter read */
	    ucrp_key_prm(		/* key field function */
		UCRP_XRL_PRM_BASE,	/* xor law parameter bit number */
		UCRP_XRL_PRM_LEN,	/* xor law parameter length */
		-1			/* read command */
	    );
	if(xrlw != UCRP_LAW_NULL) {	/* xor law don't null check */
	    if(xrpr == UCRP_LAW_NULL) {	/* law null check */
		xrstr = UMSG_LAW_NULL;	/* law null string */
	    } else
	    if(xrpr == UCRP_LAW_AD3) {	/* add law check */
		xrstr = UMSG_LAW_AD3;  	/* law ad3 */
	    } else
	    if(xrpr == UCRP_LAW_ROT) {	/* rot law check */
		xrstr = UMSG_LAW_ROT; 	/* law rot string */
	    } else
	    if(xrpr == ucrp_law_tab) {	/* tab law check */
		xrstr = UMSG_LAW_TAB; 	/* law tab string */
	    } else
	    if(xrpr == ucrp_law_bts) {	/* bts law check */
		xrstr = UMSG_LAW_BTS; 	/* law bts string */
	    } else {
		xrstr = UMSG_LAW_NULL;	/* law null string */
	    }
	} else {
	    xrstr = UMSG_LAW_NULL;	/* law null string */
	}
	ucrp_printf(			/* xor law string print */
	    UMSG_INFO_XRL, 		/* xor law format */
	    xrstr, 			/* xor law name */
	    xrlw			/* xor law numerical value */
	);
	ucrp_printf(			/* xor law parameter string print */
	    UMSG_INFO_XRL_PRM,		/* xor law parameter format */
	    xrpr			/* xor low parameter */
	);
    }
 } else {				/* not null password */
    u_int8_t xor_comp;			/* xor compatibility */
    u_int8_t xor_prm;			/* xor parameter */
    xor_comp = ucrp_comp(ucrp_xor_comp, -1); /* xor compatibility bit read */
    xor_prm = 				/* xor parameter calculate */
	ucrp_key_prm(			/* key field function */
	    UCRP_XOR_BASE,		/* xor parameter key buffer bit number */
	    UCRP_XOR_LEN,		/* xor parameter field length */
	    -1				/* read command */
	) + ucrp_passwd[0];		/* + passwword 0 character */
    ucrp_printf(
	UMSG_INFO_XOR,			/* --info xor format */
	(xor_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), 
	xor_prm
    );
    if(xor_comp != 0) {			/* xor compatibility check */
        char * xrstr;			/* xor law string */
	u_int8_t xrlw;			/* xor law */
	u_int8_t xrpr;			/* xor law parameter */
	xrlw = 				/* xor law read */
	    ucrp_key_prm(		/* key field function */
		UCRP_XRL_BASE,		/* xor law bit number */
		UCRP_XRL_LEN,		/* xor law field length */
		-1			/* read command */
	    );
	xrpr =				/* xor law parameter read */
	    ucrp_key_prm(		/* key field function */
		UCRP_XRL_PRM_BASE,	/* xor law parameter bit number */
		UCRP_XRL_PRM_LEN, 	/* xor law field length */
		-1			/* read command */
	    );
	if(xrlw != UCRP_LAW_NULL) {	/* law null check */
	    xrlw += (ucrp_passwd[0] >> 3); /* modify xor law with password 0 */
	    xrlw =  (xrlw % (UCRP_LAW_MAX + 1)); /* xor law norm */
	    if(xrpr == UCRP_LAW_AD3) {	/* add law check */
		xrpr = (xrpr + ucrp_passwd[0]); /* xor parameter modify with password 0 */
		xrstr = UMSG_LAW_AD3;	/* law add3 string */
	    } else 
	    if(xrpr == UCRP_LAW_ROT) {	/* rotate law check */
		xrpr = ((xrpr + ucrp_passwd[0]) & 0x7); /* xor parameter modify with passwd 0 */
		xrstr = UMSG_LAW_ROT;	/* law rot string */
	    } else 
	    if(xrpr == ucrp_law_tab) {	/* tab law check */
		xrpr = (xrpr + ucrp_passwd[0]); /* xor parameter modify with password 0 */
		xrstr = UMSG_LAW_TAB;	/* law tab string */
	    } else 
	    if(xrpr == ucrp_law_bts) {	/* bts law check */
		xrpr = ((xrpr + ucrp_passwd[0]) & 0x7); /* xor law parameter modify with password 0 */
		xrstr = UMSG_LAW_BTS;	/* law bts string */
	    } else {
		xrstr = UMSG_LAW_UNKN;	/* law unknown string */
	    }
	} else {
	    xrstr = UMSG_LAW_NULL;	/* law null string */
	}
	ucrp_printf(			/* xor law specification print */
	    UMSG_INFO_XRL, 		/* xor law format */
	    xrstr,			/* xor law string */
	    xrlw			/* xor law numerical */
	);
	ucrp_printf(			/* xor law parameter specification print */
	    UMSG_INFO_XRL_PRM,		/* xor law parameter format */
	    xrpr			/* xor law paracmeter */
	);
    }
 }
#endif						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
 if(ucrp_passwd[0] == '\0') {		/* password 0 check */
    u_int8_t xre_comp;			/* xre compatibility */
    u_int8_t xor_prm;			/* xor parameter */
    xre_comp = ucrp_comp(ucrp_xre_comp, -1); /* xre compatibility read */
    xor_prm = 				/* xor parameter read */
	ucrp_key_prm(			/* key field function */
	    UCRP_XOR_BASE,		/* xor parameter key buffer bit number */
	    UCRP_XOR_LEN,		/* xor parameter field length */
	    -1				/* read command */
	);
    ucrp_printf(			/* xre string print */
	UMSG_INFO_XRE, 			/* --info xre format */
	(xre_comp ? UMSG_INFO_ON : UMSG_INFO_OFF),  /* on/off string */
	xor_prm				/* xor parameter */
    );
    {
	u_int8_t xrlw;			/* xor law string */
	u_int8_t xrpr;			/* xor law parameter */
        char * xrstr;			/* xor law string */
	xrlw =				/* xor law read */
	    ucrp_key_prm(		/* key field function */
		UCRP_XRL_BASE,		/* xor law bit number */
		UCRP_XRL_LEN,		/* xor law field length */
		-1			/* read command */
	    );
	xrpr = 				/* xor law parameter read */
	    ucrp_key_prm(		/* key field function */
		UCRP_XRL_PRM_BASE,	/* xor law parameter bit number */
		UCRP_XRL_PRM_LEN,	/* xor law parameter field length */
		-1			/* read command */
	    );
	if(xrlw == UCRP_LAW_NULL) {	/* xor law null check */
	    xrstr = UMSG_LAW_NULL;	/* law null string */
	} else 
	if(xrlw == UCRP_LAW_AD3) {	/* xor law ad3 check */
	    xrstr = UMSG_LAW_AD3;	/* law ad3 string */
	} else 
	if(xrlw == UCRP_LAW_ROT) {	/* xor law rot check */
	    xrstr = UMSG_LAW_ROT;	/* law rot string */
	} else
	if(xrlw == ucrp_law_tab) {	/* xor law tab check */
	    xrstr = UMSG_LAW_TAB;	/* law tab string */
	} else
	if(xrlw == ucrp_law_bts) {	/* xor law bts check */
	    xrstr = UMSG_LAW_BTS;	/* law bts string */
	} else {
	    xrstr = UMSG_LAW_UNKN;	/* law unknown string */
	}
	ucrp_printf(			/* xor law string print */
	    UMSG_INFO_XRL,		/* xor law format */
	    xrstr, 			/* xor law name */
	    xrlw			/* xor law numerical */
	);
	ucrp_printf(			/* xor law parameter print */
	    UMSG_INFO_XRL_PRM,		/* xor law parameter format */
	    xrpr			/* xor law parameter */
	);
    }
 } else {
    u_int8_t xre_comp;			/* xre compatibility */
    xre_comp = ucrp_comp(ucrp_xre_comp, -1); /* xre compatibility read */
    if(xre_comp != 0) {			/* xre compatibility check */
	u_int8_t xor_prm;		/* xor parameter */
        char * xrstr;			/* xor law name */
	u_int8_t xrlw;			/* xor law */
	u_int8_t xrpr;			/* xor law parameter */
	xor_prm = 			/* xor parameter calculate */
	    ucrp_key_prm(		/* key field function */
		UCRP_XOR_BASE,		/* xor parameter key buffer bit number */
		UCRP_XOR_LEN, 		/* xor parameter field length */
		-1			/* read command */
	    ) + ucrp_passwd[0];		/* + password 0 byte */
	xrlw = 				/* xor law read */
	    ucrp_key_prm(		/* key field function */
		UCRP_XRL_BASE,		/* xor law bit number */
		UCRP_XRL_LEN,		/* xor law field length */
		-1			/* read command */
	    );
	xrpr = 				/* xor law parameter read */
	    ucrp_key_prm(		/* key field function */
		UCRP_XRL_PRM_BASE,	/* xor law parameter bit number */
		UCRP_XRL_PRM_LEN,	/* xor law parameter field length */
		-1			/* read command */
	    );
	xrlw += (ucrp_passwd[0] >> 3);	/* xor law modify with password 0 */
	xrlw = (xrlw % (UCRP_LAW_MAX + 1)); /* xor law norm */
	xrpr = ((xrpr + ucrp_passwd[0]) & 0x7); /* xor law parameter modify with password 0 */
	if(xrlw != UCRP_LAW_NULL) {	/* law null check */
	    if(xrlw == UCRP_LAW_AD3) {	/* ad3 law check */
		xrstr = UMSG_LAW_AD3;	/* ad3 string */
	    } else
	    if(xrlw == UCRP_LAW_ROT) {	/* rot law check */
		xrstr = UMSG_LAW_ROT;	/* law rot string */
	    } else 
	    if(xrlw == ucrp_law_tab) {	/* tab law check */
		xrstr = UMSG_LAW_TAB;	/* law tab string */
	    } else
	    if(xrlw == ucrp_law_bts) {	/* bts law check */
		xrstr = UMSG_LAW_BTS;	/* law bts string */
	    } else {
		xrstr = UMSG_LAW_UNKN;	/* law unknown string */
	    }
	} else {
	    xrstr = UMSG_LAW_NULL;	/* law null string */
	}
	ucrp_printf(			/* xre string print */
	    UMSG_INFO_XRE,		/* --info xre format */
	    (xre_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), /* on/off */ 
	    xor_prm			/* xor parameter */
	);
	ucrp_printf(			/* xor law string print */
	    UMSG_INFO_XRL,		/* xor law format */
	    xrstr,			/* xor law name */
	    xrlw			/* xor law numerical */
	);
	ucrp_printf(			/* xor law parameter string print */
	    UMSG_INFO_XRL_PRM, 		/* xor law parameter format */
	    xrpr			/* xor law parameter */
	);
    }
 }
#endif						/* __UCRP_XRE_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* add enable macro check */
 {
    u_int8_t add_comp;			/* add compatibility */
    u_int8_t add_prm;			/* add parameter */
    add_comp = ucrp_comp(ucrp_add_comp, -1); /* add compatibility read */
    if(ucrp_passwd[0] == '\0') {	/* password empty check */
	add_prm = 			/* add parameter read */
	    ucrp_key_prm(		/* key field function */
		UCRP_ADD_BASE,		/* add parameter key buffer bit number */
		UCRP_ADD_LEN,		/* add parameter field length */
		-1			/* read command */
	    );
    } else {
	add_prm = 			/* add parameter calculate */
	    ucrp_key_prm(		/* key field function */
		UCRP_ADD_BASE,		/* add parameter key buffer bit number */
		UCRP_ADD_LEN,		/* add parameter field length */
		-1			/* read command */
	    ) + ucrp_passwd[1];		/* + password 1 character */
    }
    ucrp_printf(			/* add specification print */
	UMSG_INFO_ADD, 			/* info add format */
	(add_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), /* on/off */
	add_prm				/* add parameter */
    );
 }
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
 {
    u_int8_t rot_comp;			/* rotate compatibility */
    u_int8_t rot_prm;			/* rotate parameter */
    rot_comp = ucrp_comp(ucrp_rot_comp, -1); /* rotate compatibility read */
    if(ucrp_passwd[0] == '\0') {
	rot_prm = 			/* rotate parameter read */
	    ucrp_key_prm(		/* key field function */
		UCRP_ROT_BASE,		/* rot parameter key buffer bit number */
		UCRP_ROT_LEN,		/* rot parameter field length */
		-1			/* read command */
	    );
    } else {
	rot_prm = (			/* rotate parameter calculate */
	    ucrp_key_prm(		/* key field function */
		UCRP_ROT_BASE,		/* rot parameter key buffer bit number */
		UCRP_ROT_LEN,		/* rot parameter field length */
		-1			/* read command */
	    ) + ucrp_passwd[2]		/* + password 2 character */
	) & 0x7;			/* norm */
    }
    ucrp_printf(			/* rotate specification print */
	UMSG_INFO_ROT, 			/* info rot format */
	(rot_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), /* on/off */
	rot_prm				/* rotate parameter */
    );
 }
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
 {
    u_int8_t bts_comp;			/* bit swapping compatibility */
    u_int8_t bts_prm;			/* bts value */
    u_int8_t bts_tab[8];		/* bts table */
    bts_comp = ucrp_comp(ucrp_bts_comp, -1); /* bit swapping compatibility read */
    bts_prm = 				/* bts parameter read */
	ucrp_key_prm(			/* key field function */
	    ucrp_bts_base,		/* bts parameter bit number */
	    UCRP_BTS_LEN,		/* bts parameter length */
	    -1				/* read command */
	);
    u_int8_t dspos;			/* bts table destination position */
    if(ucrp_comp_version() >= 118) {	/* version check */
	if(ucrp_passwd[0] != 0) { 	/* password enable check */
	    bts_prm = 			/* bts parameter modify with passwd 3 */
		((bts_prm + ucrp_passwd[3]) & 0x7);
	} else {
	    bts_prm &= 0x7;		/* bts parameter norm */
	}
	dspos = bts_prm;		/* destination table position init */
    } else {				/* version 1.00 format */
	dspos = 0;			/* */
    }
    ucrp_printf(			/* bts string print */
        UMSG_INFO_BTS,			/* info bts format */
        (bts_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), /* on/off */
        bts_prm				/* bts parameter */
    );
    u_int8_t btpos;			/* bts table position */
    for(btpos = 0; btpos < 8; ++btpos) { /* bts loop */
        bts_tab[dspos] =		/* bts table read */
    	    ucrp_key_prm(		/* key field function */
    		ucrp_bts_table_base + btpos * 8, /* bit number */
		8, 			/* field length */
		-1			/* read command */
	    );
	dspos = ((dspos + 1) & 0x7);	/* destination position next */
    }
    for(btpos = 0; btpos < 8; ++btpos) { /* bts loop */
        ucrp_printf(			/* bts item print */
    	    UMSG_02X,			/* hex 2 character width format */
	    bts_tab[btpos]		/* bts table */
	);
	if(btpos + 1 < 8) {
	    ucrp_printf(		/* space print */
		UMSG_SPACE		/* space format */	
	    );
	}
    }
    ucrp_printf(UMSG_CRNL);		/* <cr><nl> print */
 }
#endif						/* __UCRP_BTS_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
 {
    u_int8_t tab_comp;			/* byte table compatibility */
    u_int8_t tab_prm;			/* tab parameter, table offset */
    tab_comp = ucrp_comp(ucrp_tab_comp, -1); /* byte table enable read */
    tab_prm = 				/* table offset read */
	ucrp_key_prm(			/* key field function */
	    ucrp_tab_base,		/* table offset field bit number */
	    UCRP_TAB_LEN,		/* table offset field length */
	    -1				/* read command */
	);
    if(ucrp_comp_version() >= 118) {	/* compatibility version check, 1.18 and up  */
	if(ucrp_comp_version() >= 130) { /* version 1.30 and up check */
	    if(ucrp_passwd[0] != '\0') { /* don't empty password check */
		tab_prm = ((tab_prm + ucrp_passwd[4]) & 0xFF); /* + password character 4 */
	    }
	} else {			/* version 1.28 and low */
	    tab_prm = ((tab_prm + ucrp_passwd[4]) & 0xFF); /* => don't check password */
	}
    } else {				/* version 1.00 format */
	tab_prm = 0;			/* destination position clear */
    } 
    ucrp_printf(			/* tab string print */
	UMSG_INFO_TAB, 			/* info tab format */
	(tab_comp ? UMSG_INFO_ON : UMSG_INFO_OFF), /* on/off */
	tab_prm				/* tab parameter */
    );
    u_int8_t ttab[256];			/* translate table */
    u_int16_t ttpos;			/* translate table position */
    for(ttpos = 0; ttpos < 256; ++ttpos) { /* table read loop */
	ttab[tab_prm] = 		/* translate table read */
	    ucrp_key_prm(		/* key field function */
	        ucrp_table_base + (ttpos * 8), /* bit number position */
	        8, 			/* field length */
	        -1			/* read command */
	    );
	tab_prm = (tab_prm + 1) & 0xFF;	/* table destination position next */
    }
    ucrp_printf(UMSG_INFO_TRTABLE);	/* translate table format */
    for(ttpos = 0; ttpos < 256; ++ttpos) { /* translate table print loop */
	ucrp_printf(			/* table item print */
	    UMSG_02X,			/* hex 2 character width format */
	    ttab[ttpos]			/* translate table item */
	);
    	ucrp_printf(			/* space print */
	    UMSG_SPACE			/* space format */	
	);
	if(((ttpos + 1) & 0xF) == 0) {	
    	    ucrp_printf(UMSG_CRNL);	/* line separator print */
	}
    }
 }
#endif						/* __UCRP_TAB_ENABLE__ */
#ifdef __UCRP_ECC_ENABLE__		/* ECC enable macro check */
 {
    u_int8_t ecc_comp;			/* ecc compatibility */
    ecc_comp = ucrp_comp(ucrp_ecc_comp, -1); /* ecc compatibility read */
    ucrp_printf(			/* ecc specification print */
	UMSG_INFO_ECC, 			/* info ecc format */
	(ecc_comp ? UMSG_INFO_ON : UMSG_INFO_OFF) /* on/off */
    );
 }
#endif						/* __UCRP_ECC_ENABLE__ */
 u_int32_t srcsize;			/* source buffer size */
 srcsize = ucrp_comp_prm(		/* source buffer size read */
    UCRP_SRC_BUF,			/* source buffer size bit number */
    UCRP_SRC_BUF_LEN, 			/* source buffer size field length */
    -1					/* read command */
 );
 ucrp_printf(
    UMSG_INFO_SRC_BLEN, 		/* info source buffer size format */
    srcsize, 				/* source buffer size, decimal */
    srcsize				/* source buffer size, hexadecimal */
 );
 u_int32_t dstsize;			/* destination buffer size */
 dstsize = ucrp_comp_prm(		/* compatibility function */
    UCRP_DST_BUF, 			/* destination buffer size bit number */
    UCRP_DST_BUF_LEN, 			/* destination buffer size field length */
    -1					/* read command */
 );
 ucrp_printf(				/* destinatin buffer length print */
    UMSG_INFO_DST_BLEN,			/* destination buffer length format */
    dstsize,				/* destination buffer size */
    dstsize				/* destination buffer size */
 );
 return(1);				/* return ok */
}
int16_t ucrp_hash_init(void)		/* password hash init */
{
 int16_t rvl;				/* return code */
 rvl = 1;				/* return code start */
 u_int32_t pos;				/* password position */
 if(ucrp_passwd[0] == '\0') {		/* empty password check */
    for(pos = 0; pos < sizeof(ucrp_passwd); ++pos) { /* password init loop */
	if(ucrp_comp_version() >= 118) {/* v. 1.18 password check */
	    ucrp_passwd[pos] = 0;	/* password clear */
	} else {
	    ucrp_passwd[pos] = pos;	/* < v. 1.18 password increment filling */
	}
    }
 } else {				/* password don't empty */
    u_int32_t len;			/* password length */
    len = strlen(ucrp_passwd);		/* password length calculate */
    pos = len;				/* position to password terminator */
    while(pos < sizeof(ucrp_passwd)) {	/* password copy loop */
	u_int32_t srpos;		/* source position */
	srpos = 0;			/* source position start */
	while(srpos < len) {		/* password copy loop */
	    if(pos >= sizeof(ucrp_passwd)) { /* out of password buffer check */
		{ break; }		/* loop exit */
	    }
	    ucrp_passwd[pos] = ucrp_passwd[srpos]; /* password copy */
	    srpos += 1;			/* source position next */
	    pos += 1;			/* destination position next */
	}				/* password copy loop end */
    }					/* password filling loop end */
 }
 u_int8_t alg;				/* hash algoritm index */
#define UCRP_ALG_XOR 	 0		/* xor aloritm num 0 */
#define UCRP_ALG_ADD 	 1		/* add aloritm num 1 */
#define UCRP_ALG_ROT 	 2		/* rotate aloritm num 2 */
#define UCRP_ALG_TAB_118 3		/* tab v. 1.18 aloritm num 3 hist order */
#define UCRP_ALG_BTS_118 4		/* bts v. 1.18 aloritm num 4 hist order */
#define UCRP_ALG_BTS_130 3		/* bts v. 1.30 aloritm num 3 */
#define UCRP_ALG_TAB_130 4		/* tab v. 1.30 aloritm num 4 */
#if defined __UCRP_XOR_ENABLE__ || /* XOR enable macro check */ \
defined __UCRP_XRE_ENABLE__  		/* XRE enable macro check */
 u_int8_t xor_comp;			/* xor compatibility */
 u_int8_t xre_comp;			/* xor extended compatibility */
 u_int8_t xor_prm;			/* xor parameter */
 xor_comp =				/* xor compatibility read */
#if defined __UCRP_XOR_ENABLE__		/* XOR enable macro check */
    ucrp_comp(ucrp_xor_comp, -1);	/* xor compatibility read */
#else 
    0;					/* xor enable macro don't defined -> 0 */
#endif 					/* __UCRP_XOR_ENABLE__ */
 xre_comp =				/* xre compatibility read */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
    ucrp_comp(ucrp_xre_comp, -1); 	/* xre compatibility read */
#else 
    0;					/* xre enable macro don't defined -> 0 */
#endif						/* __UCRP_XRE_ENABLE__ */
 if(xre_comp > 0) {			/* xre compatibility enabled check */
    xor_comp = 1;			/* xor or xre enabled */
 }
 xor_prm =				/* xor parameter read */
    ucrp_key_prm(			/* key field function */
	UCRP_XOR_BASE,			/* xor parameter key buffer bit number */
	UCRP_XOR_LEN,			/* xor parameter field length */
	-1				/* read command */
    );
#endif				/* __UCRP_XOR_ENABLE__ || __UCRP_XRE_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* add enable macro check */
 u_int8_t add_comp;			/* add compatibility */
 u_int8_t add_prm;			/* add parameter */
 add_comp = ucrp_comp(ucrp_add_comp, -1); /* add compatibility read */
 add_prm = 				/* add parameter read */
    ucrp_key_prm(			/* key field function */
	UCRP_ADD_BASE,			/* add parameter key buffer bit number */
	UCRP_ADD_LEN,			/* add parameter field length */
	-1				/* read command */
    );
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
 u_int8_t rot_comp;			/* rotate compatibility */
 u_int8_t rot_prm;			/* rotate parameter */
 rot_comp = ucrp_comp(ucrp_rot_comp, -1); /* rotate compatibility read */
 rot_prm = ( 				/* rotate parameter read */
    ucrp_key_prm(			/* key field function */
	UCRP_ROT_BASE,			/* rot parameter key buffer bit number */
	UCRP_ROT_LEN,			/* rot parameter field length */
	-1				/* read command */
    ) & 0x7);				/* rotate parameter norm */
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
 u_int8_t bts_comp;			/* bit swapping compatibility */
 u_int8_t bts_prm;			/* bit swapping compatibility parameter */
 bts_comp = ucrp_comp(ucrp_bts_comp, -1); /* bit swapping compatibility read */
 bts_prm = 				/* bts parameter read */
    ucrp_key_prm(			/* key field function */
	ucrp_bts_base,			/* bts parameter bit number */
        UCRP_BTS_LEN,			/* bts parameter length */
	-1				/* read command */
    );
 u_int8_t btst[256];			/* byte translate table */
 u_int8_t bttb[8];			/* bit swapping table */
 {
    u_int8_t tval;			/* bit table checking */
    int16_t dspos;			/* bit table position */
    tval = 0;				/* test parameter init */
    dspos = (bts_prm & 0x7);		/* bit table position init */
    for(pos = 0; pos < 8; ++pos) { 	/* bit swapping table read loop */
	u_int8_t prm;			/* bit table item */
	prm =				/* bts table item read */
	    ucrp_key_prm(		/* key field function */
	        ucrp_bts_table_base + pos * 8, /* bit number calculate */
		8, 			/* field length */
		-1			/* read command */
	    );
	if(prm >= 8) {			/* bit table item check */
	    rvl = 0;			/* return code clear */
	}
	if((tval & (1 << prm)) > 0) {	/* bit table item check */
	    rvl = 0;			/* return code clear */
	} else {
	    tval |= (1 << prm); 	/* bit set in check parameter */
	}
	bttb[dspos] = prm;		/* bit swapping table write */
	dspos = ((dspos + 1) & 0x7); 	/* next read position */
    }
    for(pos = 0; pos < 256; ++pos) {	/* byte table loop */
	u_int8_t tres = 0;		/* translated byte */
	for(dspos = 0; dspos < 8; ++dspos) {		
	    if((pos & (1 << bttb[dspos]))) { /* if bit seted in position */
		tres |= (1 << dspos);	/* set bit in result byte */
	    }
	}
	btst[pos] = tres;		/* byte table write */
    }
 }
#endif						/* __UCRP_BTS_ENABLE__ */
 for(alg = 0; alg < 5; ++alg) {		/* algoritm loop */
    if(alg == UCRP_ALG_XOR) {		/* xor algoritm check */
#if defined __UCRP_XOR_ENABLE__ || /* XOR enable macro check */ \
defined __UCRP_XRE_ENABLE__  /* XRE enable macro check */
	if(xor_comp > 0) {		/* xor compatibility check */
	    for(pos = 0; pos < sizeof(ucrp_passwd); ++pos) { /* password loop */
		u_int8_t rch;		/* character readed */
		rch = ucrp_passwd[pos];	/* character read */
		rch = (rch ^ xor_prm);	/* xor operation */
		ucrp_passwd[pos] = rch;	/* character write */
	    }
	}
#else 				/* __UCRP_XOR_ENABLE__ || __UCRP_XRE_ENABLE__ */
	{ continue; }			/* xor code disabled -> next algorithm */
#endif				/* __UCRP_XOR_ENABLE__ || __UCRP_XRE_ENABLE__ */
    }					/* xor algoritm end */
    if(alg == UCRP_ALG_ADD) {		/* hash algorithm add check */
#ifdef __UCRP_ADD_ENABLE__		/* add enable macro check */
	if(add_comp > 0) {		/* add compatibility check */
	    for(pos = 0; pos < sizeof(ucrp_passwd); ++pos) { /* password loop */
		u_int8_t rch;		/* character readed */
		rch = ucrp_passwd[pos];	/* password character read */
		rch = (rch + add_prm);	/* add operation */
		ucrp_passwd[pos] = rch;	/* character write */
	    }
	}
#else						/* __UCRP_ADD_ENABLE__ */
	{ continue; }			/* add code disabled -> next algoritm */
#endif						/* __UCRP_ADD_ENABLE__ */
    }
    if(alg == UCRP_ALG_ROT) {		/* hash algoritm rot check */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
	if(rot_comp > 0) {		/* rotate compatibility check */
	    for(pos = 0; pos < sizeof(ucrp_passwd); ++pos) { /* password loop */
		u_int8_t rch;		/* character readed */
		rch = ucrp_passwd[pos]; /* character read */
		if(ucrp_comp_version() >= 118) { /* version > 118 check */
		    rch = 		/* left rotate */
			((rch << rot_prm) | 	/* left shift */
			(rch >> (8 - rot_prm))	/* back shift */
		    );
		} else {
		    rch = 		/* right rotate */
			((rch >> rot_prm) | 	/* right shift */
			(rch << (8 - rot_prm))	/* back shift */
		    );
		}
		ucrp_passwd[pos] = rch;	/* character write */
	    }
	}
#else						/* __UCRP_ROT_ENABLE__ */
	{ continue; }			/* rot code disabled -> next algoritm */
#endif						/* __UCRP_ROT_ENABLE__ */
    }
    if(
	( /* TAB_118 && >= v.1.18 && v.1.30 */
	    (alg == UCRP_ALG_TAB_118) &&    /* TAB_118 check */
	    (ucrp_comp_version() >= 118) && /* version >= 118 check */
	    (ucrp_comp_version() < 130)	    /* version < 130 check */
	) ||
	( /* TAB_130 && >= v.1.18 && v.1.30 */
	    (alg == UCRP_ALG_TAB_130) &&    /* TAB_130 check */
	    (ucrp_comp_version() >= 130)    /* version >= 130 check */
	)
    ) {
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
	u_int8_t tab_comp;		/* byte table enable read */
        u_int8_t tab_prm;		/* tab parameter */
	tab_comp = ucrp_comp(ucrp_tab_comp, -1); /* byte table enable read */
        tab_prm = 			/* table parameter read */
	    ucrp_key_prm(		/* key field function */
		ucrp_tab_base,		/* table offset field bit number */
	        UCRP_TAB_LEN,		/* table offset length */
		-1			/* read command */
	    );
	if(tab_comp > 0) {		/* tab compatibility check */
	    u_int8_t tab[256];		/* translate table */
	    u_int32_t dspos;		/* table destination position */
	    if(ucrp_comp_version() >= 118) { /* ucrp compatibility test */
		dspos = (tab_prm & 0xFF); /* destination position start value */
	    } else {
		dspos = 0;		/* destination position start value */
	    }
	    for(pos = 0; pos < 256; ++pos) { /* table read loop */
		if(ucrp_comp_version() >= 118) { /* ucrp version test */
		    tab[dspos] = 		/* table read */
			ucrp_key_prm(		/* key field function */
			    (ucrp_table_base + pos * 8), /* bit number */
		    	    8,			/* field length */
		    	    -1			/* read command */
			);
		} else {
		    tab[dspos] = pos;
		}
		dspos += 1;		/* destination position next */
		if(dspos >= 256) {	/* destination position out of upper limit check */
		    dspos = 0;		/* destination position start */
		}
	    }
#if defined __UCRP_XOR_ENABLE__ || /* XOR enable macro check */ \
defined __UCRP_XRE_ENABLE__ /* XRE enable macro check */
	    if(xor_comp > 0) {		/* xor compatibility check */
		for(pos = 0; pos < 256; ++pos) { /* table loop */
		    u_int8_t rch;	/* character readed */
	    	    rch = tab[pos];	/* character read */
		    rch = (rch ^ xor_prm); /* xor operation */
	    	    tab[pos] = rch;	/* character write */
		}
	    }
#endif			/* __UCRP_XOR_ENABLE__ || __UCRP_XRE_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* add anable macro check */
	    if(add_comp > 0) {		/* add compatibility macro check */
		for(pos = 0; pos < 256; ++pos) { /* table loop */
		    u_int8_t rch;	/* character readed */
		    rch = tab[pos];	/* character read */
	    	    rch = (rch + add_prm); /* add operation */
		    tab[pos] = rch;	/* character write */
		}
	    }
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
	    if(rot_comp > 0) {		/* rotate enable macro */
		for(pos = 0; pos < 256; ++pos) { /* table loop */
		    u_int8_t rch;	/* character readed */
		    rch = tab[pos];	/* character read */
		    if(ucrp_comp_version() >= 118) { /* version read */
	    		rch = (		/* v.1.18 an up left rotate */
			    (rch << rot_prm) | /* left shift */
			    (rch >> (8 - rot_prm)) /* right shift */
			);
		    } else {		
	    		rch = (		/* lo version right rotate */
			    (rch >> rot_prm) | /* right shift */
			    (rch << (8 - rot_prm)) /* back shift */
			);
		    }
		    tab[pos] = rch;	/* table write */
		}
	    }
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
	    if(ucrp_comp_version() >= 130) {    /* version >= 130 check */
		if(bts_comp > 0) {	/* bts compatibility test */
		    for(pos = 0; pos < 256; ++pos) { /* table loop */
			u_int8_t rch;	/* character readed */
			rch = tab[pos]; /* character read */
			rch = btst[rch];/* character translate */
	    		tab[pos] = rch;	/* table write */
		    }
		}
	    }
#endif						/* __UCRP_BTS_ENABLE__ */
	    for(pos = 0; pos < sizeof(ucrp_passwd); ++pos) { /* password loop */
		u_int8_t rch;		/* character readed */
		rch = ucrp_passwd[pos];	/* character read */
		rch = tab[rch];		/* table convert */
		ucrp_passwd[pos] = rch;	/* password character write */
	    }
	}
#else						/* __UCRP_TAB_ENABLE__ */
	{ continue; }			/* tab undefined -> algoritm next */
#endif						/* __UCRP_TAB_ENABLE__ */
    }
    if(
	(
	    (alg == UCRP_ALG_BTS_118) &&    /* bts 1.18 */
	    (ucrp_comp_version() >= 118) && /* >= v.1.18 */
	    (ucrp_comp_version() < 130)	    /* < v.1.30 */
	) ||
	(
	    (alg == UCRP_ALG_BTS_130) &&    /* bts 1.30 */
	    (ucrp_comp_version() >= 130)    /* >= v.1.30 */
	)
    ) {
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
	if(bts_comp > 0) {		/* bts compatibility test */
	    for(pos = 0; pos < sizeof(ucrp_passwd); ++pos) { /* password loop */
		u_int8_t rch;		/* character readed */
		rch = ucrp_passwd[pos]; /* character read */
		rch = btst[rch];	/* character translate */
	        ucrp_passwd[pos] = rch;	/* table write */
	    }
	}
#else						/* __UCRP_BTS_ENABLE__ */
	{ continue; }			/* bts don't defined -> next algoritm */
#endif						/* __UCRP_BTS_ENABLE__ */
    }
 }
#ifdef __UCRP_HASH_DEBUG__		/* hash debug macro check */
 ucrp_printf(UMSG_CRNL);
 ucrp_printf(UMSG_HASH);		/* hash string */
 for(pos = 0; pos < (int)sizeof(ucrp_passwd); ++pos) {
    ucrp_printf(
	UMSG_02X,			/* hex 2 character width format */
	ucrp_passwd[pos]
    );
    ucrp_printf(
	UMSG_SPACE			/* space format */
    );
 }
 ucrp_printf(UMSG_CRNL);
#endif						/* __UCRP_HASH_DEBUG__ */
 return(rvl);
}
int32_t ucrp_read(u_int8_t * buf, int32_t cnt) /* read from input file */
{
 if(					/* parameters check */
    (ucrp_finp == 0) ||			/* input file description pointer check */
    (buf == 0)				/* source buffer pointer check */
 ) {
    return(0);				/* function return */
 }
 cnt = fread(				/* file read */
    buf, 				/* buffer pointer */
    1,					/* item size */
    cnt,				/* bytes read */
    ucrp_finp				/* file input */
 );
 return(cnt);				/* bytes readed return */
}
int32_t ucrp_write(u_int8_t * buf, int32_t cnt)
{
 if(
    (ucrp_fout == 0) ||			/* file output pointer check */
    (buf == 0)				/* source buffer pointer check */
 ) {
    return(0);
 }
 cnt = fwrite(				/* file write */
    buf, 				/* buffer pointer */
    1, 					/* item size */
    cnt, 				/* item write */
    ucrp_fout				/* file output */
 );
 return(cnt);
}
#ifdef __UCRP_XOR_ENABLE__		/* XOR enable macro check */
int32_t ucrp_xor_en(u_int8_t * dst, u_int8_t * src, int len) /* xor encrypt */
{
 if(					/* input & output buffer check */
    (dst == 0) ||			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);
 }
 int32_t rvl;				/* return code */
 rvl = 1;				/* return code start */
 u_int8_t xor_prm;			/* xor mask */
 u_int8_t xrlw;				/* xor mask change law */
 u_int8_t xrpr;				/* xor mask change parameter */
 xor_prm = 				/* xor mask calculate */
    ucrp_key_prm(			/* key field function */
	UCRP_XOR_BASE,			/* xor parameter key buffer bit number */
	UCRP_XOR_LEN, 			/* xor parameter field length */
	-1				/* read command */
    ) + ucrp_passwd[0];			/* + password 0 character */
 xrlw =					/* xor law read */
    ucrp_key_prm(			/* key field function */
	UCRP_XRL_BASE,			/* xor law bit nuber */
	UCRP_XRL_LEN,			/* xor law field length */
	-1				/* read command */
    );
 xrpr =					/* xor law parameter read */
    ucrp_key_prm(			/* key field function */
	UCRP_XRL_PRM_BASE,		/* xor law parameter bit number */
	UCRP_XRL_PRM_LEN,		/* xor law parameter field length */
	-1				/* read command */
    );
 if(xrlw != UCRP_LAW_NULL) {		/* xor law null check */
    xrlw += (ucrp_passwd[0] >> 3);	/* xor law modify with password 0 */
    xrlw =  (xrlw % (UCRP_LAW_MAX + 1));/* norm */
    if(xrpr == UCRP_LAW_AD3) {		/* add law check */
	xrpr = (xrpr + ucrp_passwd[0]);	/* xor law parameter modify with password 0 */
	int32_t pos;			/* buffer position */
	for(pos = 0; pos < len; ++pos) {/* encrypt loop */
	    *(dst + pos) = (*(src + pos) ^ xor_prm); /* encrypt/decrypt byte */
	    xor_prm += xrpr;		/* xor mask modify */
	}
    } else 
    if(xrpr == UCRP_LAW_ROT) {		/* law rotate check */
	xrpr = ((xrpr + ucrp_passwd[0]) & 0x7); /* xor law parameter modify with password 0 */
	int32_t pos;			/* buffer position */
	for(pos = 0; pos < len; ++pos) { /* encrypt loop */
	    *(dst + pos) = (*(src + pos) ^ xor_prm); /* encrypt/decrypt byte */
	    xor_prm = (			/* xor mask rotate */
		(xor_prm >> xrpr) | 	/* right shift */
		(xor_prm << (8 - xrpr))); /* back shift */
	}
    } else 
    if(xrpr == ucrp_law_tab) {		/* tab law check */
	u_int8_t ttab[256];		/* translate table */
	int16_t srpos;			/* source table position */
	int16_t dspos;			/* destination table position */
	xrpr = (xrpr + ucrp_passwd[0]); /* xor law change parameter modify with password 0 */
	dspos = (xrpr & 0xFF);		/* destination position */
	for(srpos = 0; srpos < 256; ++srpos) { /* translate table read */
	    ttab[srpos] = 		/* table read */
		ucrp_key_prm(		/* key field function */
		    (ucrp_table_base + (dspos * 8)), /* bit number */
		    8, 			/* field length */
		    -1			/* read command */
		);
	    dspos += 1;			/* next position */
	    if(dspos >= 256) {		/* position check */
		dspos = 0;		/* position start */
	    }
	}
	int32_t pos;			/* buffer position */
	for(pos = 0; pos < len; ++pos) { /* buffer loop */
	    *(dst + pos) = (*(src + pos) ^ xor_prm); /* encrypt/decrypt byte */
	    xor_prm = ttab[ (xor_prm & 0xFF) ]; /* xor mask modify */
	}
    } else 
    if(xrpr == ucrp_law_bts) {		/* bts law check */
	u_int8_t bttb[8];		/* bit swapping table */
	int16_t dspos;			/* destination position */
	int16_t bpos;			/* bit position */
	int16_t tval;			/* test parameter */
	xrpr = ((xrpr + ucrp_passwd[0]) & 0x7); /* xor law parameter modify with password 0 */
	dspos = xrpr;			/* destination position init */
	if((dspos < 0) || (dspos >= 8)) { /* destination position check */
	    dspos = 0;			/* destination position clear */
	}
	tval = 0;			/* test parameter clear */
	for(bpos = 0; bpos < 8; ++bpos) { /* bit table read loop */
	    u_int8_t rch;		/* character readed */
	    rch = ucrp_key_prm(		/* key field function */
		ucrp_bts_table_base + dspos * 8, /* bit number calculate */
		8, 			/* field length */
		-1			/* read command */
	    );
	    if(rch >= 8) {		/* character readed check */
	    	rch = 0;		/* -> 0 */
		rvl = 0;		/* return code clear */
	    }
	    if((tval & (1 << rch)) > 0) {	
		rvl = 0;		/* internal error */
	    } else {
		tval |= (1 << rch);	/* set bit */
	    }
	    bttb[dspos] = rch;		/* bit table write */
	    dspos = ((dspos + 1) & 0x7); /* destination position next */
	}
	if(rvl == 0) {
	    ucrp_error(UMSG_XOR_ERR);	/* error message print */
	}
	u_int8_t ttab[256];		/* translate table */
	int16_t tpos;			/* translate table position */
	for(tpos = 0; tpos < 256; ++tpos) { /* translate table init loop */
	    u_int8_t tres = 0;		/* translated byte */
	    for(bpos = 0; bpos < 8; ++bpos) { /* bit table loop */
		if( (tpos & (1 << bttb[bpos])) ) { /* if bit seted in table index */
		    tres |= (1 << bpos);/* set bit in result */
		}
	    }
	    *(ttab + tpos) = tres;	/* translate table write */
	}
	int32_t pos;
	for(pos = 0; pos < len; ++pos) {/* xor loop */
	    *(dst + pos) = (*(src + pos) ^ xor_prm); /* encrypt/decrypt byte */
	    xor_prm = ttab[ xor_prm ];	/* xor mask modify */
	}
    } else {				/* unknown law */
	int32_t pos;
	for(pos = 0; pos < len; ++pos) { /* xor loop */
	    *(dst + pos) = (*(src + pos) ^ xor_prm); /* encrypt/decrypt byte */
	}
    }
 } else {				/* XOR law null */
    int32_t pos;			/* buffer position */
    for(pos = 0; pos < len; ++pos) {	/* xor loop */
	*(dst + pos) = (*(src + pos) ^ xor_prm); /* encrypt byte */
    }
 }
 if(rvl == 0) {				/* return code cheak */
    return(-1); 			/* return error */
 } else {
    return(len);			/* bytes encryted */
 }
}
#endif						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
int32_t ucrp_xre_en(u_int8_t * dst, u_int8_t * src, int len) /* xre encrypt */
{
 if(					/* input parametes check */
    (dst == 0) ||			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);
 }
 u_int8_t xor_prm;			/* xor mask */
 u_int8_t xrlw;				/* xor mask change law */
 u_int8_t xrpr;				/* xor mask change law parameter */
 int16_t rvl;				/* return code */
 rvl = 1;				/* return code start value */
 xor_prm =				/* xor parameter calculate */
    ucrp_key_prm(			/* key field function */
	UCRP_XOR_BASE,			/* xor parameter key buffer bit number */
	UCRP_XOR_LEN, 			/* xor parameter field length */
        -1				/* read command */
    ) + ucrp_passwd[0];			/* + password 0 character */
 xrlw = 				/* xor law read */
    ucrp_key_prm(			/* key field function */
	UCRP_XRL_BASE,			/* xor law bit number */
	UCRP_XRL_LEN,			/* xor law field length */
	-1				/* read command */
    );
 xrpr =					/* xor law parameter read */
    ucrp_key_prm(			/* key field function */
	UCRP_XRL_PRM_BASE,		/* xor law parameter bit number */
	UCRP_XRL_PRM_LEN,		/* xor law parameter field length */
	-1				/* read command */
    );
 xrlw += (ucrp_passwd[0] >> 3);		/* xor law parameter change modify with password 0  */
 xrlw = (xrlw % (UCRP_LAW_MAX + 1));	/* xor law parameter correct */
 if(xrlw != UCRP_LAW_NULL) {		/* xor law check */
    if(xrlw == UCRP_LAW_AD3) {		/* law add check */
        if(ucrp_comp_version() >= 118) { /* version check */
	    xrpr = ((xrpr + ucrp_passwd[0]) & 0xFF); /* law parameter modify with password 0 */
	} else {
	    xrpr = ((xrpr + ucrp_passwd[0]) & 0x7);
	}	
	int32_t pos;			/* buffer position */
	for(pos = 0; pos < len; ++pos) { /* buffer loop */
	    *(dst + pos) = (*(src + pos) ^ xor_prm); /* encrypt/decrypt byte */
	    xor_prm += xrpr;		/* xor mask modify */
	}
    } else 
    if(xrlw == UCRP_LAW_ROT) {		/* law rotate check */
	xrpr = ((xrpr + ucrp_passwd[0]) & 0x7); /* rotate parameter modify with password 0 */
	int32_t pos;			/* buffer position */
	for(pos = 0; pos < len; ++pos) { /* buffer loop */
	    *(dst + pos) = (*(src + pos) ^ xor_prm); /* encrypt/decrypt byte */
	    if(ucrp_comp_version() >= 118) { /* version check */
		xor_prm = (		/* xor mask modify */
		    (xor_prm << xrpr) | /* shift left */
		    (xor_prm >> (8 - xrpr))); /* shift back */
	    } else {
		xor_prm = 		/* xor mask modify */
		    (xor_prm >> xrpr) | /* shift right */
		    (xor_prm << (8 - xrpr)); /* shift back */
	    }
	}
    } else 
    if(xrlw == ucrp_law_tab) {		/* law tab check */
	if(ucrp_comp_version() >= 118) { /* version check */
	    xrpr = ((xrpr + ucrp_passwd[0]) & 0xFF); /* xor law parameter modify with password 0 */
	} else {
	    xrpr = ((xrpr + ucrp_passwd[0]) & 0x7); /* xor law parameter modify with password 0 */
	}
	u_int8_t ttab[256];		/* translate table */
	int32_t srpos;			/* source position */
	int32_t dspos;			/* destination position */
	dspos = xrpr;			/* destination position init */
	for(srpos = 0; srpos < 256; ++srpos) { /* translate table read */
	    if(ucrp_comp_version() >= 118) { /* version check */
	        ttab[dspos] = 		/* table read */
		    ucrp_key_prm(	/* key field function */
			(ucrp_table_base + (srpos * 8)),  /* bit number */
			8, 		/* field length */
			-1		/* read command */
		    );
	    } else {
	        ttab[srpos] = 		/* table read */
		    ucrp_key_prm(	/* key field function */
			(ucrp_table_base + (dspos * 8)), /* bit number */
			8,		/* field length */
			-1		/* read command */
		    );
	    }
	    dspos += 1;			/* position in translate table next */
	    if(dspos >= 256) {		/* destination position check */
		dspos = 0;		/* destination position start */
	    }
	}
	int32_t pos;			/* buffer position */
	for(pos = 0; pos < len; ++pos) { /* buffer loop */
	    *(dst + pos) = (*(src + pos) ^ xor_prm); /* encrypt/decrypt byte */
	    xor_prm = ttab[ (xor_prm & 0xFF) ]; /* xor mask modify */
	}
    } else 
    if(xrlw == ucrp_law_bts) {		/* bts law */
	xrpr = ((xrpr + ucrp_passwd[0]) & 0x7); /* xor parameter law modify */
	u_int8_t bttb[8];		/* bit swapping table */
	int16_t dspos;			/* destination position */
	int16_t btpos;			/* bit number */
	u_int32_t tval;			/* test parameter */
	dspos = xrpr;			/* destination position init */
	if((dspos < 0) || (dspos >= 8)) { /* destination position check */
	    dspos = 0;			/* destination position start */
	}
	tval = 0;			/* test parameter init */
	for(btpos = 0; btpos < 8; ++btpos) { /* bit loop */
	    u_int8_t rch;		/* character readed */
	    if(ucrp_comp_version() >= 118) { /* version check */
		rch = 			/* bts table item read */
		    ucrp_key_prm(	/* key field function */
			ucrp_bts_table_base + btpos * 8, /* bit number calculate */
			8, 		/* field length */
			-1		/* read command */
		    );
	    } else {
		rch =			/* bts table item read */
		    ucrp_key_prm(	/* key field function */
			ucrp_bts_table_base + dspos * 8, /* bit number calculate */
			8, 		/* field length */
			-1		/* read command */
		    );
	    }
	    if(rch >= 8) {		/* character readed check */
	    	rch = 0;		/* 0 - 7 */
		rvl = 0;		/* return code clear */
	    }
	    if((tval & (1 << rch)) > 0) { /* error check */
		rvl = 0;		/* return code clear */
	    } else {
		tval |= (1 << rch);	/* bit set */
	    }
	    bttb[dspos] = rch;		/* bit swapping table write */
	    dspos = ((dspos + 1) & 0x7); /* bit swapping table next position */
	}
	if(rvl == 0) {			/* return code check */
	    ucrp_error(UMSG_XOR_ERR);	/* error message print */
	}
	u_int8_t ttab[256];		/* translate table */
	int32_t pos;			/* buffer position */
	for(pos = 0; pos < 256; ++pos) {/* translate table form loop */
	    u_int8_t tres = 0;		/* translate item */
	    for(btpos = 0; btpos < 8; ++btpos) { /* bit table loop */
		if((pos & (1 << bttb[btpos]))) { /* if bit set in index */
		    tres |= (1 << btpos); /* set in translate item */
		}
	    }
	    *(ttab + pos) = tres;	/* translate table write */
	}
	for(pos = 0; pos < len; ++pos) { /* translate loop */
	    *(dst + pos) = (*(src + pos) ^ xor_prm); /* encrypt/decrypt byte */
	    xor_prm = ttab[ xor_prm ];	/* xor mask modify */
	}
    } else {				/* unknown law */
	int32_t pos;			/* buffer position */
	for(pos = 0; pos < len; ++pos) {  /* buffer loop */
	    *(dst + pos) = (*(src + pos) ^ xor_prm); /* encrypt/decrypt byte */
	}
    }
 } else {				/* law null */
    int32_t pos;			/* buffer position */
    for(pos = 0; pos < len; ++pos) {	/* buffer loop */
	*(dst + pos) = (*(src + pos) ^ xor_prm); /* encrypt/decrypt byte */
    }
 }
 if(rvl == 0) {				/* return code check */
    return(-1);				/* error return */
 } else {
    return(len);			/* bytes encrypted/decrypted */
 }
}
#endif						/* __UCRP_XRE_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* add enable macro check */	
int ucrp_add_en(u_int8_t * dst, u_int8_t * src, int32_t len) /* add encrypt */
{
 if(					/* input parameters check */
    (dst == 0) ||			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);				/* error return */
 }
 u_int8_t add_prm;			/* add parameter */
 add_prm = 				/* add parameter calculate */
    ucrp_key_prm(			/* key field function */
	UCRP_ADD_BASE,			/* add parameter key buffer bit number */
	UCRP_ADD_LEN,			/* add parameter field length */
	-1				/* read command */
    ) + ucrp_passwd[1];			/* + passwd 1 character */
 int32_t pos;				/* buffer position */
 for(pos = 0; pos < len; ++pos) {	/* buffer loop */
    u_int8_t rch;			/* character readed */
    rch = *(src + pos);			/* character read */
    rch = rch + add_prm;		/* add coding */
    *(dst + pos) = rch;			/* character write */
    if(ucrp_comp_version() >= 118) {	/* version check */
    } else {
	add_prm += 1;			/* add parameter modify */
    }
 }
 return(len);
}
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* add enable macro check */
int ucrp_add_de(u_int8_t * dst, u_int8_t * src, int32_t len) /* add decrypt */
{
 if(					/* input parametes check */
    (dst == 0) ||			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);				/* return error */
 }
 u_int8_t add_prm;			/* add parameter */
 add_prm = 				/* add parameter calculate */
    ucrp_key_prm(			/* key field function */
	UCRP_ADD_BASE,			/* add parameter key buffer bit number */
	UCRP_ADD_LEN,			/* add parameter field length */
	-1				/* read command */
    ) + ucrp_passwd[1];			/* + password 1 character */
 int32_t pos;				/* buffer position */
 for(pos = 0; pos < len; ++pos) {	/* buffer loop */
    u_int8_t rch;			/* character readed */
    rch = *(src + pos);			/* character read */
    rch = (256 + rch) - add_prm;	/* add decrypt */
    *(dst + pos) = rch;			/* destination buffer write */
    if(ucrp_comp_version() >= 118) {	/* version check */
    } else {
	add_prm += 1;
    }
 }
 return(len);				/* bytes decrypted return */
}
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
int ucrp_rot_en(u_int8_t * dst, u_int8_t * src, int32_t len) /* rotation encrypt */
{
 u_int8_t rot_prm;			/* rotate parameter */
 rot_prm = (				/* rotate parameter calculate */
    (ucrp_key_prm(			/* key field function */
	UCRP_ROT_BASE,			/* rot parameter key buffer bit number */
	UCRP_ROT_LEN,			/* rot parameter field length */
	-1				/* read command */
	) + ucrp_passwd[2]		/* + password 2 character */
    ) & 0x7);				/* norm */
 int32_t pos;				/* buffer position */
 for(pos = 0; pos < len; ++pos) {	/* buffer loop */
    u_int8_t rch;			/* character readed */
    rch = *(src + pos);			/* character read */
    if(ucrp_comp_version() >= 118) {	/* version check */
	rch = (				/* v.1.18 character encrypt */
	    (rch << rot_prm) | 		/* shift left */
	    (rch >> (8 - rot_prm))	/* back shift */
	);
    } else {
	rch = (				/* right rotate */
	    (rch >> rot_prm) | 		/* shift right */
	    (rch << (8 - rot_prm))	/* bacvk shift */
	);
    }
    *(dst + pos) = rch;			/* character write */
 }
 return(len);				/* encrypted bytes */
}
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
int ucrp_rot_de(u_int8_t * dst, u_int8_t * src, int32_t len)
{
 u_int8_t rot_prm;			/* rotate parameter */
 rot_prm = (				/* rotate parameter calculate */
    (ucrp_key_prm(			/* key field function */
	UCRP_ROT_BASE,			/* rot parameter key buffer bit number */
	UCRP_ROT_LEN,			/* rot parameter field length */
	-1				/* read command */
	) + ucrp_passwd[2]		/* + password 2 character */
    ) & 0x7);				/* norm */
 int32_t pos;				/* buffer position */
 for(pos = 0; pos < len; ++pos) {	/* buffer loop */
    u_int8_t rch;			/* character readed */
    rch = *(src + pos);			/* character read */
    if(ucrp_comp_version() >= 118) {	/* version check */
	rch = (				/* character rotate right */
	    (rch >> rot_prm) | 		/* shift right */
	    (rch << (8 - rot_prm))	/* shift back */
	);
    } else {
	rch = (				/* character rotate left */
	    (rch << rot_prm) | 		/* shift left */
	    (rch >> (8 - rot_prm))	/* shift back */
	);
    }
    *(dst + pos) = rch;
 }
 return(len);
}
#endif						/* __UCRP_ROT_ENABLE__ */
int16_t ucrp_bts_init(	/* bts table init, en - encrypt, pas - use password */
u_int8_t * ttab, u_int8_t en, u_int8_t pas)
{
 u_int8_t bttb[8];			/* bit swapping table */
 u_int8_t tval;				/* test parameter */
 int16_t btpos;				/* bit swapping table position */
 int16_t dspos;				/* destination position */
 tval = 0;				/* test parameter clear */
 if(ucrp_comp_version() >= 118) {	/* version check */
    if(pas > 0) { 			/* password enable check */
	dspos = ((			/* bts table offset calculate */
	    ucrp_key_prm(		/* key field function */
		ucrp_bts_base,		/* bts parameter bit number */
		UCRP_BTS_LEN,		/* bts parameter field length */
		-1			/* read command */
	    ) + ucrp_passwd[3]) & 0x7);	/* + password character 3 */
    } else {				/* => don't use password */
	dspos = (			/* bts table offset calculate */
	    ucrp_key_prm(		/* key field function */
		ucrp_bts_base,		/* bts parameter bit number */
		UCRP_BTS_LEN,		/* bts parameter field length */
		-1			/* read command */
	    ) & 0x7			/* bit offset norm */
	);
    }
 } else {
    dspos = (ucrp_passwd[3] & 0x7);	/* v. 1.0 table offset */
 }
 for(btpos = 0; btpos < 8; ++btpos) {	/* bit swapping table read loop */
    u_int8_t rch;			/* character readed */
    if(ucrp_comp_version() >= 118) {	/* version check */
	rch = 				/* bts table item read */	
	    ucrp_key_prm(		/* key field function */
		ucrp_bts_table_base + btpos * 8, /* bit number calculate */
		8, 			/* field length */
		-1			/* read command */
	    );
    } else {
	rch =				/* bts table item read  */	
	    ucrp_key_prm(		/* key field function */
		ucrp_bts_table_base + dspos * 8, /* bit number calculate */
		8, 			/* field length */
		-1			/* read command */
	    );
    }
    if(rch >= 8) {			/* bit number error */
	return(0);			/* translate table form error */
    }
    if((tval & (1 << rch)) > 0) {	/* bit check */
	return(0);			/* translate table form error */
    } else {
	tval |= (1 << rch);		/* bit set */
    }
    bttb[dspos] = rch;			/* character write */
    dspos = ((dspos + 1) & 0x7);	/* destination position next */
 }
 int16_t trpos;				/* translate table position */
 for(trpos = 0; trpos < 256; ++trpos) {	
    u_int8_t tres = 0;			/* translate charater */
    for(btpos = 0; btpos < 8; ++btpos) { /* bit loop */
	if((trpos & (1 << bttb[btpos]))) { /* if bit bttb[btpos] set in translate table index */
	    tres |= (1 << btpos);	/* set bit in result */
	}
    }
    *(ttab + trpos) = tres;		/* translate table write */
 }
#if __UCRP_BTS_TABLE_DEBUG__		/* bts table debug macro check */
 ucrp_printf(UMSG_CRNL);		/* <cr><nl> print */
 ucrp_printf(UMSG_BTTB);		/* bit table title */
 for(trpos = 0; ; ++trpos) {		/* bit table loop */
    ucrp_printf(			/* bit table item print */
	UMSG_02X,			/* hex 2 character width format */
	(u_int8_t) bttb[trpos]		/* bit table item */
    );
    if(trpos >= 7) {			/* last item check */
	{ break; }			/* loop exit */
    }
    ucrp_printf(			/* table items separator print */
        UMSG_SPACE			/* space format */
    );
 }
 ucrp_printf(UMSG_CRNL);		/* <cr><nl> print */
 for(trpos = 0; ; ++trpos) {		/* translate table print */
    ucrp_printf(
	UMSG_02X,			/* hex 2 character width format */
	(u_char) *(ttab + trpos)
    );
    if(trpos >= 255) {			/* translate table index check */
	{ break; }			/* loop exit */
    }
    if(((trpos + 1) & 0xF) == 0) {
	ucrp_printf(UMSG_CRNL);		/* <cr><nl> print */
    } else {
	ucrp_printf(			/* table item separator print */
	    UMSG_SPACE			/* space format */
	);
    }
 }
#endif					/* __UCRP_BTS_TABLE_DEBUG__ */
 if(en == 1) {				/* encrypt table form check */
    return(1);
 }
 u_int8_t rvtab[256];			/* reverse table */
 for(trpos = 0; trpos < 256; ++trpos) { /* table loop */
    u_int8_t rch;			/* character readed */
    rch = (ttab[trpos] & 0xFF);		/* character read from translate table */
    rvtab[rch] = trpos;			/* reverse table write */
 }
 for(trpos = 0; trpos < 256; ++trpos) {	/* copy loop */
    *(ttab + trpos) = rvtab[trpos];	/* reverce table copy */
 }
#if __UCRP_BTS_TABLE_DEBUG__		/* bts table debug check */
 ucrp_printf(UMSG_CRNL);		/* <cr><nl> print */
 ucrp_printf(UMSG_RTAB);		/* reverse table string */
 for(trpos = 0; ; ++trpos) {		/* reverse table print loop */
    ucrp_printf(			/* translate table print */
	UMSG_02X,			/* hex 2 character width format */
	(u_int8_t) *(ttab + tpos)	/* translate table item */
    );
    if(trpos >= 255) {			/* translate table index check */
	{ break; }			/* loop exit */
    }
    if(((trpos + 1) & 0xF) == 0) {
	ucrp_printf(UMSG_CRNL);		/* <cr><nl> print */
    } else {
	ucrp_printf(
	    UMSG_SPACE			/* space format */
	);
    }
 }
#endif					/* __UCRP_BTS_TABLE_DEBUG__ */
 return(1);
}
int ucrp_bts_en(u_int8_t * dst, u_int8_t * src, int32_t len) /* bts encrypt */
{
 if(					/* input parameters check */
    (dst == 0) || 			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);
 }
 u_int8_t ttab[256];			/* translate table */
 if(ucrp_bts_init(			/* translate table init */
    ttab,				/* translate table */
    1,					/* bts encrypt */
    1					/* use password */
 ) == 0) {				/* translate table init return code check */
    return(0);				/* return error */
 }
 int32_t bpos;				/* buffer position */
 for(bpos = 0; bpos < len; ++bpos) {	/* buffer loop */
    u_int8_t rch;			/* character readed */
    rch = *(src + bpos);		/* character read */
    rch = ttab[rch];			/* character translate */
    *(dst + bpos) = rch;		/* destination buffer write */
 }
 return(len);
}
int ucrp_bts_de(u_int8_t * dst, u_int8_t * src, int32_t len) /* bts decrypt */
{
 if(					/* source and destination buffer check */
    (dst == 0) || 			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);
 }
 u_int8_t ttab[256];			/* translate table */
 if(ucrp_bts_init(			/* translate table init */
    ttab, 				/* translate table */
    0, 					/* decrypt */
    1					/* password use */
 ) == 0) {				/* translate table init return code check */
    return(0);				/* return error */
 }
 int32_t bpos;				/* buffer position */
 for(bpos = 0; bpos < len; ++bpos) {	/* buffer position loop */
    u_int8_t rch;			/* character readed */
    rch = *(src + bpos);		/* character read */
    rch = ttab[rch];			/* character translate */
    *(dst + bpos) = rch;		/* character write */
 }
 return(len);				/* bytes decrypted */
}
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
int ucrp_tab_en(u_int8_t * dst, u_int8_t * src, int32_t len) /* table encrypt */
{
 u_int8_t tab[256];			/* primary table */
 u_int16_t dspos;			/* table destination position */
 if(ucrp_comp_version() >= 118) {	/* compatibility version check */
    dspos =				/* table offset calculate */
	ucrp_key_prm(			/* key field function */
	    ucrp_tab_base,		/* table offset field bit number */
	    UCRP_TAB_LEN,		/* table offset field length */
	    -1				/* read command */
	);
    if(ucrp_comp_version() >= 130) {	/* version 1.30 and up check */
	if(ucrp_passwd[0] != '\0') {	/* don't empty password check */
	    dspos = (dspos + ucrp_passwd[4]) & 0xFF; /* + password character 4 */
	}
    } else {				/* version less then 1.30 */
	dspos = (dspos + ucrp_passwd[4]) & 0xFF; /* => don't check password */
    }
 } else {
    dspos = 0;				/* destination position clear */
 } 
 int32_t pos;				/* table/buffer position */
 for(pos = 0; pos < 256; ++pos) {	/* table read loop */
    tab[dspos] = 			/* table read */
	ucrp_key_prm(			/* key field function */
	    (ucrp_table_base + pos * 8), /* bit number */
	    8,				/* field length */
	    -1				/* read command */
	);
    dspos += 1;				/* destination position next */
    if(dspos >= 256) {			/* destination position out of buffer check */
	dspos = 0;			/* loop exit */
    }
 }
 u_int8_t xor_comp;			/* xor compatibility */
 xor_comp = ucrp_comp(ucrp_xor_comp,-1);/* xor compatibility read */
 if(ucrp_comp_version() >= 130) {	/* compatibility version 1.30 check */
    u_int8_t xre_comp;			/* xre compatibility */
    xre_comp = ucrp_comp(ucrp_xre_comp,-1); /* xre compatibility read */
    if(xre_comp > 0) {			/* xre compatibility enable check */
	xor_comp = 1; 			/* xor compatibility set */
    }
 }
 u_int8_t xor_prm;			/* xor parameter */
 xor_prm = 				/* xor parameter read */
    ucrp_key_prm(			/* key field function */
	UCRP_XOR_BASE,			/* xor parameter key buffer bit number */
	UCRP_XOR_LEN,			/* xor parameter field length */
	-1				/* read command */
    );
 u_int8_t add_comp;			/* add compatibility */
 u_int8_t add_prm;			/* add parameter */
 add_comp = ucrp_comp(ucrp_add_comp,-1);/* add compatibility read */
 add_prm =				/* add parameter read */
    ucrp_key_prm(			/* key field function */
	UCRP_ADD_BASE,			/* add parameter key buffer bit number */
	UCRP_ADD_LEN,			/* add parameter field length */
	-1				/* read command */
    );					/* norm */
 u_int8_t rot_comp;			/* rotate compatibility */
 u_int8_t rot_prm;			/* rotate parameter */
 rot_comp = ucrp_comp(ucrp_rot_comp,-1);/* rotate compatibility read */
 rot_prm = (				/* rotate parameter calculate */
    ucrp_key_prm(			/* key field function */
	UCRP_ROT_BASE,			/* rot parameter key buffer bit number */
	UCRP_ROT_LEN,			/* rot parameter field length */
	-1				/* read command */
    ) & 7);				/* norm */
 u_int8_t bts_comp;			/* bit swapping compatibility */
 bts_comp = ucrp_comp(ucrp_bts_comp,-1);/* bit swapping compatibility read */
 u_int8_t trtb[256];			/* bts translate table */
 if(ucrp_bts_init(			/* bts translate table init */
    trtb,				/* translate table */
    1,					/* encrypt table generate */
    0					/* don't use password */
 ) == 0) {				/* translate table init return code check */
    return(0);				/* function return */
 }
 for(pos = 0; pos < len; ++pos) {	/* copy loop */
    *(dst + pos) = *(src + pos);	/* source buffer copy to destination buffer */
 }
 int16_t hlen;				/* hash length */
 if(ucrp_comp_version() >= 118) {	/* compatibility check */
    hlen = 256;				/* 256 bytes */
 } else {
    hlen = 64;				/* 64 bytes */
 }
 int32_t hnum;				/* hash byte number */
 for(hnum = 0; hnum < hlen; ++hnum) {	/* hash loop */
    u_int8_t hch;    			/* hash character */
#ifdef __UCRP_TAB_DEBUG__		/* table encrypt/decrypt debug macro check */
    {
	ucrp_printf(UMSG_TAB_EN, hnum);	/* hash byte number print */
	for(pos = 0; ; ++pos) {		/* table loop */
	    ucrp_printf(		/* table item print */
		UMSG_02X,		/* hex 2 character width format */
		(u_int8_t)tab[pos]	/* table item */
	    );
	    if(pos >= 255) {		/* table index out of buffer check */
		{ break; }		/* -> loop exit */
	    }
	    if(((pos + 1) & 0xF) == 0) { /* table rows separator print check */
		ucrp_printf(UMSG_CRNL);	/* <CR><NL> print */
	    } else {
		ucrp_printf(		/* space separator print */
		    UMSG_SPACE		/* space format */
		);
	    }
	}	
	ucrp_printf(UMSG_CRNL);		/* <CR><NL> print */
    }
#endif 						/* __UCRP_TAB_DEBUG__ */
    for(pos = 0; pos < len; ++pos) {	/* encrypt loop */
	u_int8_t rch;			/* character readed */
	rch = *(dst + pos);		/* character read */
	rch = tab[rch];			/* table encryption */
	*(dst + pos) = rch;		/* destination buffer write */
    }
    if(ucrp_comp_version() >= 118) {	/* compatibility version 1.18 and up check */
	hch = (ucrp_passwd[hnum] >> 4);	/* top half byte read */
    } else {
	hch = ucrp_passwd[hnum];	/* bottom half byte read */
    }
    if(					/* xor encryption check */ 
    	(ucrp_comp_version() < 118) ||	/* version less then 1.18 check or */
	(xor_comp > 0) 			/* xor or xre compatibility enabled */
    ) {
	if((hch & 1) != 0) {		/* bit 0, mask 0x01 seted check */
	    for(pos = 0; pos < 256; ++pos) { /* table loop */
		tab[pos] = (tab[pos] ^ xor_prm); /* table byte xor */
	    }
	}
    }
    if(					/* add encryption check */
	(ucrp_comp_version() < 118) ||	/* version less then 1.18 check or */
	(add_comp > 0) 			/* add compatibility enabled check */
    ) {
	if((hch & 2) != 0) {		/* bit 1, mask 0x02 seted check */
	    for(pos = 0; pos < 256; ++pos) { /* table add encrypt loop */
		tab[pos] = (tab[pos] + add_prm); /* table item add encryption */
	    }
	}
    }
    if(					/* rotate encryption check */
	(ucrp_comp_version() < 118) ||	/* version less then 1.18 enable check */
	(rot_comp > 0) 			/* or rotate compatibility enable check */
    ) {
	if((hch & 4) != 0) {		/* bit 2, mask 0x4 compatibility check */
	    for(pos = 0; pos < 256; ++pos) { /* table rotate loop */
		if(ucrp_comp_version() >= 118) { /* version 1.18 and up check */
		    tab[pos] = 
			(tab[pos] << rot_prm) | /* shift left */
			(tab[pos] >> (8 - rot_prm)); /* shift back */
		} else {
		    tab[pos] = 
			(tab[pos] >> rot_prm) |  /* shift right */
			(tab[pos] << (8 - rot_prm)); /* shift back */
		}
	    }
	}
    }
    if(					/* bts table coding check */
	(ucrp_comp_version() >= 118) && /* v. 1.18 and up check AND (from 1.18 only) */
	(bts_comp > 0) 			/* bts compatibility enable check */
    ) {
	if((hch & 8) != 0) {		/* bit 3, mask 0x8 check */
	    for(pos = 0; pos < 256; ++pos) { /* bts table encrypt loop */
		tab[pos] = trtb[tab[pos]]; /* table encrypt */
	    }
	}
    }
#ifdef __UCRP_TAB_DEBUG__		/* dable encrypt debug macro check */
    {
	ucrp_printf(UMSG_TAB_EN, hnum); /* hash byte number print */
        for(pos = 0; ; ++pos) { 	/* table loop */
	    ucrp_printf(		/* table item print */
		UMSG_02X,		/* hex 2 character width format */
		tab[pos]		/* table item */
	    );
	    if(pos >= 255) {		/* table end item check */
		{ break; }		/* loop exit */
	    }
	    if(((pos + 1) & 0xF) == 0) { /* row end check */
		ucrp_printf(UMSG_CRNL);	/* row separator print */
	    } else {
		ucrp_printf(		/* space separator print */
		    UMSG_SPACE		/* space format */
		);
	    }
	}	
	ucrp_printf(UMSG_CRNL);		/* <CR><NL> print */
    }
#endif						/* __UCRP_TAB_DEBUG__ */
    for(pos = 0; pos < len; ++pos) {	/* buffer encrypt loop */
	u_int8_t rch;			/* character readed */
	rch = *(dst + pos);		/* character read */
	rch = tab[rch];			/* character encrypt */
	*(dst + pos) = rch;		/* character write */
    }
    if(ucrp_comp_version() >= 118) {	/* version up 1.18 check */
	hch = ucrp_passwd[hnum];	/* low half hash byte read */
    } else {
	hch = (ucrp_passwd[hnum] >> 4);	/* top half hash byte read */
    }
    if(					/* xor table encryption check */
	(ucrp_comp_version() < 118) ||	/* version less then 1.18 or */
	(xor_comp > 0) 			/* xor enabled or xre enabled */
    ) {
	if((hch & 1) != 0) {		/* bit 0, mask 0x1 check */
	    for(pos = 0; pos < 256; ++pos) { /* table xor loop */
		tab[pos] = (tab[pos] ^ xor_prm); /* table item xor encrypt */
	    }
	}
    }
    if(					/* table add encryption check */
	(ucrp_comp_version() < 118) ||	/* version less then 1.18 or */
	(add_comp > 0) 			/* add enabled check */
    ) {
	if((hch & 2) != 0) {		/* bit number 1, mask 0x2 check */
	    for(pos = 0; pos < 256; ++pos) { /* table add encrypt check */
		tab[pos] = tab[pos] + add_prm; /* table item add encrypt */
	    }
	}
    }
    if(					/* table rotate encryption check */
	(ucrp_comp_version() < 118) ||	/* compatibility version less then 1.18 check */
	(rot_comp > 0) 			/* or rotate compatibility enabled */
    ) {
	if((hch & 4) != 0) {		/* bit 2, mask 0x4 check */
	    for(pos = 0; pos < 256; ++pos) { /* table rotate loop */
		if(ucrp_comp_version() >= 118) { /* version 1.18 or up check */
	    	    tab[pos] = 
			(tab[pos] << rot_prm) | /* left shift */
			(tab[pos] >> (8 - rot_prm)); /* shift back */
		} else {
	    	    tab[pos] =
			(tab[pos] >> rot_prm) | /* right shift */
			(tab[pos] << (8 - rot_prm)); /* shift back */
		}
	    }
	}
    }
    if(					/* bts table encryption check */
	(ucrp_comp_version() >= 118) && /* version 1.18 or up check (from 1.18 only) */
	(bts_comp > 0) 			/* and bts compatibility enable check */
    ) {
	if((hch & 8) != 0) {		/* bit 3, mask 0x8 check */
	    for(pos = 0; pos < 256; ++pos) { /* table bts encryption loop */
		tab[pos] = trtb[tab[pos]]; /* table item encrypt */
	    }
	}
    }
 }
#ifdef __UCRP_TAB_DEBUG__		/* dable encrypt debug macro check */
 {
    ucrp_printf(UMSG_TAB_EN_END);	/* table title */
    for(pos = 0; ; ++pos) {		/* table print loop */
        ucrp_printf(			/* table item print */
    	    UMSG_02X,			/* hex 2 character width format */
    	    tab[pos]			/* table item */
	);
	if(pos >= 255) {		/* position end check */
	    { break; }			/* loop exit */
	}
	if(((pos + 1) & 0xF) == 0) {	/* row end check */
	    ucrp_printf(UMSG_CRNL);	/* <CR><NL> */
	} else {
	    ucrp_printf(		/* space separator print */
		UMSG_SPACE		/* space format */
	    );
	}
    }	
    ucrp_printf(UMSG_CRNL);		/* <CR><NL> */
 }
#endif						/* __UCRP_TAB_DEBUG__ */
 return(len);				/* bytes in destination buffer */
}
#endif						/* __UCRP_TAB_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
int ucrp_tab_de(u_int8_t * dst, u_int8_t * src, int32_t len) /* table decrypt */
{
 u_int8_t tab[256];			/* primary table */
 int32_t dspos;				/* destination table */
 if(ucrp_comp_version() >= 118) {	/* version 1.18 or up check */
    dspos =				/* table offset calculate */
	ucrp_key_prm(			/* key field function */
	    ucrp_tab_base,		/* table offset field bit number */
	    UCRP_TAB_LEN,		/* table offset field length */
	    -1				/* read command */
	);
    if(ucrp_comp_version() >= 130) {	/* version 1.30 and up check */
	if(ucrp_passwd[0] != '\0') {	/* don't empty password check */
	    dspos = (dspos + ucrp_passwd[4]) & 0xFF; /* + password character 4 */
	}
    } else {				/* version less then 1.30 */
	dspos = (dspos + ucrp_passwd[4]) & 0xFF; /* => don't check password */
    }
 } else {
    dspos = 0;				/* destination position clear */
 } 
 int32_t pos;				/* table/buffer position */
 for(pos = 0; pos < 256; ++pos) {	/* table read loop */
    tab[dspos] = 			/* table read */
	ucrp_key_prm(			/* key field function */
	    (ucrp_table_base + pos * 8), /* bit number */
	    8, 				/* field length */
	    -1				/* read command */
	);
    dspos += 1;				/* destination position next */
    if(dspos >= 256) {			/* position out of buffer check */
	dspos = 0;			/* positio start */
    }
 }
 u_int8_t xor_comp;			/* xor compatibility */
 u_int8_t xor_prm;			/* xor parameter */
 xor_comp = ucrp_comp(ucrp_xor_comp,-1); /* xor compatibility read */
 if(ucrp_comp_version() >= 130) {	/* version 1.30 or up check */
    u_int8_t xre_comp;			/* xre compatibility */
    xre_comp = ucrp_comp(ucrp_xre_comp,-1); /* xre compatibility read */
    if(xre_comp > 0) {			/* xre compatibility enabled check */
	xor_comp = 1;			/* xor compatibility set */
    } 
 }
 xor_prm = 				/* xor parameter read */
    ucrp_key_prm(			/* key field function */
	UCRP_XOR_BASE,			/* xor parameter key buffer bit number */
	UCRP_XOR_LEN,			/* xor parameter field length */
	-1				/* read command */
    );
 u_int8_t add_comp;			/* add compatibility */
 u_int8_t add_prm;			/* add parameter */
 add_comp = ucrp_comp(ucrp_add_comp,-1); /* add compatibility read */
 add_prm = 				/* add parameter read */
    ucrp_key_prm(			/* key field function */
	UCRP_ADD_BASE,			/* add parameter key buffer bit number */
	UCRP_ADD_LEN,			/* add parameter field length */
	-1				/* read command */
    );
 u_int8_t rot_comp;			/* rotate compatibility */
 u_int8_t rot_prm;			/* rotate parameter */
 rot_comp = ucrp_comp(ucrp_rot_comp,-1); /* rotate compatibility read */
 rot_prm = (				/* rotate parameter calculate */
    ucrp_key_prm(			/* key field function */
	UCRP_ROT_BASE,			/* rot parameter key buffer bit number */
	UCRP_ROT_LEN,			/* rot parameter field length */
	-1				/* read command */
    ) & 7);				/* rotate parameter norm */
 u_int8_t bts_comp;			/* bit swapping compatibility */
 bts_comp = ucrp_comp(ucrp_bts_comp,-1); /* bit swapping compatibility read */
 u_int8_t trtb[256];			/* bts translate table */
 if(ucrp_bts_init(			/* bts translate table init */
    trtb,				/* translate table */
    1,					/* encrypt */
    0					/* don't use password */
 ) == 0) {				/* bts table init return code check */
    return(0);				/* function return */
 }
 for(pos = 0; pos < len; ++pos) {	/* source buffer copy loop */
    *(dst + pos) = *(src + pos);	/* byte copy */
 }
 int32_t hlen;				/* hash length */
 if(ucrp_comp_version() >= 118) {	/* compatibility version 1.18 or up check */
    hlen = 256;				/* 256 bytes */
 } else {
    hlen = 64;				/* 64 bytes */
 }
 int32_t hnum;				/* hash byte number */
 for(hnum = 0; hnum < hlen; ++hnum) {	/* hash loop */
#ifdef __UCRP_TAB_DEBUG__ /* TODO */	/* table encrypt/decrypt debug macro check */
    {
	ucrp_printf(UMSG_TAB_EN, hnum);	/* hash byte number print */
	for(pos = 0; ; ++pos) {		/* table loop */
	    ucrp_printf(		/* table item print */
		UMSG_02X,		/* hex 2 character width format */
		(u_int8_t)tab[pos]	/* table item */
	    );
	    if(pos >= 255) {		/* table index out of buffer check */
		{ break; }		/* -> loop exit */
	    }
	    if(((pos + 1) & 0xF) == 0) { /* table rows separator print check */
		ucrp_printf(UMSG_CRNL);	/* <CR><NL> print */
	    } else {
		ucrp_printf(		/* space separator print */
		    UMSG_SPACE		/* space format */
		);
	    }
	}	
	ucrp_printf(UMSG_CRNL);		/* <CR><NL> print */
    }
#endif 						/* __UCRP_TAB_DEBUG__ */
    u_int8_t hch;    			/* hash character */
    if(ucrp_comp_version() >= 118) {	/* version 1.18 or up check */
	hch = (ucrp_passwd[hnum] >> 4);	/* upper half byte read */
    } else {
	hch = ucrp_passwd[hnum];	/* low half byte read */
    }
    if(					/* xor encrypt/decrypt check */
	(ucrp_comp_version() < 118) ||	/* version less then 1.18 check */
	(xor_comp > 0) 			/* or xor compatibility enabled */
    ) {
	if((hch & 1) != 0) {		/* bit 0, mask 0x1 check */
	    for(pos = 0; pos < 256; ++pos) { /* xor encrypt loop */
		tab[pos] = (tab[pos] ^ xor_prm); /* table item xor encrypt */
	    }
	}
    }
    if(					/* add encrypt check */
	(ucrp_comp_version() < 118) ||	/* version less then 1.18 check */
	(add_comp > 0) 			/* or add compatibility check */
    ) {
	if((hch & 2) != 0) {		/* bit 1, mask 0x2 check */
	    for(pos = 0; pos < 256; ++pos) { /* table add encrypt loop */
		tab[pos] = tab[pos] + add_prm; /* table item add encrypt */
	    }
	}
    }
    if(					/* rot encrypt check */
	(ucrp_comp_version() < 118) ||	/* version less then 1.18 check */
	(rot_comp > 0) 			/* or rotate compatibility enabled */
    ) {
	if((hch & 4) != 0) {		/* bit 2, mask 0x4 check */
	    for(pos = 0; pos < 256; ++pos) { /* table rotate encrypt */
		if(ucrp_comp_version() >= 118) { /* version 1.18 or up check */
		    tab[pos] = 
			(tab[pos] << rot_prm) | /* left shift */
			(tab[pos] >> (8 - rot_prm)); /* shift back */
		} else {
	    	    tab[pos] = 
			(tab[pos] >> rot_prm) | /* right shift */
			(tab[pos] << (8 - rot_prm)); /* shift back */
		}
	    }
	}
    }
    if(					/* bts encryption check */
	(ucrp_comp_version() >= 118) && /* version 1.18 up and (only from 1.18)*/
	(bts_comp > 0) 			/* bts enabled */
    ) {
	if((hch & 8) != 0) {		/* bit 3, mask 0x8 check */
	    for(pos = 0; pos < 256; ++pos) { /* bts encrypt loop */
		tab[pos] = trtb[tab[pos]]; /* bts encrypt item */
	    }
	}
    }
#ifdef __UCRP_TAB_DEBUG__ /* TODO */	/* table encrypt/decrypt debug macro check */
    {
	ucrp_printf(UMSG_TAB_EN, hnum);	/* hash byte number print */
	for(pos = 0; ; ++pos) {		/* table loop */
	    ucrp_printf(		/* table item print */
		UMSG_02X,		/* hex 2 character width format */
		(u_int8_t)tab[pos]	/* table item */
	    );
	    if(pos >= 255) {		/* table index out of buffer check */
		{ break; }		/* -> loop exit */
	    }
	    if(((pos + 1) & 0xF) == 0) { /* table rows separator print check */
		ucrp_printf(UMSG_CRNL);	/* <CR><NL> print */
	    } else {
		ucrp_printf(		/* space separator print */
		    UMSG_SPACE		/* space format */
		);
	    }
	}	
	ucrp_printf(UMSG_CRNL);		/* <CR><NL> print */
    }
#endif 						/* __UCRP_TAB_DEBUG__ */
    if(ucrp_comp_version() >= 118) {	/* version 1.18 and up check */
	hch = ucrp_passwd[hnum];	/* low hash half byte read  */
    } else {
	hch = (ucrp_passwd[hnum] >> 4);	/* upper hask half byte read */
    }
    if(					/* xor encrypt table check */
	(ucrp_comp_version() < 118) ||	/* version less then 1.18 check */
	(xor_comp > 0) 			/* or xor compatibility check */
    ) {
	if((hch & 1) != 0) {		/* bit 0, mask 0x1 check */
	    for(pos = 0; pos < 256; ++pos) { /* table xor encrypt loop */
		tab[pos] = (tab[pos] ^ xor_prm); /* table item xor encrypt */
	    }
	}
    }
    if(					/* table add encrypt check */
	(ucrp_comp_version() < 118) ||	/* version less then 1.18 check */
	(add_comp > 0) 			/* add compatibility enabled check */
    ) {
	if((hch & 2) != 0) {		/* bit 1, mask 0x2 check */
	    for(pos = 0; pos < 256; ++pos) { /* table add encryption */
		tab[pos] = tab[pos] + add_prm; /* table item add encryption */
    	    }
	}
    }
    if(					/* table rotate encryption check */
	(ucrp_comp_version() < 118) ||	/* version less then 1.18 check or */
	(rot_comp > 0) 			/* rotate compatibility check */
    ) {
	if((hch & 4) != 0) {		/* bit 2, mask 0x4 check */
	    for(pos = 0; pos < 256; ++pos) { /* table rotate encrypt */
		if(ucrp_comp_version() >= 118) { /* version 1.18 and up check */
		    tab[pos] = 
			(tab[pos] << rot_prm) | /* shift left */
			(tab[pos] >> (8 - rot_prm)); /* shift back */
		} else {
		    tab[pos] = 
			(tab[pos] >> rot_prm) | /* shift right */
			(tab[pos] << (8 - rot_prm)); /* shift back */
		}
	    }
	}
    }
    if(					/* bts encryption check */
	(ucrp_comp_version() >= 118) && /* version 1.18 and up and (from 1.18 only) */
	(bts_comp > 0) 			/* bts enabled check */
    ) {
	if((hch & 8) != 0) {		/* bit 3, mask 0x8 check */
	    for(pos = 0; pos < 256; ++pos) { /* bts encrypt loop */
		tab[pos] = trtb[tab[pos]]; /* table item bts encryption */
	    }
	}
    }
 }
#ifdef __UCRP_TAB_DEBUG__		/* table encrypt/decrypt check */
 {
    ucrp_printf(UMSG_TAB_DE_END);	/* end table print */
    for(pos = 0; ; ++pos) {		/* table print loop */
        ucrp_printf(			/* table item print */
    	    UMSG_02X,			/* hex 2 character width format */
	    tab[pos]			/* table item */
	);
	if(pos >= 255) {		/* table end item check */
	    { break; }			/* loop exit */
	}
	if(((pos + 1) & 0xF) == 0) { /* row end item check */
	    ucrp_printf(UMSG_CRNL);	/* <CR><NL> print */
	} else {
	    ucrp_printf(		/* space separator print */
	        UMSG_SPACE		/* space format */
	    );
	}
    }	
    ucrp_printf(UMSG_CRNL);		/* <CR><NL> */
 }
#endif						/* __UCRP_TAB_DEBUG__ */
 if(ucrp_bts_init(			/* bts decrypt table init */
    trtb,				/* translate table */
    0,					/* decrypt */
    0					/* don't use password */
 ) == 0) {				/* bts table init return code check */
    return(0);				/* function error */
 }
 for(hnum = hlen - 1; hnum >= 0; --hnum) { /* back hash loop */
    u_int8_t hch;    			/* hash character */
    if(ucrp_comp_version() >= 118) {	/* version 1.18 and up check */
	hch = ucrp_passwd[hnum];	/* low half byte read */
    } else {
	hch = (ucrp_passwd[hnum] >> 4); /* hi half byte read */
    }
    if(					/* bts decrypt check */
	(ucrp_comp_version() >= 118) &&	/* version 1.18 and up check and (only form 1.18) */
	(bts_comp > 0) 			/* bts compatibility check */
    ) {
	if((hch & 8) != 0) {		/* bit 3, mask 0x8 check */
	    for(pos = 0; pos < 256; ++pos) { /* bts decode loop */
		tab[pos] = trtb[tab[pos]]; /* table item decrypt */
	    }
	}
    }
    if(					/* table rotate decrypt check */
	(ucrp_comp_version() < 118) ||	/* version less then 118 */
	(rot_comp > 0) 			/* rotate compatibility enabled */
    ) {
	if((hch & 4) != 0) {		/* bit 2, mask 0x4 check */
	    for(pos = 0; pos < 256; ++pos) { /* table rotate decrypt loop */
		if(ucrp_comp_version() >= 118) { /* version 1.18 and up check */
		    tab[pos] = 
			(tab[pos] >> rot_prm) | /* shift right */
			(tab[pos] << (8 - rot_prm)); /* shift back */
		} else {
		    tab[pos] = 
			(tab[pos] << rot_prm) | /* shift left */
			(tab[pos] >> (8 - rot_prm)); /* shift back */
		}
	    }
	}
    }
    if(					/* table add decrypt check */
	(ucrp_comp_version() < 118) ||	/* version less then 1.18 check */
	(add_comp > 0) 			/* add compatibility check */
    ) {
	if((hch & 2) != 0) {		/* bit 1, mask 0x2 check */
	    for(pos = 0; pos < 256; ++pos) { /* table decrypt loop */
		tab[pos] = tab[pos] - add_prm; /* add decrypt */
	    }
	}
    }
    if(					/* table xor decrypt check */
	(ucrp_comp_version() < 118) ||	/* version les then 1.18 check */
	(xor_comp > 0) 			/* xor compatibility check */
    ) {
	if((hch & 1) != 0) {		/* bit 0, mask 0x1 check */
	    for(pos = 0; pos < 256; ++pos) { /* table loop */
		tab[pos] = (tab[pos] ^ xor_prm); /* xor decrypt */
	    }
	}
    }
#ifdef __UCRP_TAB_DEBUG__		/* table encrypt/decrypt debug macro check */
    {
	ucrp_printf(UMSG_TAB_DE, hnum);	/* table title & hash number check */
	for(pos = 0; ; ++pos) {		/* table print loop */
	    ucrp_printf(		/* table item print */
		UMSG_02X,		/* hex 2 character width format */
		tab[pos]		/* table item */
	    );
	    if(pos >= 255) {		/* position end check */
		{ break; }		/* loop exit */
	    } 
	    if(((pos + 1) & 0xF) == 0) { /* row end check */
		ucrp_printf(UMSG_CRNL); /* <CR><NL> print */
	    } else {
		ucrp_printf(		/* space separator print */
		    UMSG_SPACE		/* space format */
		);
	    }
	}	
	ucrp_printf(UMSG_CRNL);		/* <CR><NL> */
    }
#endif						/* __UCRP_TAB_DEBUG__ */
    u_int8_t rev[256];			/* reverse table */
    for(pos = 0; pos < 256; ++pos) {	/* reverse table form loop */
	u_int8_t rch;			/* character readed */
	rch = tab[pos];			/* character read */
        rev[rch] = pos;			/* position write */
    }
    for(pos = 0; pos < len; ++pos) {	/* decrypt loop */
	u_int8_t rch;			/* character readed */
	rch = *(dst + pos);		/* character read */
	rch = rev[rch];			/* character decrypt */
	*(dst + pos) = rch;		/* character write */
    }
    if(ucrp_comp_version() >= 118) {	/* version 1.18 and up check */
	hch = (ucrp_passwd[hnum] >> 4);	/* top half byte read */
    } else {
	hch = ucrp_passwd[hnum];	/* bottom half byte read */
    }
    if(					/* bts table decrypt check */
	(ucrp_comp_version() >= 118) &&	/* version 1.18 and up check and (from 1.18 only) */
	(bts_comp > 0) 			/* bts compatibility enable */
    ) {
	if((hch & 8) != 0) {		/* bit 3, mask 0x8 check */
	    for(pos = 0; pos < 256; ++pos) { /* bts decrypt */
		tab[pos] = trtb[tab[pos]]; /* table item decrypt */
	    }
	}
    }
    if(					/* table rotate decrypt check */
	(ucrp_comp_version() < 118) || 	/* version less then 1.18 check */
	(rot_comp > 0) 			/* rotate compatibility check */
    ) {
	if((hch & 4) != 0) {		/* bit 2, mask 0x4 check */
	    for(pos = 0; pos < 256; ++pos) { /* table rotate decrypt loop */
		if(ucrp_comp_version() >= 118) { /* version 1.18 and up check */
		    tab[pos] = 
			(tab[pos] >> rot_prm) | /* shift right */
			(tab[pos] << (8 - rot_prm)); /* shift back */
		} else {
		    tab[pos] = 
			(tab[pos] << rot_prm) | /* shift left */
			(tab[pos] >> (8 - rot_prm)); /* shift back */
		}
	    }
	}
    }
    if(					/* table add decrypt check */
	(ucrp_comp_version() < 118) ||	/* version less then 1.18 check */
	(add_comp > 0) 			/* add compatibility check */
    ) {
	if((hch & 2) != 0) {		/* bit 1, mask 2 check */
	    for(pos = 0; pos < 256; ++pos) { /* table add decrypt */
		tab[pos] = tab[pos] - add_prm; /* table item decrypt */
	    }
	}
    }
    if(					/* table xor decrypt check */
	(ucrp_comp_version() < 118) ||	/* version less then 1.18 check */
	(xor_comp > 0) 			/* xor compatibility check */
    ) {
	if((hch & 1) != 0) {		/* bit 0, mask 0x1 check */
	    for(pos = 0; pos < 256; ++pos) { /* table xor decrypt */
		tab[pos] = (tab[pos] ^ xor_prm); /* table item decrypt */
	    }
	}
    }
#ifdef __UCRP_TAB_DEBUG__
    {
	ucrp_printf(UMSG_TAB_DE_END);	/* table title print */
	for(pos = 0; ; ++pos) {		/* table loop */
	    ucrp_printf(		/* table item print */
		UMSG_02X,		/* hex 2 character width format */
		tab[pos]		/* table item */
	    );
	    if(pos >= 255) {		/* table item end check */
		{ break; }		/* loop exit */
	    }
	    if(((pos + 1) & 0xF) == 0) { /* row end check */
		ucrp_printf(UMSG_CRNL);	/* <CR><NL> print */
	    } else {
		ucrp_printf(		/* space separator print */
		    UMSG_SPACE		/* space */
		);
	    }
	}	
	ucrp_printf(UMSG_CRNL);		/* <CR><NL> */
    }
#endif						/* __UCRP_TAB_DEBUG__ */
    for(pos = 0; pos < 256; ++pos) {	/* reverse table form loop */
	u_int8_t rch;			/* character readed */
	rch = tab[pos];			/* character read */
        rev[rch] = pos;			/* position write */
    }
    for(pos = 0; pos < len; ++pos) {	/* decrypt loop */
	u_int8_t rch;			/* character readed */
	rch = *(dst + pos);		/* character read */
	rch = rev[rch];			/* character decrypt */
	*(dst + pos) = rch;		/* character write */
    }
 }
 return(len);
}
#endif						/* __UCRP_TAB_ENABLE__ */
extern void nand_calculate_ecc(const u_char * dat, u_char * ecc_code);
extern int nand_correct_data(u_char *dat, u_char * read_ecc, u_char *calc_ecc);
#ifdef __UCRP_ECC_ENABLE__		/* ECC enable macro check */
/* #define NO_ECC */
int ucrp_ecc_en(u_int8_t * dst, u_int8_t * src, int32_t len)
{
 if(					/* input parameters check */
    (dst == 0) ||			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);				/* function return */
 }
#ifdef NO_ECC				/* no ecc macro defined check */
 memcpy(dst,src,len);			/* source buffer copy to destination buffer */
 return(len);				/* bytes in destination buffer return */
#else					/* else NO_ECC */
 u_int8_t ecc[3];			/* ecc calculated */
 int32_t dtpos;				/* destination position */
 int32_t srpos;				/* source position */
 dtpos = 0;				/* destination position start */
 for(srpos = 0; srpos < len;) {		/* source buffer loop */
    int32_t cnt;			/* bytes processed */
    if(srpos + 256 <= len) {		/* out of buffer check */
	nand_calculate_ecc(src + srpos, ecc); /* ecc calculate */
	cnt = 256;			/* bytes processed */
    } else {				/* partitional block */
	cnt = len - srpos;		/* block size calculate */
	u_int8_t buf[256];		/* temp buffer */
	int32_t pos;			/* buffer position */
	for(pos = 0; pos < cnt; ++pos) { /* copy loop */
	    buf[pos] = *(src + srpos + pos); /* byte copy */
	}
	while(pos < 256) {		/* tail fill loop */
	    buf[pos++] = 0xFF;		/* character set */
	}
	nand_calculate_ecc(buf, ecc);	/* calculate ecc */
    }
    int32_t pos;			/* buffer position */
    for(pos = 0; pos < cnt; ++pos) {	/* copy loop */
	*(dst + dtpos) = *(src + srpos); /* byte copy */
	 dtpos += 1;			/* destination position next */
  	 srpos += 1;			/* source position next */
    }
    *(dst + dtpos++) = ecc[0];		/* ecc byte 0 copy to destination buffer */
    *(dst + dtpos++) = ecc[1];		/* ecc byte 1 copy to destination buffer */
    *(dst + dtpos++) = ecc[2];		/* ecc byte 2 copy to destination buffer */
 }
 return(dtpos);				/* bytes in destination buffer return */
#endif						/* NO_ECC */
}
#endif						/* __UCRP_ECC_ENABLE__ */
#ifdef __UCRP_ECC_ENABLE__		/* ECC enable macro check */
int ucrp_ecc_de(u_int8_t * dst, u_int8_t * src, int32_t len) /* ecc decrypt */
{
 if(					/* input parameters check */
    (dst == 0) ||			/* destination buffer pointer check */
    (src == 0)				/* source buffer pointer check */
 ) {
    return(0);				/* function error return */
 }
#ifdef NO_ECC				/* no ecc macro defined check */
 memcpy(dst,src,len);			/* source buffer copy */
 return(len);				/* destination bytes count return */
#else
 u_int8_t calc[3];			/* calculated ecc */
 u_int8_t * ecc;			/* readed ecc */
 int32_t dtpos;				/* destination position */
 int32_t srpos;				/* source position */
 dtpos = 0;				/* destination position start */
 for(srpos = 0; srpos < len;) {		/* decrypt loop */
    int32_t cnt;			/* bytes processed */
    if(srpos + 256 + 3 <= len) {	/* full block check */
	ecc = src + srpos + 256;	/* ecc read */
	nand_calculate_ecc(src + srpos, calc); /* ecc calculate */
	nand_correct_data(src + srpos, ecc, calc); /* data correct */
	cnt = 256;			/* bytes processed count */
	int32_t pos;			/* buffer position */
	for(pos = 0; pos < cnt; ++pos) { /* copy loop */
	    *(dst + dtpos) = *(src + srpos); /* byte copy */
	     dtpos += 1;		/* destination position next */
	     srpos += 1;		/* source position next */
	}
	srpos += 3;			/* ecc bytes skip */
    } else {
	cnt = len - srpos - 3;		/* bytes in source buffer without ecc */
	u_int8_t buf[256];		/* temporary buffer */
	int32_t pos;			/* temporary buffer position */
	for(pos = 0; pos < cnt; ++pos) { /* copy to temporary buffer */
	    buf[pos] = *(src + srpos + pos); /* byte copy */
	}
	ecc = src + srpos + pos;	/* ecc read */
	while(pos < 256) {		/* tail fill */
	    buf[pos++] = 0xFF;		/* byte set */
	}
	nand_calculate_ecc(buf, calc);	/* ecc calculate */
	nand_correct_data(buf, ecc, calc); /* data correct */
	for(pos = 0; pos < cnt; ++pos) { /* copy loop */
	    *(dst + dtpos) = *(buf + pos); /* byte copy */
	    dtpos += 1;			/* destination position next */
	}
	{ break; }			/* loop exit */
    }
 }
 return(dtpos);				/* bytes in destination buffer */
#endif						/* NO_ECC */
}
#endif 	
int16_t ucrp_proc_clear(void)		/* free disk space clear */
{
 u_int8_t * buf;			/* buffer */
 u_int32_t bufsize;			/* buffer size */
 bufsize = 0x10000;			/* buffer size init 64K */
 buf = (u_int8_t *)malloc(bufsize);	/* buffer allocation */
 if(buf == 0) {				/* buffer allocation check */
    ucrp_error(				/* buffer allocation error print */
	UMSG_ALLOC_ERROR,		/* message format */
	bufsize,			/* buffer size decimal */
	bufsize				/* buffer size hexadecimal */
    );
    return(0);				/* function return */
 }
 int16_t rvl;				/* return code */
 rvl = 1;				/* return code start */
 {					/* history, log files remove */
    struct passwd * pwd;		/* passwd structure */
    uid_t uid;				/* user id */
    u_int8_t * home;			/* home directory */
    u_int8_t * sep;			/* tail name separator */
    uid = geteuid();			/* efective user id get */
    pwd = getpwuid(uid);		/* password structure read */ 
    home = (u_int8_t *) UMSG_ROOT;	/* home directory start */
    sep = (u_int8_t *) UMSG_SLASH;	/* separator string */
    if(pwd != 0) {			/* password structure null pointer check */
	if(pwd->pw_name != 0) {		/* user name pointer check */
	    if(pwd->pw_dir != 0) {	/* home directory pointer check */
		home = (u_int8_t *)pwd->pw_dir; /* home directory write */
		if(*home == '/') {	/* start separator check */
		    home += 1;		/* start separator skip */
		}
		int len;		/* home directory name */
		len = strlen((char *)home); /* home directory length calculate */
		if(len > 0) {		/* lenght don't zero check */
		    if(*(home + (len - 1)) == '/') { /* end character check */
			sep = (u_int8_t *)UMSG_EMPTY; /* end separator clear */
		    }
		}
	    }
	}
    }
    char * fmtspec[] = {		/* format specificator */
	UMSG_BASH_HISTORY,		/* bash history format specificator */
	UMSG_MC_HISTORY,		/* mc history format specificator */
	0
    };
    int16_t num;			/* format number */
    for(num = 0; fmtspec[num] != 0; ++num) { /* file name loop */
	snprintf(			/* file name form */
	    (char *)buf,		/* file name buffer */
	    bufsize,			/* file name buffer size */
	    fmtspec[num],		/* file name format */
	    home,			/* home directory */
	    sep				/* separator string */
	);
	if(ucrp_file(			/* file exist check */
	    UMSG_FILE_EXIST, 		/* f specificator */
	    (char *)buf			/* file name buffer */
	) > 0) {
	    if(ucrp_clear((char *)buf) <= 0) { /* file clear */
		rvl = 0;		/* return code clear */
	    }
	    ucrp_printf(		/* unlink message print */
		UMSG_FILE_UNLINK,	/* unlink format */
		buf			/* file name */
	    );
	    if(ucrp_unlink((char *)buf) == 0) { /* file unlink */
		rvl = 0;		/* return code clear */
	    }
	} else {			/* => file absent */
	    ucrp_printf(		/* absent message print */
		UMSG_FILE_ABSENT, 	/* message format */
		buf			/* file name */
	    );
	}
    }
 }
 int16_t num;				/* file number */
 num = 0;				/* file number start */
 while(1) {				/* file loop */
    snprintf(				/* file name form */
	(char *)buf,			/* file name buffer */
	bufsize,			/* file name buffer size */
	UMSG_UCRP_CLEAR,		/* file clear format */
	num				/* file number */
    );
    int des;				/* file deescriptor */
    des = open((char *)buf, (O_RDWR|O_CREAT)); 	/* file open */
    if(des < 0) {			/* file open error check */
	rvl = 0;			/* return code clear */
	ucrp_error(			/* error message print */
	    UMSG_OPEN_ERROR,		/* file open error format */
	    buf				/* file name */
	);
	{ break; }			/* loop exit */
    }
    ucrp_printf(UMSG_BREKET_START);	/* breket start print */
    ucrp_printf(UMSG_STRING,buf);	/* file name print */
    ucrp_printf(UMSG_BREKET_END);	/* breket end print */
    u_int8_t psbf[32];			/* file position buffer */
    snprintf(				/* file position buffer form */
	(char *)psbf,			/* position buffer */
	sizeof(psbf),			/* position buffer size */
	UMSG_BREKET_POS,		/* format */
	0				/* start position */
    );
    ucrp_printf(UMSG_17S,psbf);		/* position buffer print */
    u_int32_t pos;			/* file position */	
    pos = 0;				/* file position start */
    while(pos < bufsize) {		/* buffer fill loop */
	if(ucrp_clear_mode == UCRP_CLEAR_CLR) { /* clear mode check */
	    *(buf + pos) = 0;		/* bufer item clear */
	} else
	if(ucrp_clear_mode == UCRP_CLEAR_SET) { /* set item mode check */
	    *(buf + pos) = 0xFF;	/* item set */
	} else
	if(ucrp_clear_mode == UCRP_CLEAR_INC) { /* increment filling mode */
	    *(buf + pos) = pos;		/* item write */
	} else {
	    *(buf + pos) = pos;
	}
	pos += 1;			/* position next */
    }	
    pos = 0;				/* file position start */
    while(pos < 1024 * (1024 * 1024)) { /* file write loop */
	int32_t wcnt;			/* bytes writed */
	wcnt = write(des, buf, bufsize);/* buffer write */ 
	if(wcnt < 0) {			/* error code check */
	    { break; }			/* loop exit */
	}
	pos += wcnt;			/* file position next */
	if((pos % (1024 * 128)) == 0) { /* file position print check */
	    int16_t bsn;		/* back space number */
	    for(bsn = 0; bsn < 17; ++bsn) { /* file position clear */
		ucrp_printf(UMSG_BACKSPACE); /* back space print */
	    }
	    snprintf(			/* position buffer form */
		(char *)psbf,		/* position buffer */
		sizeof(psbf),		/* position buffer size */
		UMSG_BREKET_POS,	/* format */ 
		pos			/* file position */
	    );
	    ucrp_printf(UMSG_17S,psbf);	/* file position buffer print */
	}
	if(wcnt != (int)bufsize) {	/* write full check */
	    { break; }			/* loop exit */
	}
    }
    {
	int16_t bsn;			/* back space number */
	for(bsn = 0; bsn < 17; ++bsn) {	/* file position clear loop */
	    ucrp_printf(UMSG_BACKSPACE);
	}
	snprintf(			/* position buffer form */
	    (char *)psbf,		/* position buffer */
	    sizeof(psbf),		/* position buffer size */
	    UMSG_BREKET_POS,		/* format */ 
	    pos				/* file position */
	);
	ucrp_printf(UMSG_17S,psbf);	/* file position buffer print */
	ucrp_printf(UMSG_CRNL);		/* <CR><NL> print */
    }
    if(fsync(des) < 0) {		/* file sync */
	rvl = 0;			/* return code clear */
    }
    if(des != 0) {			/* descriptor check */
	if(close(des) < 0) {		/* file cose */
	    rvl = 0;			/* return code clear */
	}
	des = 0;			/* file descriptor clear */
    }
    if(pos < 1024 * (1024 * 1024)) {
	{ break; }			/* all files writed */
    }
    num += 1;				/* file number next */
 }
 while(num >= 0) {			/* remove files loop */
    snprintf(				/* file name form */
	(char *)buf,			/* file name buffer */
	bufsize,			/* file name buffer size */
	UMSG_UCRP_CLEAR,		/* format */
	num				/* file number */
    );
    if(ucrp_file(UMSG_FILE_EXIST,(char *)buf) > 0) { /* file existance check */
	ucrp_printf(UMSG_FILE_UNLINK,buf); /* unlink message print */
	if(unlink((char *)buf) < 0) {	/* unlink file */
	    rvl = 0;			/* return code clear */
	}
    }
    num -= 1;				/* number prev */
 }
 if(buf != 0) {				/* buffer pointer check */
    free(buf);				/* buffer free */
    buf = 0;				/* buffer pointer clear */
 }
 return(rvl);
}
static u_int8_t * ucrp_sig_name(siginfo_t * sinfo, u_int8_t * name, u_int16_t size) /* signal name return */
{
 static struct {			/* signal name specification */ 
    int32_t num;			/* signal number */
    char * name;			/* signal name */
 } sginfo[] = {				/* signal specificator array */
    {SIGHUP   , UMSG_SIGHUP},		/* 1 */
    {SIGINT   , UMSG_SIGINT},		/* 2 */
    {SIGQUIT  , UMSG_SIGQUIT},		/* 3 */
    {SIGILL   , UMSG_SIGILL}, 		/* 4 */
    {SIGTRAP  , UMSG_SIGTRAP},		/* 5 */
    {SIGABRT  , UMSG_SIGABRT},		/* 6 */
    {SIGBUS   , UMSG_SIGBUS}, 		/* 7 */
    {SIGFPE   , UMSG_SIGFPE},		/* 8 */
/* not processed signal */
    {SIGKILL  , UMSG_SIGKILL},		/* 9 */
    {SIGUSR1  , UMSG_SIGUSR1}, 		/* 10 */
    {SIGSEGV  , UMSG_SIGSEGV}, 		/* 11 */
    {SIGUSR2  , UMSG_SIGUSR2}, 		/* 12 */
    {SIGPIPE  , UMSG_SIGPIPE},		/* 13 */
    {SIGALRM  , UMSG_SIGALRM},		/* 14 */
    {SIGTERM  , UMSG_SIGTERM},		/* 15 */
    {SIGSTKFLT, UMSG_SIGSTKFLT}, 	/* 16 */
/* 17-31 */
    {SIGCHLD  , UMSG_SIGCHLD}, 		/* 17 */
    {SIGCONT  , UMSG_SIGCONT}, 		/* 18 */
    {SIGSTOP  , UMSG_SIGSTOP},		/* 19 */
    {SIGTSTP  , UMSG_SIGTSTP}, 		/* 20 */
    {SIGTTIN  , UMSG_SIGTTIN}, 		/* 21 */
    {SIGTTOU  , UMSG_SIGTTOU}, 		/* 22 */
    {SIGURG   , UMSG_SIGURG},		/* 23 */
    {SIGXCPU  , UMSG_SIGXCPU}, 		/* 24 */
    {SIGXFSZ  , UMSG_SIGXFSZ}, 		/* 25 */
    {SIGVTALRM, UMSG_SIGVTALRM}, 	/* 26 */
    {SIGPROF  , UMSG_SIGPROF}, 		/* 27 */
    {SIGWINCH , UMSG_SIGWINCH}, 	/* 28 */
    {SIGIO    , UMSG_SIGIO},		/* 29 */
    {SIGPWR   , UMSG_SIGPWR}, 		/* 30 */
    {SIGSYS   , UMSG_SIGSYS}		/* 31 */
 };
 u_int8_t * ptr;			/* signal name pointer */
 if(
    (sinfo->si_signo >= 1) && 		/* signal number low spec limit check */
    (sinfo->si_signo <= 31)		/* signal number upper spec limit check */
 ) {
    if(sginfo[sinfo->si_signo-1].num == sinfo->si_signo) {
	ptr = (u_int8_t *)sginfo[sinfo->si_signo-1].name;
    } else {
	return(0);			/* name form error */
    }
 } else {
    return(0);				/* name form error */
 }
 if(					/* input parameters check */
    (name == 0) || 			/* name pointer 0 check */
    (size == 0)				/* size 0 check */
 ) {
    return(ptr);			/* static pointrer return */
 }
 snprintf(
    (char *)name,			/* signal name buffer */
    size,				/* buffer size */
    UMSG_STRING,			/* format */
    ptr
 );    
 return(name);				/* buffer pointer return */
} 
static int16_t ucrp_sig_info(siginfo_t * sinfo, void * sptr) /* signal information print */
{
 if(					/* input parameter check */
    (sinfo == 0) || 			/* signal info structure check */
    (sptr == 0)
 ) {
    ucrp_error(				/* error message print */
	UMSG_BAD_SIGNAL_CALL,		/* message format */ 
	sinfo	,			/* signal info pointer */
	sptr				/* signal pointer */
    );
    return(0);				/* return error */
 }
 u_int8_t name[32];			/* signal name */
 if(ucrp_sig_name(			/* signal name form */
    sinfo,				/* signal info */
    name,				/* name buffer */
    sizeof(name)			/* buffer size */
 ) != 0) {				/* name form return code check */
    ucrp_printf(			/* signal name print */
	UMSG_STRING,			/* message format */
	name				/* signal name */
    );
    ucrp_printf(UMSG_CRNL);		/* <CR><NL> print */
 }
 ucrp_printf(				/* signal description print */
    UMSG_SIG_INFO	,		/* message format */
    sinfo->si_signo	,		/* 1 signal number */
    sinfo->si_errno	,		/* 2 errno code */
    sinfo->si_code 	,		/* 3 code */
    sinfo->si_pid 	,		/* 4 process id */
    sinfo->si_uid	,		/* 5 user if */
    sinfo->si_status	,		/* 6 */
    sinfo->si_utime	,		/* 7 user time */
    sinfo->si_stime	,		/* 8 system time */
    sinfo->si_value	,		/* 9 signal parameter */
    sinfo->si_int 	,		/* 10 int */
    sinfo->si_ptr	,		/* 11 ptr */
    sinfo->si_addr	,		/* 12 addr */
    sinfo->si_band	,		/* 13 */
    sinfo->si_fd	,		/* 14 file descriptor */
    sptr				/* 15 */
 );
 return(1);	
}
int ucrp_sig_log(siginfo_t * sinfo, void * sptr) /* signal log */
{
 if(sinfo == 0) {			/* signal info pointer check */
    return(0);
 } 
 int16_t rvl;				/* return code */
 rvl = 1;				/* return code start */
 u_int8_t buf[32];			/* file name/signal name buffer */
 snprintf(				/* file name form */
    (char *)buf,			/* file name buffer */
    sizeof(buf),			/* file name buffer size */
    UMSG_VAR_LOG,			/* name format */
    UMSG_SIGNAL_DUMP			/* signal dump file name */
 );
 FILE * flog;				/* log file decription pointer */
 flog = fopen((char *)buf, UMSG_READBYTE); /* open in read mode */
 if(flog == 0) {			/* file decription pointer check */
    flog = fopen((char *)buf, UMSG_WRITEBYTE); /* open file in write mode */
    if(flog == 0) {			/* open file error */
	return(0);			/* function error */
    }
 } else {
    if(fseek(flog, 0, SEEK_END) < 0) {	/* seek end file */
	rvl = 0;			/* return code clear */
    }
 }
 if(ucrp_sig_name(sinfo,buf,sizeof(buf)) <= 0) {
    fprintf(flog, UMSG_DECIMAL, sinfo->si_signo);
 } else {
    fprintf(flog, UMSG_STRING, buf);
 }
 fprintf(flog, UMSG_CRNL);
 if(fclose(flog) < 0) {			/* log file close */
    rvl = 0;
 }
 if(rvl == 0) {
    return(0);
 }
 return(1);
}
volatile u_int8_t kill_link = 0;	/* terminate signal received */
static void ucrp_sig_term_ext(int sig, siginfo_t * sinfo, void * sptr)
{
 if(kill_link == 0) {			/* first signal received check */
    kill_link = 1;			/* terminate signal received */
#ifdef __UCRP_SIG_DEBUG__		/* signal defined macro check */
    ucrp_sig_info(sinfo,sptr);		/* signal description print */
    ucrp_sig_log(sinfo,sptr);		/* signal log write */
#endif						/* __UCRP_SIG_DEBUG__ */
 } else {
    kill_link += 1;			/* signal count calculate */
 }
}
static void ucrp_bad_signal_ext(int sig, siginfo_t * sinfo, void * sptr) /* unknown signal received */
{
#ifdef __UCRP_SIG_DEBUG__		/* signal defined macro check */
 ucrp_sig_info(sinfo,sptr);		/* signal information print */
 ucrp_sig_log(sinfo,sptr);		/* signal log print */
#endif						/* __UCRP_SIG_DEBUG__ */
 exit(1);		
}
int16_t ucrp_signal(int32_t num, void (*fun)(int, siginfo_t *, void *)) 
{
 if(					/* signal number check */
    (num == SIGKILL) ||			/* unkond kill */
    (num == SIGSTOP)			/* stop process check */
 ) {
    return(0);				/* no processed signal */
 } 
 struct sigaction sact;			/* signal action */
 sact.sa_handler = 0;			/* signal handler */
 sact.sa_sigaction = fun;		/* signal function */
 sact.sa_restorer = 0;			/* restore function */
 sact.sa_flags = (			/* signal processing settings */
    SA_SIGINFO |			/* 3 args */
    SA_RESTART |			/* */
    SA_NOMASK				/* don't mask this signal */
 );
 if(sigemptyset(&sact.sa_mask) < 0) {	/* blocket signal mask clear */
    ucrp_error(				/* error message print */
	UMSG_SIG_EMPTY_SET_ERROR	/* message format */
    );
    return(0);
 }
 if(
    (sigaddset(&sact.sa_mask, SIGHUP)  < 0) || /* SIGHUP */ 
    (sigaddset(&sact.sa_mask, SIGINT)  < 0) || /* SIGINT */ 
    (sigaddset(&sact.sa_mask, SIGTERM) < 0) || /* SIGTERM */ 
    (sigaddset(&sact.sa_mask, SIGCHLD) < 0) || /* SIGCHLD */ 
    (sigaddset(&sact.sa_mask, SIGUSR2) < 0)    /* SIGUSR2 */ 
 ) {	
    ucrp_error(				/* error message print */
	UMSG_SIG_ADD_SET_ERROR,		/* message format */
	sact.sa_mask			/* mask */
    );
    return(0);
 }
 if(sigaction(num, &sact, 0) < 0) {	/* set signal handler */
    ucrp_error(				/* error message print */
	UMSG_SIG_ACTION_ERROR,		/* message format */
	num				/* signal number */
    );
    return(0);
 }
 return(1);
}
int main(int ucrp_argc, char * ucrp_args[])
{
 {
    u_int32_t ptsz;			/* path size */
    ptsz = _POSIX_NAME_MAX;		/* name size max write */
    if(ptsz < _POSIX_PATH_MAX) {	/* path size check */
	ptsz = _POSIX_PATH_MAX; 	/* path size write */
    }
    if(ptsz > ucrp_path_size) {		
	if(ptsz >= 0xFFFF) {
	    ptsz = 0xFFFF; 		/* path size set to upper limit */
	}
	ucrp_path_size = ptsz;		/* path size write */
    }
 }
 int rvl; 				/* return code */
 rvl = 1;				/* return code start */
 if(ucrp_init() == 0) {			/* init */
    rvl = 0;				/* return code clear */
 }
 {
    struct {				/* signal definition */
        int32_t num;			/* signal number */
        void (*fun)(int, siginfo_t *, void *); /* function */
    } sigdef[] = {
	{SIGHUP   , ucrp_bad_signal_ext},	/* 1 - Hangup */
	{SIGINT   , ucrp_bad_signal_ext},	/* 2 - Interrupt */
	{SIGQUIT  , ucrp_bad_signal_ext},	/* 3 - Quit */
	{SIGILL   , ucrp_bad_signal_ext},	/* 4 - Illegal instruction */
	{SIGTRAP  , ucrp_bad_signal_ext},	/* 5 - trap */
	{SIGABRT  , ucrp_bad_signal_ext},	/* 6 - abort */
	{SIGBUS   , ucrp_bad_signal_ext},	/* 7 - buss error */
	{SIGFPE   , ucrp_bad_signal_ext},	/* 8 - floating point exeption */
/* not processed signal */
/*	{SIGKILL  , ucrp_bad_signal_ext}, */	/* 9 - kill */
	{SIGUSR1  , ucrp_bad_signal_ext},	/* 10 - user signal 1 */
	{SIGSEGV  , ucrp_bad_signal_ext},	/* 11 - segmentation violation */
	{SIGUSR2  , ucrp_bad_signal_ext},	/* 12 - user signal 2 */
	{SIGPIPE  , ucrp_bad_signal_ext},	/* 13 - pipe */
	{SIGALRM  , ucrp_bad_signal_ext},	/* 14 - alarm */
	{SIGTERM  , ucrp_sig_term_ext},		/* 15 - term */
	{SIGSTKFLT, ucrp_bad_signal_ext},	/* 16 - stack fault */
/* 17-31 */
	{SIGCHLD  , ucrp_bad_signal_ext},	/* 17 - children */
	{SIGCONT  , ucrp_bad_signal_ext},	/* 18 - continue */
/* not processed signal */
/*	{SIGSTOP  , ucrp_bad_signal_ext}, */	/* 19 - stop */
	{SIGTSTP  , ucrp_bad_signal_ext},	/* 20 - terminal stop, terminal command */
	{SIGTTIN  , ucrp_bad_signal_ext},	/* 21 - terminal read */
	{SIGTTOU  , ucrp_bad_signal_ext},	/* 22 - terminal write */
	{SIGURG   , ucrp_bad_signal_ext},	/* 23 - urg */
	{SIGXCPU  , ucrp_bad_signal_ext},	/* 24 - cpu limit exceeded */
	{SIGXFSZ  , ucrp_bad_signal_ext},	/* 25 - file size exceeded */
	{SIGVTALRM, ucrp_bad_signal_ext},	/* 26 - virtual alarm clock */
	{SIGPROF  , ucrp_bad_signal_ext},	/* 27 - profilling alarm clock */
	{SIGWINCH , ucrp_bad_signal_ext},	/* 28 - window size changed */
	{SIGIO    , ucrp_bad_signal_ext},	/* 29 - I/O */
	{SIGPWR   , ucrp_bad_signal_ext},	/* 30 - power failure restart */
	{SIGSYS   , ucrp_bad_signal_ext},	/* 31 - bad system call */
	{0, 0}
    };
    int16_t idx;			/* signal index */
    idx = 0;				/* index start */
    while(1) {				/* signal loop */
	int32_t num;			/* signal number */
	num = sigdef[idx].num;		/* signal number read */
        if(num == 0) {			/* terminator check */
    	    { break; }			/* loop exit */
	}
	if(ucrp_signal(num,sigdef[idx].fun) == 0) { /* set function */
    	    rvl = 0;			/* set signal error */
	}
	idx += 1;			/* index next */
    }
    for(idx = SIGRTMIN; idx < SIGRTMAX; ++idx) { /* 35 - 64 */
	if(ucrp_signal(idx, ucrp_bad_signal_ext) == 0) {
	    rvl = 0;			/* set signal error */
	}
    }
 }
 u_int16_t ecmd = 0;			/* end command */
 int32_t argn;				/* argument number */
 int16_t argd;				/* argument increment */
 u_int8_t rqag;				/* argument required */
 rqag = 0;				/* argument required clear */	
 for(argn = 1; argn < ucrp_argc; argn += argd) { /* argument loop */
    argd = 1;				/* one argument processed */
    if(strcmp(ucrp_args[argn], UMSG_OPT_ENCRYPT) == 0) { /* encrypt option check */
	    ucrp_cmd |= UCRP_ENCRYPT;	/* encrypt command bit write */
    } else
    if(strcmp(ucrp_args[argn], UMSG_OPT_DECRYPT) == 0) { /* decrypt string check */
	    ucrp_cmd |= UCRP_DECRYPT;	/* decrypt command write */
    } else
    if(strcmp(ucrp_args[argn], UMSG_OPT_CLEAR) == 0) { /* clear command check */
	    ucrp_cmd |= UCRP_CLEAR;	/* clear command write */
	    if(argn + 1 < ucrp_argc) { 	/* command parameter check */
		if(strcmp(		/* inc string check */
		    ucrp_args[argn+1],	/* option argument */
		    UMSG_ARG_CLEAR_INC	/* inc string */
		) == 0) {
		    ucrp_clear_mode = UCRP_CLEAR_INC; /* clear mode increment */
		    argd = 2;		/* two argument processed */
		} else
		if(strcmp(		/* 0 string check */
		    ucrp_args[argn+1],	/* option argument */
		    UMSG_ARG_CLEAR_CLR	/* 0 string */
		) == 0) {
		    ucrp_clear_mode = UCRP_CLEAR_CLR;  /* clear mode clear */ 
		    argd = 2;		/* two argument processed */
		} else
		if(strcmp(		/* 1 string check */
		    ucrp_args[argn+1],	/* option argument */
		    UMSG_ARG_CLEAR_SET	/* 1 string */
		) == 0) {
		    ucrp_clear_mode = UCRP_CLEAR_SET; /* clear mode set */
		    argd = 2;		/* two argument processed */
		}
	    }
    } else
    if(strcmp(ucrp_args[argn], UMSG_OPT_QUIET) == 0) { /* quiet string check */
	ucrp_quiet = 1;			/* quiet mode */
    } else
    if( 				/* change dirrectory command check */
	(strcmp(ucrp_args[argn], UMSG_OPT_CURRENT_DIR) == 0) || /* current-dir command check */
	(strcmp(ucrp_args[argn], UMSG_OPT_CHANGE_DIR) == 0) /* change-dir command check */
    ) {
	if(argn + 1 >= ucrp_argc) {	/* option argument check */
	    rqag = 1;			/* argument required */
	    { break; }			/* loop exit */
	} else {
	    snprintf(			/* directory name save */
		ucrp_curdir,		/* directory name buffer */
		sizeof(ucrp_curdir), 	/* directory name buffer size */
		UMSG_STRING,		/* string format */
		ucrp_args[argn+1]	/* option parameter */ 
	    );
	    argd = 2;			/* two argument processed */
	}
    } else 
    if(strcmp(ucrp_args[argn], UMSG_OPT_DISABLE) == 0) { /* disable option check */
	    if(argn + 1 >= ucrp_argc) {	/* option argument check */
		rqag = 1;		/* argument required */
		{ break; }		/* loop exit */
	    } else {
		int16_t num;		/* argument number */
		for(num = argn + 1; num < ucrp_argc; ++num) { /* option argument loop */
#ifdef __UCRP_XOR_ENABLE__		/* XOR enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_XOR) == 0) { /* xor argument check */
			ucrp_enable &= (~UCRP_XOR_ENABLE); /* xor disable */
		    } else
#endif						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* ADD enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_ADD) == 0) {  /* add argument check */
			ucrp_enable &= (~UCRP_ADD_ENABLE); /* add disable */
		    } else
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE___		/* ROT enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_ROT) == 0) { /* rot argument check */
			ucrp_enable &= (~UCRP_ROT_ENABLE); /* rot disable */
		    } else
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_TAB) == 0) { /* tab argument check */
			ucrp_enable &= (~UCRP_TAB_ENABLE); /* tab disable */
		    } else
#endif						/* __UCRP_TAB_ENABLE__ */
#ifdef __UCRP_ECC_ENABLE__		/* ECC enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_ECC) == 0) { /* ecc argument check */
			ucrp_enable &= (~UCRP_ECC_ENABLE); /* ecc disable */
		    } else
#endif						/* __UCRP_ECC_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_BTS) == 0) { /* bts argument check */
			ucrp_enable &= (~UCRP_BTS_ENABLE); /* bts disable */
		    } else
#endif						/* __UCRP_BTS_ENABLE__ */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_XRE) == 0) { /* xre argument check */
			ucrp_enable &= (~UCRP_XRE_ENABLE); /* xre disable */
		    } else
#endif						/* __UCRP_XRE_ENABLE__ */
		    if(strcmp(ucrp_args[num],UMSG_ARG_STD) == 0) { /* standart algoritm disable */ 
			ucrp_enable &= (~(0xFFFFFFFF & (~UCRP_XOR_ENABLE)));
		    } else {		/* unknown algoritm */
			{ break; }	/* loop exit */
		    }
		}
		argd = num - argn;		/* option processed */
		if(argd <= 1) {		/* disable without argument check */
		    rqag = 1;		/* argument required */
		    { break; }		/* loop exit */
		}
	    }
    } else 
    if((strcmp(ucrp_args[argn], UMSG_OPT_ENABLE) == 0)) { /* enable option check */
	    if(argn + 1 >= ucrp_argc) {	/* option argument check */
		rqag = 1;		/* argument required */
		{ break; }		/* loop exit */
	    } else {
		int16_t num;		/* argument number */
		for(num = argn + 1; num < ucrp_argc; ++num) { /* option argument loop */
#ifdef __UCRP_XOR_ENABLE__		/* XOR enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_XOR) == 0) { /* xor argument check */
			ucrp_enable |= UCRP_XOR_ENABLE; /* xor enable */
		    } else
#endif						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* ADD enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_ADD) == 0) { /* add argument check */
			ucrp_enable |= UCRP_ADD_ENABLE; /* add enable */
		    } else
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_ROT) == 0) { /* rotate argument check */
			ucrp_enable |= UCRP_ROT_ENABLE; /* rotate enable */
		    } else
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_TAB) == 0) {  /* tab argument check */
			ucrp_enable |= UCRP_TAB_ENABLE; /* tab enable */ 
		    } else
#endif						/* __UCRP_TAB_ENABLE__ */
#ifdef __UCRP_ECC_ENABLE__		/* ECC enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_ECC) == 0) { /* ecc argument check */
			ucrp_enable |= UCRP_ECC_ENABLE; /* ecc enable */
		    } else
#endif						/* __UCRP_ECC_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_BTS) == 0) { /* bts argument check */
			ucrp_enable |= UCRP_BTS_ENABLE; /* bts enable */
		    } else
#endif						/* __UCRP_BTS_ENABLE__ */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
		    if(strcmp(ucrp_args[num],UMSG_ARG_XRE) == 0) { /* xre argument check */
			ucrp_enable |= UCRP_XRE_ENABLE; /* xre enable */
		    } else
#endif						/* __UCRP_XRE_ENABLE__ */
		    if(strcmp(ucrp_args[num],UMSG_ARG_STD) == 0) { /* std argument check */
			ucrp_enable |= (0xFFFFFFFF & (~UCRP_XOR_ENABLE)); /* standart enable */
		    } else {
			{ break; }	/* loop exit */ 
		    }
		}
		argd = num - argn;	/* option processed */
		if(argd <= 1) {		/* option processed check */
		    rqag = 1;		/* argument required */
		    { break; }		/* loop exit */
		}
	    }
    } else 
    if(strcmp(ucrp_args[argn], UMSG_OPT_BLOCK_SIZE) == 0) { /* option block size check */
	    if(argn + 1 >= ucrp_argc) {	/* option argument check */
		rqag = 1;		/* argument required */
		{ break; }		/* loop exit */
	    } else {
		u_int32_t size;		/* block size */
		char * str;		/* text pointer */
		str = ucrp_args[argn+1];/* text pointer read */
		if(str != 0) {		/* text pointer check */
		    if(			/* leading x or X check */
			(*str == 'x') ||/* x check */
			(*str == 'X')	/* X check */
		    ) { 
			char * endptr;	/* */
			size = strtoul(str + 1, &endptr, 16); /* size read */
		    } else 
		    if(			/* leading 0x or 0X check */
			(*str == '0') &&/* start 0 check */
			(
			    (*(str + 1) == 'x') || /* x check */
			    (*(str + 1) == 'X')	/* X check */
			)
		    ) {
			char * endptr;	/* */
			size = strtoul(str + 2, &endptr, 16); /* size hex read */
		    } else {
			size = atoi(str);/* decimal read */
		    }
		} else {
		    rqag = 1;		/* argument required */
		    { break; }		/* loop exit */
		}
		if(size < 256) {	/* size low limit check */
		    size = 256; 	/* low limit write */
		}
		ucrp_src_buf_size = ((size + 255) / 256) * 256;  /* source buffer size */
		ucrp_dst_buf_size =	/* destination buffer size */
		    ucrp_src_buf_size
#ifdef __UCRP_ECC_ENABLE__		/* ecc macro check */
		    + (((ucrp_src_buf_size + 255) / 256) * 3)
#endif						/* __UCRP_ECC_ENABLE__ */
		;
		argd = 2;		/* two argument processed */
	    }
    } else
    if(strcmp(ucrp_args[argn], UMSG_OPT_PASSWD) == 0) {
	    if(argn + 1 >= ucrp_argc) {
		rqag = 1;		/* argument required */
		{ break; }		/* loop exit */
	    } else {
		if(ucrp_passwd[0] == '\0') { /* password empty check */
		    snprintf(		/* password buffer check */
			ucrp_passwd,	/* password buffer */
			sizeof(ucrp_passwd), /* password buffer size */
			UMSG_STRING,	/* string format */
			ucrp_args[argn+1] /* passvord option argument */
		    );
	        } else {		/* => password file name */
		    snprintf(		/* password file name form */
			ucrp_pfile,	/* password file name buffer */
			sizeof(ucrp_pfile), /* password file name buffer size */
			UMSG_STRING,	/* string format */
			ucrp_args[argn+1] /* passvord option argument */
		    );
		}
	    	argd = 2;		/* two argument processed */
	    }
    } else
    if(strcmp(ucrp_args[argn], UMSG_OPT_INFO) == 0) {
	    ucrp_cmd |= UCRP_INFO;	/* info bit set */
    } else
    if(strcmp(ucrp_args[argn], UMSG_OPT_HIDE_PASSWD) == 0) { /* hide password option check */
	    if(strcmp(			/* standart passwd hidding check */
		ucrp_args[argn+1], 	/* option argument */
		UMSG_ARG_HIDE_PASSWD_STD /* standart hiding string */
	    ) == 0) {
    		ucrp_hide_passwd = 1;	/* */
	    	argd = 2;		/* two argument processed */
	    } else
	    if(strcmp(			/* $ password hidding check */
		ucrp_args[argn+1], 	/* option argument */
		UMSG_ARG_HIDE_PASSWD_DOL  /* $ hiding string */
	    ) == 0) {
    		ucrp_hide_passwd = 2;
	    	argd = 2;		/* two argument processed */
	    } else
	    if(strcmp(			/* d password hidding check */
		ucrp_args[argn+1],	/* option argument */
		UMSG_ARG_HIDE_PASSWD_D	/* d hidding string */
	    ) == 0) {
    		ucrp_hide_passwd = 3;
	    	argd = 2;		/* two argument processed */
	    } else {
    		ucrp_hide_passwd = 1;	/* default */
	    }
    } else
    if(strcmp(ucrp_args[argn],UMSG_OPT_SHOW_PASSWD) == 0) { /* show-password option check */
	    ucrp_hide_passwd = 0;	/* hide password clear */
    } else
    if(strcmp(ucrp_args[argn], UMSG_OPT_HELP) == 0) { /* help option check */
	ecmd = UCRP_HELP;		/* end command write */
	{ break; }			/* loop exit */
    } else
    if(strcmp(ucrp_args[argn], UMSG_OPT_LICENSE) == 0) { /* license option check */
	ecmd = UCRP_LIC;		/* end command write */
	{ break; }			/* loop exit */
    } else
    if(strcmp(ucrp_args[argn], UMSG_OPT_LICENSE_RU) == 0) { /* license-ru option check */
	ecmd = UCRP_LICRU;		/* end command write */
	{ break; }			/* loop exit */
    } else
    if(strcmp(ucrp_args[argn], UMSG_OPT_VERSION) == 0) { /* version command check */
	ecmd = UCRP_VERSION;		/* end command write */
	{ break; }			/* loop exit */
    } else
    if(strcmp(ucrp_args[argn], UMSG_OPT_TODO) == 0) { /* todo option check */
	ecmd = UCRP_TODO;		/* end command write */
	{ break; }			/* loop exit */
    } else
    if(strcmp(ucrp_args[argn], UMSG_OPT_CHANGELOG) == 0) { /* change log command check */
	ecmd = UCRP_CHANGELOG;		/* end command write */
	{ break; }			/* loop exit *//* loop exit */
    } else
    if(strcmp(ucrp_args[argn], UMSG_OPT_MAN) == 0) { /* man page option check */
	ecmd = UCRP_MAN;		/* end command write */
	{ break; }			/* loop exit */
    } else
    {
	if(argn + 1 == ucrp_argc - 1) {	/* two argument remain */
	    snprintf(			/* input file name save */
		ucrp_inpfile,		/* input file name buffer */
		sizeof(ucrp_inpfile), 	/* input file name size */
		UMSG_STRING, 		/* string format */
		ucrp_args[argn]		/* command line option */
	    );	
	    snprintf(			/* output file name form */
		ucrp_outfile,		/* output file name buffer */
		sizeof(ucrp_outfile), 	/* output file name buffer size */
		UMSG_STRING,		/* string format */
		ucrp_args[argn+1]	/* command line option + 1 */
	    );
	    argd = 2;			/* two argument processed */
	} else
	if(argn == ucrp_argc - 1) {	/* only input file name */
	    snprintf(			/* input file name save */
		ucrp_inpfile,		/* input file name buffer */
		sizeof(ucrp_inpfile), 	/* input file name buffer size */
		UMSG_STRING,		/* string format */
		ucrp_args[argn]		/* command line argument */
	    );
	}
    }
 }
 if(
    (rqag > 0)				/* argument required check */
  ) {
    	ucrp_error(UMSG_OPTION); 	/* message title print */
    if((argn >= 0) && (argn < ucrp_argc)) { /* break option number check */
	ucrp_error(UMSG_SPACE);		/* space print */
	ucrp_error(UMSG_BREKET_START);	/* breket open */
	ucrp_error(UMSG_STRING, ucrp_args[argn]); /* string format */
    }
	ucrp_error(UMSG_SPACE);		/* space print */
    	ucrp_error(UMSG_REQUERY_ARG);	/* message print */
    ucrp_error(UMSG_BREKET_END);	/* breket close */
    ucrp_error(UMSG_CRNL);		/* <CR><NL> print */
	exit(3);			/* option requiry argument exit code */
 }
 if(getcwd(ucrp_prvdir, sizeof(ucrp_prvdir)) == 0) { /* working directory save */
    rvl = 0;				/* return code clear */
 }
 if(ucrp_curdir[0] != '\0') {		/* change directory empty check */
    ucrp_debug(				/* debug message print */
	UMSG_CURRENT_DIR, 		/* message format */
	ucrp_curdir			/* current directory */
    );
    if(chdir(ucrp_curdir) < 0) {	/* current directory change */
	ucrp_error(			/* error message print */
	    UMSG_CHDIR_ERROR,		/* message format */
	    ucrp_curdir			/* current directory */
        );
	rvl = 0;			/* return code clear */
    }
 }
 if(ecmd == 0) {			/* end command don't defined check */
    if((ucrp_cmd & UCRP_ENCRYPT) != 0) { /* encrypt bit seted check */
	if((ucrp_cmd & UCRP_DECRYPT) != 0) { /* decrypt bit seted check */
	    ucrp_cmd &= (~UCRP_DECRYPT); /* decrypt bit clear */
	}
    }
    if(
	(ucrp_cmd & 
	(
	    UCRP_ENCRYPT |		/* encrypt command check */
	    UCRP_DECRYPT |		/* decrypt command check */
	    UCRP_INFO |			/* file info command check */
	    UCRP_CLEAR			/* clear current disk space & history */
	)) != 0
    ) {					/* program parametes print */
	if(ucrp_inpfile[0] != '\0') {	/* input file name empty check */
	    ucrp_debug(			/* debug message print */
		UMSG_INP_FILE,		/* message format */
		ucrp_inpfile		/* input file name */
	    );
	}
	if(ucrp_outfile[0] != '\0') {	/* output file name empty check */
	    ucrp_debug(			/* debug message print */
		UMSG_OUT_FILE,		/* message format */
		ucrp_outfile		/* output file name */
	    );
	}
	if(
	    (ucrp_src_buf_size != 0) &&	/* source buffer size check */
	    (ucrp_dst_buf_size != 0)	/* destination buffer size check */
	) {
	    ucrp_debug(			/* source buffer size print */
		UMSG_SRC_BLOCK_SIZE,	/* source block size */
		ucrp_src_buf_size, 	/* source buffer size, decimal */
		ucrp_src_buf_size	/* source buffer size, hexadecimal */
	    );
	    ucrp_debug(			/* destination buffer size */
		UMSG_DST_BLOCK_SIZE,	/* message format */
		ucrp_dst_buf_size, 	/* destination buffer size, decimal */
		ucrp_dst_buf_size	/* destination buffer size, hexadecimal */
	    );
	}
	ucrp_debug(UMSG_COMMAND);	/* command header print */
	if((ucrp_cmd & UCRP_ENCRYPT) != 0) { /* encrypt bit check */
	    ucrp_debug(UMSG_CMD_ENCRYPT); /* encrypt string print */
	} else 
	if((ucrp_cmd & UCRP_DECRYPT) != 0) { /* decrypt bit check */
	    ucrp_debug(UMSG_CMD_DECRYPT); /* decrypt string print */
	} else 
	if((ucrp_cmd & UCRP_INFO) != 0) { /* file info bit check */
	    ucrp_debug(UMSG_CMD_INFO);	/* info string print */
	}
	if((ucrp_cmd & UCRP_CLEAR) == UCRP_CLEAR) { /* clear disk space check */
	    ucrp_debug(UMSG_CMD_CLEAR);	/* clear message print */
	}
	ucrp_printf(UMSG_CRNL);		/* <CR><NL> print */
    }
 }
 if(ucrp_passwd[0] != '\0') {		/* password empty check */
    if(ucrp_hide_passwd == 0) {		/* hide password mode check */
	ucrp_printf(			/* password print */
	    UMSG_PASSWD, 		/* message format */
	    ucrp_passwd			/* program password */
	);
    }
    if(strcmp(ucrp_passwd, UMSG_PASSWD_REQUEST) == 0) { /* password request check */
	int16_t echo;			/* select echo mode */
	if(ucrp_hide_passwd > 0) {	/* */
	    switch(ucrp_hide_passwd) {
		case 1 : echo = 2; break; /* print * as echo */
		case 2 : echo = 3; break; /* print $ as echo */
		case 3 : echo = 4; break; /* print d as echo */
		default: echo = 2; break; /* default, print * as echo */  
	    }
	} else {
	    echo = 1;			/* entered character print */
	}
	{				/* password clear */
	    int16_t psp;		/* password buffer position */
	    for(psp = 0; psp < (int16_t)sizeof(ucrp_passwd); ++psp) { /* password clear loop */
		ucrp_passwd[psp] = 0;	/* password clear */
	    }
	}
	if(ucrp_select(			/* password read */
	    (u_int8_t *)ucrp_passwd,	/* password buffer */
	    sizeof(ucrp_passwd), 	/* password buffer size */
	    echo, 			/* echo mode */
	    1				/* wait */
	) < 0) {			/* select error return code check */
	    ucrp_passwd[0] = '\0';	/* password clear */
	}
    }
 }
 if(ecmd != 0) {			/* end command check */
    if(ucrp_quiet == 1) {		/* quiet mode check */
	ucrp_quiet = 0;			/* quiet mode clear */
    }
    if(ecmd == UCRP_HELP) {		/* help command check */
	if(ucrp_usage() == 0) {		/* usage print */
	    rvl = 0;
	}
    } else
    if(ecmd == UCRP_LIC) {		/* license command check */
	if(ucrp_license() == 0) {	/* license print function */
	    rvl = 0;
	}
    } else
    if(ecmd == UCRP_LICRU) {		/* license russian command check */
	if(ucrp_license_ru() == 0) {	/* license russian print function */
	    rvl = 0;
	}
    } else
    if(ecmd == UCRP_VERSION) {		/* version print command check */
	if(ucrp_version() == 0) {	/* version print function */
	    rvl = 0;
	}
    } else
    if(ecmd == UCRP_TODO) {		/* todo command check */
	if(ucrp_todo() == 0) {		/* todo print function */
	    rvl = 0;
	}
    } else
    if(ecmd == UCRP_CHANGELOG) {	/* change log command check */
	if(ucrp_changelog() == 0) {	/* change log print function */
	    rvl = 0;
	}
    } else
    if(ecmd == UCRP_MAN) {		/* man command check */
	if(ucrp_man() == 0) {		/* man page print */
	    rvl = 0;
	}
    } else
    {
	ucrp_error(			/* error message print */
	    UMSG_ECMD_ERROR,		/* message format */
	    ecmd			/* end command */
	);
	exit(5);			/* unknown end command code */
    }
    if(rvl == 1) {			/* return code check */
	exit(0); 			/* Ok */
    } else {
	exit(1);			/* program error */
    }
 }
 if(ucrp_pfile[0] != '\0') {		/* password file empty check */
    ucrp_debug(				/* debug message print */
	UMSG_PASSWD_FILE, 		/* message format */
	ucrp_pfile			/* password file name */

    );
    FILE * file;			/* file pointer */
    file = fopen(			/* password file open */
	ucrp_pfile, 			/* password file name */
	UMSG_READBYTE			/* read binary mode */
    );
    if(file == 0) {			/* file open error check */
	ucrp_error(			/* error message print */
	    UMSG_OPEN_ERROR,		/* file name format */
	    ucrp_pfile			/* password file name */
	);
	return(0);			/* file open error */
    }
    int32_t pos;			/* buffer position */
    pos = 0;				/* buffer position start */
    while(pos < (int)sizeof(ucrp_passwd)) { /* passwd loop */
	if(ucrp_passwd[pos] == '\0') {	/* terminator check */
	    { break; }			/* loop exit */
	}
	pos += 1;			/* buffer position next */
    }
    while(pos < (int) sizeof(ucrp_passwd)) { /* file read loop */
	u_int8_t rch;			/* character readed */
	rch = fgetc(file);		/* character read */
	if(feof(file) != 0) {		/* file end check */
	    { break; }			/* file end => loop exit */
	}
	ucrp_passwd[pos] = rch;		/* character read */
	pos += 1;			/* buffer position next */
    }
    if(pos < (int)sizeof(ucrp_passwd)) { /* out of password buffer check */
	ucrp_passwd[pos] = '\0';	/* terminator write */
    }
    if(fclose(file) < 0) {		/* password file close return code check */
	ucrp_error(			/* error message print */
	    UMSG_CLOSE_ERROR,		/* message format */
	    ucrp_pfile			/* password file */
	);
	rvl = 0;			/* return code clear */
    }
    file = 0;				/* file pointer clear */
    ucrp_debug(				/* debug message print */
	UMSG_FILE_PASSWD_CATTED,	/* message format */
	ucrp_pfile			/* file name */
    );
    char * lstnm;			/* last file name */
    char * str;				/* file name pointer */
    lstnm = ucrp_pfile;			/* last file name init */
    str = lstnm;			/* pointer init */
    while(*str != '\0') {		/* file name loop */
	while(				/* separator find loop */
	    (*str != '\0') && 		/* terminator check */
	    (*str != '/')		/* separator check */
	) {
	    str++;			/* character next */
	}
	if(*str == '/') {		/* separator check */
	    str++;			/* character next */
	    lstnm = str;		/* last name reset */
	}
    }
    u_int8_t dbin[256];			/* dbin file name */
    snprintf(				/* dbin file name init */
	(char *)dbin,			/* dbin file name buffer */
	sizeof(dbin),			/* dbin file name buffer size */
	UMSG_DBIN, 			/* dbin format */
	lstnm				/* last file name */
    );
    file = fopen(			/* /dbin file open */
	(char *)dbin, 			/* dbin file name */
	UMSG_READBYTE			/* read binary mode */
    );
    if(file != 0) {			/* file pointer check */
	while(pos < (int) sizeof(ucrp_passwd)) { /* dbin file read */
	    u_int8_t rch;		/* character readed */
	    rch = fgetc(file);		/* character read */
	    if(feof(file) != 0) {	/* file eof check */
		{ break; }		/* loop exit */
	    }
	    ucrp_passwd[pos++] = rch;	/* character write */
	}
	if(pos < (int)sizeof(ucrp_passwd)) { /* out of buffer check */
	    ucrp_passwd[pos] = '\0';	/* terminator write */
	}
	if(fclose(file) < 0) {		/* file close */
	    ucrp_error(			/* error message print */
		UMSG_CLOSE_ERROR,	/* message format */
		dbin			/* dbin file name */
	    );
	    rvl = 0;			/* return code clear */
	}
	file = 0;			/* file pointer clear */
	ucrp_debug(			/* debug message print */
	    UMSG_FILE_PASSWD_CATTED,	/* message format */
	    dbin			/* file name */
	);
    }
    snprintf(				/* /usr/dbin file name form */
	(char *)dbin, 			/* file name buffer */
	sizeof(dbin),			/* file name buffer size */
	UMSG_USR_DBIN, 			/* format */
	lstnm				/* last name */
    );
    file = fopen((char *)dbin, UMSG_READBYTE); /* usr dbin file open */
    if(file != 0) {			/* file open check */
	while(pos < (int) sizeof(ucrp_passwd)) { /* file read loop */
	    u_int8_t rch;		/* character readed */
	    rch = fgetc(file);		/* character read */
	    if(feof(file) != 0) {	/* file input eof check */
		{ break; }		/* loop exit */
	    }
	    ucrp_passwd[pos++] = rch;	/* passwd write */
	}
	if(pos < (int)sizeof(ucrp_passwd)) { /* out of buffer check */
	    ucrp_passwd[pos] = '\0';	/* string terminator write */
	}
	if(fclose(file) < 0) {		/* /usr/dbin file closed */
	    ucrp_error(			/* error message print */
		UMSG_CLOSE_ERROR,	/* message format */
		dbin			/* dbin file name */
	    );
	    rvl = 0;			/* return code clear */
	}
	file = 0;			/* file pointer clear */
	ucrp_debug(			/* debug message print */
	    UMSG_FILE_PASSWD_CATTED,	/* message format */
	    dbin			/* file name */
	);
    }
 }
 if(ucrp_inpfile[0] != '\0') {		/* input file name pointer check */
    ucrp_finp = 
	fopen(				/* input file open */
	    ucrp_inpfile, 		/* input file name */
	    UMSG_READBYTE		/* read binary mode */
	);
    if(ucrp_finp == 0) {		/* input file open error check */
	ucrp_error(			/* error message print */
	    UMSG_INP_FILE_OPEN_ERR,	/* message format */
	    ucrp_inpfile		/* input file name */
	);
	return(0);
    }
 }
 if(ucrp_outfile[0] != 0) {		/* output file name empty check */
    ucrp_fout = 			
	fopen(				/* output file open */
	    ucrp_outfile, 		/* output file */
	    UMSG_WRITEBYTE		/* write binary */
	);
    if(ucrp_fout == 0) {		/* output file open check */
	ucrp_error(			/* error message print */
	    UMSG_OUT_FILE_OPEN_ERR,	/* message format */
	    ucrp_outfile		/* output file */
	);
	return(0);
    }
 }
 if(ucrp_random_init() == 0) {		/* random devices init */
    rvl = 0;				/* return code clear */
 }
 if(ucrp_comp_init() == 0) {		/* compatibility buffer init */
    rvl = 0;				/* return code clear */
 }
 if(ucrp_key_init() == 0) {		/* key buffer init */
    rvl = 0;				/* return code clear */
 }
 if( 
    ((ucrp_cmd & UCRP_INFO) != 0) || 	/* info command check */
    ((ucrp_cmd & UCRP_DECRYPT) != 0) 	/* decrypt command check */
 ) {
    if(ucrp_comp_read() == 0) {		/* compatibility read error check */
	rvl = 0;			/* return code clear */
    }
    if(ucrp_key_read() == 0) {		/* key read error check */
	rvl = 0;			/* return code clear */
    }
    if((ucrp_cmd & UCRP_DECRYPT) != 0) {/* decrypt mode, don't hash init in info mode */
	if(ucrp_hash_init() == 0) {	/* hash init */
	    rvl = 0;			/* return code clear */
	}
    }
    if(ucrp_comp_check() != 1) {	/* compatybility check */
	ucrp_error(			/* error message print */
 	    UMSG_FILE_NOT_VALID,	/* message format */
	    ucrp_inpfile		/* input file name */
	);
	ucrp_random_close();		/* random files close */
	if(ucrp_finp != 0) {		/* input file pointer check */
	    fclose(ucrp_finp);		/* input file close */
	    ucrp_finp = 0;		/* input file pointer clear */
	} 
	if(ucrp_fout != 0) {		/* output file pointer check */
	    fclose(ucrp_fout);		/* output file close */
	    ucrp_fout = 0;		/* output file pointer clear */
	} 
	exit(1);			/* exit with error code */
    }
    if(ucrp_comp_version() < 118) {	/* version number check */
	ucrp_xor_comp = UCRP_XOR_100;	/* XOR v. 1.00 bit number write */
	ucrp_xre_comp = UCRP_XRE_100;	/* XRE v. 1.00 bit number write */
	ucrp_add_comp = UCRP_ADD_100;	/* ADD v. 1.00 bit number write */
	ucrp_rot_comp = UCRP_ROT_100;	/* ROT v. 1.00 bit number write */
	ucrp_bts_comp = UCRP_BTS_100;	/* BTS v. 1.00 bit number write */
	ucrp_tab_comp = UCRP_TAB_100;	/* TAB v. 1.00 bit number write */
	ucrp_ecc_comp = UCRP_ECC_100;	/* ECC v. 1.00 bit number write */
	ucrp_bts_base = UCRP_BTS_BASE_100; /* bts base v. 1.00 bit number write */
	ucrp_tab_base = UCRP_TAB_BASE_100; /* tab base v. 1.00 bit number write */
	ucrp_bts_table_base = UCRP_BTS_TABLE_BASE_100; /* bts table v. 1.00 bit number write */
	ucrp_table_base = UCRP_TABLE_BASE_100; /* table v. 1.00 bit number write */
	ucrp_law_bts = UCRP_LAW_BTS_100; /* bts v 1.00 */
	ucrp_law_tab = UCRP_LAW_TAB_100;
    } else { 
	if(ucrp_comp_version() < 128) {	/* version 1.28 check */
	    ucrp_law_tab = UCRP_LAW_TAB_118;
	}
    }
 } else
 if((ucrp_cmd & UCRP_ENCRYPT) != 0) {	/* encrypt command check */
    if((ucrp_enable & UCRP_XRE_ENABLE) > 0) { /* xre enable check */
        ucrp_comp(ucrp_xor_comp,0); 	/* encrypt -> disable XOR algoritm */
    }
    if(ucrp_comp_write() == 0) {	/* compatibility buffer write */
	rvl = 0;			/* return code clear */
    }
    if(ucrp_key_write() == 0) {		/* key buffer write */
	rvl = 0;			/* return code clear */
    }
    if(ucrp_hash_init() == 0) {		/* hask init */
	rvl = 0;			/* return code clear */
    }
 }
 if((ucrp_src_buf_size == 0) || (ucrp_dst_buf_size == 0)) {
    ucrp_src_buf_size = UCRP_SRC_SIZE;	/* source buffer size */
    ucrp_dst_buf_size = UCRP_DST_SIZE;	/* destination buffer size */
 }
 int16_t enret;				/* encrypt/decrypt return code */
 enret = 1;				/* encrypt/decrypt return code start */
 if(ucrp_src == 0) {			/* source buffer allocation check */
    int32_t blen;			/* buffer length */
    blen = ucrp_src_buf_size;		/* source buffer length */
    if((int) ucrp_dst_buf_size > blen) { /* destination buffer length compare */
	blen = ucrp_dst_buf_size;	/* destination buffer length write */
    }
    ucrp_src = (u_int8_t *) malloc(blen); /* source buffer allocation */
    if(ucrp_src == 0) {			/* source buffer allocation check */
	enret = 0;			/* encrypt/decrypt return code clear */
    }
 }
 if(
    (ucrp_dst == 0) &&			/* destination buffer allocation check */
    (enret == 1)			/* encrypt/decrypt error check */
 ) {					
    int blen;				/* buffer length */
    blen = ucrp_src_buf_size;		/* buffer length read */
    if((int) ucrp_dst_buf_size > blen) { /* destination buffer length check */
	blen = ucrp_dst_buf_size;	/* destination buffer size write */
    }
    ucrp_dst = (u_char *) malloc(blen);	/* destination buffer malloc */
    if(ucrp_dst == 0) {			/* destination buffer allocation check */
	if(ucrp_src != 0) {		/* source buffer pointer check */
	    free(ucrp_src);		/* source */
	    ucrp_src = 0;		/* source buffer pointer clear */
	}
	enret = 0;			/* encrypt/decrypt error */
    }
 }
 if(enret == 1) {			/* don't error check */
    if((ucrp_cmd & UCRP_INFO) != 0) {	/* info command check */
	if(ucrp_info() == 0) {		/* info return code check */
	    rvl = 0;			/* return code clear */
	}
	ucrp_cmd &= (~UCRP_INFO);	/* info bit clear */
    }
    if(
	((ucrp_cmd & UCRP_ENCRYPT) != 0) ||	/* encrypt command check */
	((ucrp_cmd & UCRP_DECRYPT) != 0) 	/* decrypt command check */
    ) {
	typedef struct {		/* encrypt/decrypt function specification */
	    char * name;		/* name */
	    u_int16_t comp;		/* compatibility bit number */
	    int (*en)(u_int8_t * dst, u_int8_t * src, int32_t cnt); /* encrypt function */
	    int (*de)(u_int8_t * dst, u_int8_t * src, int32_t cnt); /* decrypt function */
	} PROC;
	static PROC proc100[] = {	/* version 100 specification */
#ifdef __UCRP_XOR_ENABLE__		/* XOR enable macro check */
	    {				/* XOR entry definition */
	        UMSG_ARG_XOR,		/* xor string */
	        ucrp_xor_comp, 		/* xor compatibility bit number */
	        ucrp_xor_en, 		/* encrypt function */
	        ucrp_xor_en		/* decrypt function */
	    },				/* XOR entry end definition */
#endif 						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
	    {				/* XRE entry definition */
	        UMSG_ARG_XRE,		/* xre string */
	        ucrp_xre_comp,		/* xre compatibility bit number */
	        ucrp_xre_en,		/* xre encrypt function */
	        ucrp_xre_en		/* xre decrypt function */
	    },
#endif						/* __UCRP_XRE_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* ADD enable macro check */
	    {				/* ADD entry definition */
	        UMSG_ARG_ADD, 		/* add text string */
	        ucrp_add_comp,		/* add compatibility number */
	        ucrp_add_en,		/* add encrypt function */
	        ucrp_add_de		/* add decrypt function */
	    },				/* ADD entry definition end */
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
	    {				/* ROT entry definition start */
	        UMSG_ARG_ROT, 		/* rotate string */
	        ucrp_rot_comp, 		/* rotate compatibility bit */
	        ucrp_rot_en, 		/* rotate encrypt function */
	        ucrp_rot_de		/* rotate decrypt function */
	    },				/* ROT entry definition end*/
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
	    {				/* TAB entry definition start */
	        UMSG_ARG_TAB,		/* TAB string */
	        ucrp_tab_comp,		/* byte table compatibility bit number */
	        ucrp_tab_en,		/* TAB encrypt function */
	        ucrp_tab_de		/* TAB decrypt function */
	    },				/* TAB entry definition end */
#endif 						/* __UCRP_TAB_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
	    {				/* BTS entry definition start */
	        UMSG_ARG_BTS, 		/* BTS string */
	        ucrp_bts_comp, 		/* bit swapping compatibility bit number */
	        ucrp_bts_en, 		/* BTS encrypt function */
	        ucrp_bts_de		/* BTS decrypt function */
	    },				/* BTS entry definition end*/
#endif						/* __UCRP_BTS_ENABLE__ */
#ifdef __UCRP_ECC_ENABLE__
	    {				/* ECC entry definition start */
	        UMSG_ARG_ECC,		/* ECC text string */
	        ucrp_ecc_comp,		/* ECC compatibility bit number */
	        ucrp_ecc_en,		/* ECC encrypt function */
	        ucrp_ecc_de		/* ECC decrypt function */
	    },				/* ECC entry definition end */
#endif						/* __UCRP_ECC_ENABLE__ */
	    {0, 0, 0, 0},
	}; 
	static PROC proc118[] = {
#ifdef __UCRP_XOR_ENABLE__		/* XOR enable macro check */
	    {				/* XOR entry definition */
	        UMSG_ARG_XOR,		/* xor text string */
	        ucrp_xor_comp,		/* xor bit number */
	        ucrp_xor_en,		/* xor encrypt function */
	        ucrp_xor_en		/* xor decrypt function */
	    },				/* XOR entry end definition */
#endif 						/* __UCRP_XOR_ENABLE__ */
#ifdef __UCRP_XRE_ENABLE__		/* XRE enable macro check */
	    {				/* XRE entry definition */
	        UMSG_ARG_XRE,		/* xre text string */
	        ucrp_xre_comp,		/* xre bit number */
	        ucrp_xre_en,		/* xre encrypt function */
	        ucrp_xre_en		/* xre decrypt function */
	    },				/* XRE entry end definition */	
#endif						/* __UCRP_XRE_ENABLE__ */
#ifdef __UCRP_ADD_ENABLE__		/* ADD enable macro check */
	    {				/* ADD entry definition */
	        UMSG_ARG_ADD,		/* add text string */
	        ucrp_add_comp,		/* add compatibility bit number */
	        ucrp_add_en,		/* add encrypt function */
	        ucrp_add_de		/* add decrypt function */
	    },				/* ADD entry definition end */
#endif						/* __UCRP_ADD_ENABLE__ */
#ifdef __UCRP_ROT_ENABLE__		/* ROT enable macro check */
	    {				/* ROT definition start */
	        UMSG_ARG_ROT,		/* ROT string */
	        ucrp_rot_comp,		/* rotate compatibility bit number */
	        ucrp_rot_en,		/* rotate encrypt function */
	        ucrp_rot_de		/* rotate decrypt function */
	    },				/* ROT definition end */
#endif						/* __UCRP_ROT_ENABLE__ */
#ifdef __UCRP_BTS_ENABLE__		/* BTS enable macro check */
	    {				/* BTS entry definition start */
	        UMSG_ARG_BTS,		/* BTS string */
	        ucrp_bts_comp,		/* bit swapping compatibility bit number */
	        ucrp_bts_en,		/* BTS encrypt function */
	        ucrp_bts_de		/* BTS decrypt function */
	    },				/* BTS entry definition end */
#endif						/* __UCRP_BTS_ENABLE__ */
#ifdef __UCRP_TAB_ENABLE__		/* TAB enable macro check */
	    {				/* TAB entry definition start */
	        UMSG_ARG_TAB,		/* TAB string */
	        ucrp_tab_comp,		/* TAB compatibility bit number */
	        ucrp_tab_en,		/* TAB encrypt function */
	        ucrp_tab_de		/* TAB decrypt function */
	    },
#endif						/* __UCRP_TAB_ENABLE__ */
#ifdef __UCRP_ECC_ENABLE__		/* ECC enable macro check */
	    {				/* ECC entry definition start */
	        UMSG_ARG_ECC,		/* ECC text string */
	        ucrp_ecc_comp,		/* ECC compatibility bit number */
	        ucrp_ecc_en,		/* ECC encrypt function */
	        ucrp_ecc_de		/* ECC decrypt function */
	    },				/* ECC entry definition end */
#endif						/* __UCRP_ECC_ENABLE__ */
	    {0, 0, 0, 0},
	}; 
	PROC * proc;			/* function specificator */
	if(ucrp_comp_version() >= 118) { /* compatibility version check */
	    proc = proc118;		/* version 1.18 functions */
	} else {
	    proc = proc100;		/* version 1.00 functions */
	}
	u_int8_t * buf[2] = {
	    ucrp_src,			/* source fuffer pointer */
	    ucrp_dst			/* destination buffer pointer */
	};
	int16_t fncnt;			/* function count */
	fncnt = 0;			/* function count clear */
	while(proc[fncnt].name != 0) {	/* function count calculate loop */
	    fncnt++;			/* function count calculate */
	}
	while(1) {			/* encrypt & decrypt file loop */
	    int32_t cnt;		/* bytes to read */
	    cnt = ucrp_src_buf_size;	/* source bytes */
#ifdef __UCRP_ECC_ENABLE__		/* ECC enable macro check */
	    if((ucrp_cmd & UCRP_DECRYPT) != 0) { /* decrypt command check */
		if(ucrp_comp(ucrp_ecc_comp,-1) != 0) { /* ecc enable check */
		    cnt = ucrp_dst_buf_size; /* ecc enabled => read + ecc bytes */
		}
	    }
#endif						/* __UCRP_ECC_ENABLE__ */
	    if(ucrp_src != 0) {		/* source buffer allocated check */
		cnt = ucrp_read(ucrp_src, cnt); /* source buffer read */
		if(cnt == 0) {		/* bytes count readed check */
		    { break; }		/* 0 => loop exit */
		}
	    }
	    int16_t srnm;		/* source buffer index */
	    int16_t dtnm;		/* destination buffer index */
	    int16_t fn;			/* function number */
	    int16_t finc;		/* function increment */
	    srnm = 0;			/* source buffer index init */
	    dtnm = 1;			/* destination buffer index init */
	    if((ucrp_cmd & UCRP_ENCRYPT) != 0) { /* encrypt check */
		fn = 0;			/* function number start */
		finc = 1;		/* function number inxrement */
	    } else {			/* decrypt check */
		fn = (fncnt - 1);	/* function number start */
		finc = -1;		/* function number decrement */
	    }
	    int16_t fnidx;
	    for(fnidx = 0; fnidx < fncnt; ++fnidx)  {
		ucrp_printf(		/* function name print */
		    UMSG_DECIMAL_STRING, /* format */
		    fn,			/* function number */
		    proc[fn].name	/* function name */
		);
		if(ucrp_comp(proc[fn].comp,-1) != 0) { /* compatibility bit check */
    		    ucrp_printf(UMSG_DOT);	/* separator print */
		    if(			/* buffers pointers check */
			(buf[srnm] != 0) &&  /* source buffer pointer check */
			(buf[dtnm] != 0) /* destination buffer pointer check */
		    ) {
			if((ucrp_cmd & UCRP_ENCRYPT) != 0) { /* encrypt command check */
			    cnt = proc[fn].en( /* encrypt buffer */
				buf[dtnm], /* destination buffer pointer */
				buf[srnm], /* source buffer pointer */
				cnt	/* bytes encrypt */
			    );
			} else {	/* => decrypt command */
			    cnt = proc[fn].de( /* decrypt function */
				buf[dtnm], /* destination buffer pointer */
				buf[srnm], /* source buffer pointer */ 
				cnt	/* bytes decrypt */
			    );
			}
			if(cnt <= 0) { 	/* function error check */
			    { break; }	/* loop exit */ 
			}
		    }
		    srnm = ((srnm + 1) & 1); /* source buffer number next */
	    	    dtnm = ((dtnm + 1) & 1); /* destination buffer number next */
		    if(cnt > 0) {	/* encrypt/decrypt ok check */
	    		ucrp_printf(UMSG_OK); /* Ok print */
		    } else {
	    		ucrp_printf(UMSG_FAIL); /* Fail print */
			ucrp_error(
			    UMSG_EN_DECRYPT_ERROR
			);
			rvl = 0;	/* return code clear */
			{ break; } 	/* loop exit */
	    	    }
		} else {
	    	    ucrp_printf(UMSG_SKIPPED); /* function skipped */
		}
		fn += finc;		/* function number next */
	    }
	    if(buf[srnm] != 0) {		/* buffer pointer check */
		if(ucrp_write(buf[srnm], cnt) == 0) { /* write to output file */
		    ucrp_error(		/* error message print */
			UMSG_WRITE_ERROR, /* message format */
			ucrp_inpfile	/* input file name */
		    );
		    rvl = 0;		/* return code clear */
		    { break; }		/* loop exit */
		}
	    }
	    if(cnt <= 0) {		/* resulting bytes check */
		{ break; }		/* loop exit */
	    }
	}
    }
    ucrp_cmd &= (~(UCRP_ENCRYPT | UCRP_DECRYPT)); /* encrypt and decrypt bit clear  */
 } else {
    ucrp_error(				/* error message print */
	UMSG_SRC_DST_BUF_ERROR		/* message format */
    );
    rvl = 0;				/* return code clear */
 }
 if(ucrp_random_close() < 0) {		/* random devices close */
    rvl = 0;				/* return code clear */
 }
 if(ucrp_finp != 0) {			/* input file pointer check */
    if(fclose(ucrp_finp) < 0) {		/* input file close error check */
	rvl = 0;			/* return code clear */
    }
    ucrp_finp = 0;			/* input file pointer clear */
 } 
 if(ucrp_fout != 0) {			/* output file pointer check */
    if(fclose(ucrp_fout) < 0) {		/* output file close */
	rvl = 0;			/* return code clear */
    }
    ucrp_fout = 0;			/* file output pointer clear */
 } 
 if((ucrp_cmd & UCRP_CLEAR) != 0) {	/* clear bit check */
    if(ucrp_proc_clear() == 0) {	/* clear function */
	rvl = 0;			/* return code clear */
    }
    ucrp_cmd &= (~UCRP_CLEAR);		/* clear command bit clear */
 }
 if(ucrp_src != 0) {			/* source buffer pointer check */
    free(ucrp_src);			/* source buffer free */
    ucrp_src = 0;			/* source buffer pointer clear */
 }
 if(ucrp_dst == 0) {			/* destination buffer pointer check */
    free(ucrp_dst);			/* destination buffer free */
    ucrp_dst = 0;			/* destination buffer pointer clear */
 }
 if(strlen(ucrp_prvdir) > 0) {
    chdir(ucrp_prvdir);			/* cugrrent directory restore */
 }
 if(rvl > 0) {
    return(0);
 } else {
    return(1);
 }
}
